/*
--	Copyright (c) 2007 - 2018 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_PADDING ON
GO

DECLARE @Statement					NVARCHAR(MAX);
DECLARE @ObjectSchema				NVARCHAR(300);
DECLARE @ObjectName					NVARCHAR(300);
DECLARE @ObjectType					NVARCHAR(30);
DECLARE @ObjectTypeForDelete		NVARCHAR(50);

PRINT 'Deleting [gen].[*] SYNONYMs, and [dim].[*], [dwetl].[*] objects' -- the objects include stored procedures, views etc. When tables are recreated the procs etc should be recreated as well
DECLARE DbObjectsCursor CURSOR LOCAL FOR 
SELECT s.name, obj.name, obj.[type] FROM sys.objects AS obj INNER JOIN sys.schemas AS s ON obj.schema_id = s.schema_id
WHERE s.name IN ('dim', 'dwetl', 'gen' )
ORDER BY obj.[type]
OPEN DbObjectsCursor

WHILE (1=1)
BEGIN
	
	FETCH FROM DbObjectsCursor INTO @ObjectSchema, @ObjectName, @ObjectType
	IF @@FETCH_STATUS != 0
		BREAK

	SET @ObjectName = '[' + RTRIM(@ObjectSchema)  + '].[' + RTRIM(@ObjectName)  + ']'
	SET @ObjectTypeForDelete = CASE @ObjectType WHEN 'U' THEN 'TABLE'
												WHEN 'P' THEN 'PROCEDURE'
												WHEN 'SN' THEN 'SYNONYM'
												WHEN 'FN' THEN 'FUNCTION'
												WHEN 'IF' THEN 'FUNCTION'
												WHEN 'V' THEN 'VIEW'
												ELSE NULL
												END
	IF @ObjectTypeForDelete IS NULL
		CONTINUE

	IF @ObjectSchema = 'gen' AND @ObjectType != 'SN' -- in gen schema only delete synonims
		CONTINUE

	SELECT @Statement = 'DROP ' + @ObjectTypeForDelete + ' ' + @ObjectName
	EXEC  sp_executesql @Statement
	
	PRINT '    DELETED ' + @ObjectTypeForDelete + ' ' + @ObjectName
END
CLOSE DbObjectsCursor
DEALLOCATE DbObjectsCursor 
GO

-- Create schemas first. Check for correct DB
PRINT 'Creating schemas'
DECLARE	@ErrorMessage		VARCHAR(2000)
DECLARE	@DbNameExpected		VARCHAR(20)
DECLARE	@SchemaName			VARCHAR(20)
DECLARE	@Statement			NVARCHAR(200)

SET @DbNameExpected = '%Data%Mart'

DECLARE @Schema TABLE
(
	[Schema Name]	VARCHAR(100)
)
INSERT @Schema VALUES ( 'dim' ), ('dwetl'), ( 'gen' )

PRINT 'Creating schemas'
DECLARE SchemaCreateCursor CURSOR LOCAL FOR 
SELECT [Schema Name]
FROM @Schema
OPEN SchemaCreateCursor

WHILE (1=1)
BEGIN
	
	FETCH FROM SchemaCreateCursor INTO @SchemaName
	IF @@FETCH_STATUS != 0
		BREAK

	IF UPPER(DB_NAME()) NOT LIKE '%' + @DbNameExpected + '%'
	BEGIN
		SET @ErrorMessage = 'Wrong database ' + DB_NAME() + ' for objects.'
		RAISERROR ( @ErrorMessage, 16, 0 );
		BREAK
	END

	IF NOT EXISTS ( SELECT * FROM sys.schemas WHERE name = @SchemaName ) 
	BEGIN
		SET @Statement = N'CREATE SCHEMA ' + @SchemaName
		EXEC sp_executesql @Statement
			
		PRINT 'Created schema ' + @SchemaName + ' in database ' + DB_NAME()
	END
	ELSE
		PRINT 'Did not created schema ' + @SchemaName + ' in database ' + DB_NAME() + '. It already exists'

END
CLOSE SchemaCreateCursor
DEALLOCATE SchemaCreateCursor 
GO

PRINT 'Creating tables'

CREATE TABLE [dim].[Dim Data Source]
(
	[Data Source Key]					TINYINT NOT NULL UNIQUE CLUSTERED,
	[Data Source Internal Id]			VARCHAR (50) NOT NULL,	-- Internal name - not visible to the users, so we hard code it and don't need to change it
	[Data Source Name]					VARCHAR (50) NOT NULL,
	[Data Source Description]			VARCHAR (200) NOT NULL,
	[Primary Record Name]				VARCHAR (50) NOT NULL,
	[Transaction Name]					VARCHAR (50) NOT NULL,
)
CREATE UNIQUE INDEX [Dim Data Source Indx1] ON [dim].[Dim Data Source] ( [Data Source Internal Id] ) INCLUDE ( [Data Source Key] )
IF ( SELECT OBJECT_ID( '[dim].[Dim Data Source]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED TABLE [dim].[Dim Data Source]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [dim].[Dim Data Source]'
END
GO

CREATE TABLE [dim].[Dim Team Role]
(
	[Team Role Key]						TINYINT NOT NULL UNIQUE CLUSTERED,
	[Team Role]							VARCHAR (50) NOT NULL,	-- Offense/Defense/Penalty/Timeout/
)
CREATE UNIQUE INDEX [Dim Team Role Indx1] ON [dim].[Dim Team Role] ( [Team Role] ) INCLUDE ( [Team Role Key] )
IF ( SELECT OBJECT_ID( '[dim].[Dim Team Role]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED TABLE [dim].[Dim Team Role]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [dim].[Dim Team Role]'
END
GO

CREATE TABLE [dim].[Dim Date]
(
	[Date Key]				INT NOT NULL UNIQUE CLUSTERED,		-- generated
	[Date] 					DATE NOT NULL, 
	[Date Desc] 			VARCHAR(25) NOT NULL,   -- 2001-11-03
	[Month Name]			VARCHAR(9) NOT NULL,	-- Nov
	[Year]					SMALLINT NOT NULL,		-- 2001
	[Quarter]				SMALLINT NOT NULL,		-- 20011,20012,20013,20014
	[Quarter Desc]			VARCHAR (8) NOT NULL,   -- e.g., 2001 Q1
	[Season Year]			DATE NOT NULL,		-- 2001-01-01
	[Season Description]	VARCHAR(200) NOT NULL,  -- 2001-2002
)
CREATE UNIQUE INDEX [Dim Date Indx1] ON [dim].[Dim Date] ( [Date] ) INCLUDE ( [Date Key] )
IF ( SELECT OBJECT_ID( '[dim].[Dim Date]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED TABLE [dim].[Dim Date]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [dim].[Dim Date]'
END
GO

CREATE TABLE [dim].[Dim Team]
(
	[Team Key]							SMALLINT NOT NULL UNIQUE CLUSTERED,
	[Team Original Name]				VARCHAR(250) NOT NULL,  -- e.g., Oilers -> Texans, or if Redskins gets ever renamed
	[Team Name]							VARCHAR(250) NULL,
	[Team Alias]						VARCHAR(250) NULL,
	[Team City]							VARCHAR(250) NULL,
	[Team Conference]					VARCHAR(220) NULL,
	[Sport Abbrev]						VARCHAR(220) NULL   CHECK ( [Sport Abbrev] IN ( 'NFL' ) ),
	[Hist Team Name]					VARCHAR(250) NULL,

	---- End of data
	[Source System]	        			VARCHAR (100) NOT  NULL,
	[Source System Id]  				VARCHAR (40)   NULL,
	[Row Effective Date]				DATE NOT NULL DEFAULT '20000101',
	[Row Expiration Date]				DATE NOT NULL DEFAULT '20500101',
	[Row Most Recent Indicator]			BIT NOT NULL DEFAULT 1,
	[Row Hash]							BINARY (16)   NULL,
)
CREATE UNIQUE INDEX [Dim Team Indx1] ON [dim].[Dim Team] ( [Team Original Name], [Row Effective Date] )
IF ( SELECT OBJECT_ID( '[dim].[Dim Team]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED TABLE [dim].[Dim Team]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [dim].[Dim Team]'
END
GO

-- SELECT * FROM [dim].[Dim Player]
CREATE TABLE [dim].[Dim Player]
(
	[Player Key]				INT NOT NULL  UNIQUE CLUSTERED,
	[Player Full Name]			VARCHAR(200) NOT NULL,
	[Player First Name]			VARCHAR(200) NULL,
	[Player Last Name]			VARCHAR(200) NULL,
	[Player Birthday]			DATE NULL,
	[Player Alias]				VARCHAR(200) NULL,
	
	---- End of data
	[Source System]	        		VARCHAR (100) NOT NULL,
	[Source System Id]  			VARCHAR (40)   NULL,
	[Row Effective Date]			DATE NOT NULL DEFAULT '20000101',
	[Row Expiration Date]			DATE NOT NULL DEFAULT '20500101',
	[Row Most Recent Indicator]		BIT NOT NULL DEFAULT 1,
	[Row Hash]						BINARY (16)   NULL,
)
CREATE UNIQUE INDEX [Dim Player Indx1] ON [dim].[Dim Player] ( [Player Full Name], [Player Birthday], [Row Effective Date] ) INCLUDE ( [Player Key], [Row Expiration Date] )
IF ( SELECT OBJECT_ID( '[dim].[Dim Player]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED TABLE [dim].[Dim Player]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [dim].[Dim Player]'
END
GO

-- For uniformity each dimension will have compoound key even if it is a singlr field
-- SELECT * FROM [dim].[Dim Game]
-- DELETE [dim].[Dim Game] WHERE [Game Key] != 0
CREATE TABLE [dim].[Dim Game]
(
	[Game Key]					INT NOT NULL  UNIQUE CLUSTERED,
	[Game Id]					VARCHAR(200) NOT NULL,
	[Game Teams]				VARCHAR(200) NOT NULL,	-- e.g., NYG/NYJ
	
	---- End of data
	[Source System]	        		VARCHAR (100) NOT NULL,
	[Source System Id]  			VARCHAR (40)   NULL,
	[Row Effective Date]			DATE NOT NULL DEFAULT '20000101',
	[Row Expiration Date]			DATE NOT NULL DEFAULT '20500101',
	[Row Most Recent Indicator]		BIT NOT NULL DEFAULT 1,
	[Row Hash]						BINARY (16)   NULL,
)
CREATE UNIQUE INDEX [Dim Game Indx1] ON [dim].[Dim Game] ( [Game Id], [Row Effective Date] ) INCLUDE ( [Game Key] )
IF ( SELECT OBJECT_ID( '[dim].[Dim Game]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED TABLE [dim].[Dim Game]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [dim].[Dim Game]'
END
GO

-- SELECT * FROM [dim].[Dim Play Type]
CREATE TABLE [dim].[Dim Play Type]
(
	[Play Type Key]					SMALLINT NOT NULL  UNIQUE CLUSTERED,
	[Formation]						VARCHAR(200) NOT NULL,
	[Play Type]						VARCHAR(200) NOT NULL,
	[Pass Type]						VARCHAR(200) NOT NULL,
	[Penalty Type]					VARCHAR(200) NOT NULL,
	
	---- End of data
	[Source System]	        		VARCHAR (100) NOT NULL,
	[Source System Id]  			VARCHAR (40)  NULL,
	[Row Hash]						BINARY (16)   NULL,
)
CREATE UNIQUE INDEX [Dim Play Type Indx1] ON [dim].[Dim Play Type] ( [Formation], [Play Type], [Pass Type], [Penalty Type] ) INCLUDE ( [Play Type Key] )
IF ( SELECT OBJECT_ID( '[dim].[Dim Play Type]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED TABLE [dim].[Dim Play Type]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [dim].[Dim Play Type]'
END
GO

CREATE TABLE [dim].[Dim Field Position]
(
	[Field Position Key]			SMALLINT NOT NULL UNIQUE CLUSTERED,
	[Down]							VARCHAR(20) NOT NULL,
	[Yard Line]						VARCHAR(20) NOT NULL,
	[Rush Direction]				VARCHAR(200) NOT NULL,
	[Yard Line Fixed]				VARCHAR(20) NOT NULL,
	[Yard Line Direction]			VARCHAR(200) NOT NULL,
	
	---- End of data
	[Source System]	        		VARCHAR (100) NOT NULL,
	[Source System Id]  			VARCHAR (40)   NULL,
	[Row Hash]						BINARY (16)   NULL,
)
CREATE UNIQUE INDEX [Dim Field Position Indx1] ON [dim].[Dim Field Position] ( [Down], [Yard Line], [Rush Direction], [Yard Line Fixed], [Yard Line Direction] ) INCLUDE ( [Field Position Key] )
IF ( SELECT OBJECT_ID( '[dim].[Dim Field Position]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED TABLE [dim].[Dim Field Position]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [dim].[Dim Field Position]'
END
GO


-- SELECT * FROM [dim].[Dim Play Time]
CREATE TABLE [dim].[Dim Play Time]
(
	[Play Time Key]					INT NOT NULL  UNIQUE CLUSTERED,
	[Quarter]						VARCHAR (10)	NOT NULL,
	[Minute]						VARCHAR (10)	NOT NULL,
	[Second]						VARCHAR (10)	NOT NULL,
	
	---- End of data
	[Source System]	        		VARCHAR (100) NOT NULL,
	[Source System Id]  			VARCHAR (40)   NULL,
	[Row Hash]						BINARY (16)   NULL,
)
CREATE UNIQUE INDEX [Dim Play Time Indx1] ON [dim].[Dim Play Time] ( [Quarter], [Minute], [Second] ) INCLUDE ( [Play Time Key] )
IF ( SELECT OBJECT_ID( '[dim].[Dim Play Time]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED TABLE [dim].[Dim Play Time]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [dim].[Dim Play Time]'
END
GO

-- SELECT * FROM [dim].[Dim Play Details]
CREATE TABLE [dim].[Dim Play Details]
(
	[Play Details Key]						SMALLINT NOT NULL  UNIQUE CLUSTERED,
	[Is Rush]								VARCHAR(10) NOT NULL CHECK ( [Is Rush] IN ( 'Y', 'N', '', 'Unknown' ) ),
	[Is Pass]								VARCHAR(10) NOT NULL CHECK ( [Is Pass] IN ( 'Y', 'N', '', 'Unknown' ) ),
	[Is Incomplete]							VARCHAR(10) NOT NULL CHECK ( [Is Incomplete] IN ( 'Y', 'N', '', 'Unknown' ) ),
	[Is Touchdown]							VARCHAR(10) NOT NULL CHECK ( [Is Touchdown] IN ( 'Y', 'N', '', 'Unknown' ) ),
	[Is Sack]								VARCHAR(10) NOT NULL CHECK ( [Is Sack] IN ( 'Y', 'N', '', 'Unknown' ) ),
	[Is Challenge]							VARCHAR(10) NOT NULL CHECK ( [Is Challenge] IN ( 'Y', 'N', '', 'Unknown' ) ),
	[Is Challenge Reversed]					VARCHAR(10) NOT NULL CHECK ( [Is Challenge Reversed] IN ( 'Y', 'N', '', 'Unknown' ) ),
	[Is Measurement]						VARCHAR(10) NOT NULL CHECK ( [Is Measurement] IN ( 'Y', 'N', '', 'Unknown' ) ),
	[Is Interception]						VARCHAR(10) NOT NULL CHECK ( [Is Interception] IN ( 'Y', 'N', '', 'Unknown' ) ),
	[Is Fumble]								VARCHAR(10) NOT NULL CHECK ( [Is Fumble] IN ( 'Y', 'N', '', 'Unknown' ) ),
	[Is Penalty]							VARCHAR(10) NOT NULL CHECK ( [Is Penalty] IN ( 'Y', 'N', '', 'Unknown' ) ),
	[Is Two Point Conversion]				VARCHAR(10) NOT NULL CHECK ( [Is Two Point Conversion] IN ( 'Y', 'N', '', 'Unknown' ) ),
	[Is Two Point Conversion Successful]	VARCHAR(10) NOT NULL CHECK ( [Is Two Point Conversion Successful] IN ( 'Y', 'N', '', 'Unknown' ) ),
	[Is Penalty Accepted]					VARCHAR(10) NOT NULL CHECK ( [Is Penalty Accepted] IN ( 'Y', 'N', '', 'Unknown' ) ),
	[Is No Play]							VARCHAR(10) NOT NULL CHECK ( [Is No Play] IN ( 'Y', 'N', '', 'Unknown' ) ),
	
	---- End of data
	[Source System]	        				VARCHAR (100) NOT NULL,
	[Source System Id]  					VARCHAR (40)   NULL,
	[Row Hash]								BINARY (16)   NULL,
)
CREATE UNIQUE INDEX [Dim Play Details Indx1] ON [dim].[Dim Play Details] 
																	(	[Is Rush]							,
																		[Is Pass]							,
																		[Is Incomplete]						,
																		[Is Touchdown]						,
																		[Is Sack]							,
																		[Is Challenge]						,
																		[Is Challenge Reversed]				,
																		[Is Measurement]					,
																		[Is Interception]					,	
																		[Is Fumble]							,	
																		[Is Penalty]						,
																		[Is Two Point Conversion]			,	
																		[Is Two Point Conversion Successful],
																		[Is Penalty Accepted]				,
																		[Is No Play]					) INCLUDE ( [Play Details Key] )
IF ( SELECT OBJECT_ID( '[dim].[Dim Play Details]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED TABLE [dim].[Dim Play Details]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [dim].[Dim Play Details]'
END
GO

-- ETL Objects
CREATE TABLE [dwetl].[Data Mart Properties]
(
	[Data Mart Properties Key]										TINYINT NOT NULL IDENTITY PRIMARY KEY,
	[Is using Partitioned Tables For Fact Tables]					BIT NOT NULL,
	[Is using Partitioned Views For Fact Tables]					BIT NOT NULL,
	[Is Report Unknown Keys In Facts]								BIT NOT NULL, -- when set the unknown keys will be reported. This flag can be turned of for ETL performance reason
	[Is Verify Fact Data]											BIT NOT NULL, -- when set the the fact tables will be verified at the end of the ETL process. This flag can be turned of for ETL performance reason
	[Is Adjust Dim Date On Initial Load Based On Available Data]	BIT NOT NULL, -- when set delete dates based on available data. Will be set to false when data availability does not translate directly into valid dates
	[Date Column To Consolidate Dates Changed]						VARCHAR(100) NULL, -- if not NULL, distinct values of this column will be selected instead of effective date column
)
GO
IF ( SELECT OBJECT_ID( '[dwetl].[Data Mart Properties]') ) IS NOT NULL
	PRINT 'CREATED TABLE [dwetl].[Data Mart Properties]'
ELSE
	PRINT 'FAILED CREATING TABLE [dwetl].[Data Mart Properties]'	
GO

-- Incremental load process table
CREATE TABLE [dwetl].[Facts Source Table]
(
	TrackedSourceTableKey						TINYINT IDENTITY,
	TableServer									VARCHAR (255) NULL, -- If set it is a Linked server, NULL when the same Server as dimensions
	TableDatabase								VARCHAR (255) NULL, -- NULL when the same DB as dimensions
	TableSchema									VARCHAR (255),
	TableName									VARCHAR (255) UNIQUE,
	IsTrackedForChanges							BIT NOT NULL,
	EffectiveDateColumn							VARCHAR (255),
	IsPrimaryForMinStartDate					BIT NOT NULL,
	IsPrimaryForMaxEndDate						BIT NOT NULL,
	ETLComponentsAffected						VARCHAR(50) NOT NULL DEFAULT ( 'FACT' ), -- comma delimited list of components affected by this source, e.g., FACT, ADJ
	EarliestEffectiveDateChanged				DATE NULL,
	LastEffectiveDate							DATE NULL,
	LastLoadedTableVersion						BIGINT NULL,
	EarliestEffectiveDateChangedBeingProcessed	DATE NULL,
	LastEffectiveDateBeingProcessed				DATE NULL,
	LastLoadedTableVersionBeingProcessed		BIGINT NULL,
	EtlRanOn									DATETIME NOT NULL DEFAULT GETDATE(),
)
GO
IF ( SELECT OBJECT_ID( '[dwetl].[Facts Source Table]') ) IS NOT NULL
	PRINT 'CREATED TABLE [dwetl].[Facts Source Table]'
ELSE
	PRINT 'FAILED CREATING TABLE [dwetl].[Facts Source Table]'	
GO


CREATE TABLE [dwetl].[ETLFactsLog]
(
	LogKey						INT NOT NULL IDENTITY(0,1) UNIQUE ,
	EffectiveDate				DATE,
	EtlStartedOn				DATETIME,
	EtlCompletedOn				DATETIME,
)
IF ( SELECT OBJECT_ID( '[dwetl].[ETLFactsLog]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED TABLE [dwetl].[ETLFactsLog]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [dwetl].[ETLFactsLog]'
END
GO

CREATE TABLE [dwetl].[Dimension Change Tracking]
(
	SSASServerName															VARCHAR(100)	NOT NULL 	,
	SSASDBName																VARCHAR(100)	NOT NULL 	,
	SSASDimensionID															VARCHAR(100)	NOT NULL 	,

	[Is Synchronize Dimension From Data Mart]								BIT				NOT NULL	,	-- true when the dimension is synched from another DM as opposed to Dimensional Authority. It is possible for some dimensions to be syncronized from Dimensional Authority when some are synched from a data Mart
	[ETL Components Affected]												VARCHAR(50) NOT NULL DEFAULT ( 'FACT' ), -- comma delimited list of components affected by this dimension, e.g., FACT, ADJ, or blank when none affected
	[Dimension Schema]														VARCHAR(20)		NOT NULL DEFAULT 'dim',
	[Dimension Name]														VARCHAR(100)	NOT NULL 	,
	[Dimension Key Name]													VARCHAR(100)	NOT NULL 	,
	[SSAS Source View Schema]												VARCHAR(20)		NOT NULL  DEFAULT 'ssas',
	[SSAS Source View Name]													VARCHAR(100)	NOT NULL 	,
	[SSAS Source View Key Name]												VARCHAR(100)	NOT NULL 	,
	[Is SSAS Date Dimension]												BIT				NOT NULL	,		
	[SSAS Source View Where Clause To Filter On SSAS Server And Database]	VARCHAR(3000)   NULL,		-- special case for dimension whose data is specific to SSAS Server and DB. The example of this is Roles dimension that will have to be filtered on SSAS source and DB. The where clause is used to construct SSAS instabce specific dimension views
	[Dynamic SSAS Source View Schema]										VARCHAR(20)		NOT NULL  DEFAULT 'ssasprocess',
	[Added Rows View Name Prefix]											VARCHAR(100)	NOT NULL  DEFAULT 'Added Rows '	,
		
	-- Changes
	[Dimension Changed Flag]												CHAR(1)			NOT NULL DEFAULT 'N',
	[Dimension Change Date]													DATE			NULL DEFAULT NULL,
	[Last Loaded In SSAS Key]												BIGINT 			NULL DEFAULT NULL,
)
CREATE UNIQUE INDEX [Dimension Change Tracking Indx1] ON [dwetl].[Dimension Change Tracking] ( SSASServerName, SSASDBName, SSASDimensionID )
CREATE UNIQUE INDEX [Dimension Change Tracking Indx2] ON [dwetl].[Dimension Change Tracking] ( SSASServerName, SSASDBName, [SSAS Source View Name] )

IF ( SELECT OBJECT_ID( '[dwetl].[Dimension Change Tracking]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED TABLE [dwetl].[Dimension Change Tracking]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [dwetl].[Dimension Change Tracking]'
END
GO

CREATE TABLE [dwetl].[ETLParallelLoadController]
(
	UniqueSessionId				VARCHAR(200) NOT NULL,
)
CREATE UNIQUE CLUSTERED INDEX etldwETLParallelLoadController_Indx ON [dwetl].[ETLParallelLoadController] ( UniqueSessionId )

IF ( SELECT OBJECT_ID( '[dwetl].[ETLParallelLoadController]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED TABLE [dwetl].[ETLParallelLoadController]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [dwetl].[ETLParallelLoadController]'
END
GO

CREATE TABLE [dwetl].[Data Mart Consumer Syncronization]
(
	DMConsumerServerName									VARCHAR(100) NOT NULL 	,
	DMConsumerDBName										VARCHAR(100) NOT NULL 	,
	DMConsumerName											VARCHAR(100) NOT NULL 	DEFAULT '',

	-- Processing attributes
	SQLServerConnectionString								VARCHAR(400) NULL DEFAULT NULL,
	FullLoad												BIT NULL DEFAULT NULL, 
	StartDateOverride										DATE NULL DEFAULT NULL,				-- When set will be used instead of actual data available
			
	-- Fact changes
	FromChangedEffectiveDate								DATE NULL DEFAULT NULL,
	ToChangedEffectiveDate									DATE NULL DEFAULT NULL,				-- This is the max date for any of the loaded facts. Will be used to determine the last date the Date dimension must have 
	LastProcessedEffectiveDate								DATE NULL DEFAULT NULL,
	LastProcessedInFullLoadEffectiveDate					DATE NULL DEFAULT NULL,
)
CREATE UNIQUE INDEX [Data Mart Consumer Syncronization Indx1] ON [dwetl].[Data Mart Consumer Syncronization] ( DMConsumerServerName, DMConsumerDBName, DMConsumerName )
IF ( SELECT OBJECT_ID( '[dwetl].[Data Mart Consumer Syncronization]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED TABLE [dwetl].[Data Mart Consumer Syncronization]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [dwetl].[Data Mart Consumer Syncronization]'
END

CREATE TABLE [dwetl].[ETL Component Synchronization]
(
	ComponentType					VARCHAR(10)	NOT NULL CHECK ( ComponentType IN ( 'SYNCFACT', 'FACT', 'ADJ', 'CALC' ) ),
	PrecedingComponentType			VARCHAR(10)	NULL CHECK ( PrecedingComponentType IN ( 'SYNCFACT', 'FACT', 'ADJ', 'CALC' ) ),
	MaxDaysToLoadOnIncrementalLoad	INT NULL,
	FromChangedEffectiveDate		DATE NULL DEFAULT NULL,
	ToChangedEffectiveDate			DATE NULL DEFAULT NULL,
)

IF ( SELECT OBJECT_ID( '[dwetl].[ETL Component Synchronization]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED TABLE [dwetl].[ETL Component Synchronization]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [dwetl].[ETL Component Synchronization]'
END

SET ANSI_PADDING OFF

PRINT 'Created Data Mart tables in ' + DB_NAME() + ' database'