/*
--	Copyright (c) 2007 - 2018 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_PADDING ON
GO


DECLARE @Statement					NVARCHAR(MAX);
DECLARE @ObjectSchema				NVARCHAR(300);
DECLARE @ObjectName					NVARCHAR(300);
DECLARE @ObjectType					NVARCHAR(30);
DECLARE @ObjectTypeForDelete		NVARCHAR(50);

PRINT 'Deleting [ssas].[*] and [ssasprocess].[*] objects' -- the objects include stored procedures, views, synonyms. When views are recreated the procs etc should be recreated as well
DECLARE DbObjectsCursor CURSOR LOCAL FOR 
SELECT s.name, obj.name, obj.[type] FROM sys.objects AS obj INNER JOIN sys.schemas AS s ON obj.schema_id = s.schema_id
WHERE s.name IN ('dw', 'fact', 'dwprocess', 'ssasload', 'ssasload')
		AND ( obj.[type] = 'SN'  ) 
ORDER BY obj.[type]
OPEN DbObjectsCursor

WHILE (1=1)
BEGIN
	
	FETCH FROM DbObjectsCursor INTO @ObjectSchema, @ObjectName, @ObjectType
	IF @@FETCH_STATUS != 0
		BREAK

	SET @ObjectName = '[' + RTRIM(@ObjectSchema)  + '].[' + RTRIM(@ObjectName)  + ']'
	SET @ObjectTypeForDelete = CASE @ObjectType WHEN 'V' THEN 'VIEW'
												WHEN 'SN' THEN 'SYNONYM'
												ELSE NULL
												END
	IF @ObjectTypeForDelete IS NULL
		CONTINUE

	SELECT @Statement = 'DROP ' + @ObjectTypeForDelete + ' ' + @ObjectName
	EXEC  sp_executesql @Statement
	
	PRINT '    DELETED ' + @ObjectTypeForDelete + ' ' + @ObjectName
END
CLOSE DbObjectsCursor
DEALLOCATE DbObjectsCursor 
GO

-- Create schemas first. Check for correct DB
PRINT 'Creating schemas'
DECLARE	@ErrorMessage		VARCHAR(2000)
DECLARE	@DbNameExpected		VARCHAR(20)
DECLARE	@SchemaName			VARCHAR(20)
DECLARE	@Statement			NVARCHAR(200)

SET @DbNameExpected = '%Data%Mart%'

-- Create schema
DECLARE @Schema TABLE
(
	[Schema Name]	VARCHAR(100)
)
INSERT @Schema VALUES ('dw' ), ('fact'), ('dwprocess'), ('ssasload'), ('ssasload') -- the first one is dimensions spesific to Data Mart

PRINT 'Creating schemas'
DECLARE SchemaCreateCursor CURSOR LOCAL FOR 
SELECT [Schema Name]
FROM @Schema
OPEN SchemaCreateCursor

WHILE (1=1)
BEGIN
	
	FETCH FROM SchemaCreateCursor INTO @SchemaName
	IF @@FETCH_STATUS != 0
		BREAK

	IF UPPER(DB_NAME()) NOT LIKE '%' + @DbNameExpected + '%'
	BEGIN
		SET @ErrorMessage = 'Wrong database ' + DB_NAME() + ' for objects.'
		RAISERROR ( @ErrorMessage, 16, 0 );
		BREAK
	END

	IF NOT EXISTS ( SELECT * FROM sys.schemas WHERE name = @SchemaName ) 
	BEGIN
		SET @Statement = N'CREATE SCHEMA ' + @SchemaName
		EXEC sp_executesql @Statement
			
		PRINT 'Created schema ' + @SchemaName + ' in database ' + DB_NAME()
	END
	ELSE
		PRINT 'Did not created schema ' + @SchemaName + ' in database ' + DB_NAME() + '. It already exists'

END
CLOSE SchemaCreateCursor
DEALLOCATE SchemaCreateCursor 
GO

----------------------------------------------------------------------------
-- ETL INTERFACE SYNONYMs
-- 
-- Create synonyms for stored procedures that are called from 
-- ETL Jobs to have common interface for different DWs
-- It is OK to create SYNONYMs here before the stored procedures are compiled as 
-- SQL Server does not check for the availability of the referenced object
-------------------------------------------------------------------------------
CREATE SYNONYM [dw].[ETL Data Mart - Facts]
FOR  [dw].[ETL Data Mart - Facts - NFL];
IF ( SELECT OBJECT_ID( '[dw].[ETL Data Mart - Facts]') ) IS NOT NULL
	PRINT 'CREATED SYNONYM [dw].[ETL Data Mart - Facts]'
GO

CREATE SYNONYM [dw].[ETL Data Mart - Prepare Shared Data For Effective Date]
FOR  [dw].[ETL Data Mart - Prepare Shared Data For Effective Date - NFL];
IF ( SELECT OBJECT_ID( '[dw].[ETL Data Mart - Prepare Shared Data For Effective Date]') ) IS NOT NULL
	PRINT 'CREATED SYNONYM [dw].[ETL Data Mart - Prepare Shared Data For Effective Date]'
GO

----------------------------------------------------------------------------
-- SSAS LOAD INTERFACE SYNONYMS
----------------------------------------------------------------------------
CREATE SYNONYM [ssasload].[Get SSAS BASE DB Create Script]
FOR [ssasload].[Get SSAS BASE DB Create Script - NFL]
IF ( SELECT OBJECT_ID( '[ssasload].[Get SSAS BASE DB Create Script]') ) IS NOT NULL
	PRINT 'CREATED SYNONYM [ssasload].[Get SSAS BASE DB Create Script]'
GO

CREATE SYNONYM [ssasload].[Is SSAS Object Changed Since Last Load]
FOR [ssasload].[Is SSAS Object Changed Since Last Load - NFL]
IF ( SELECT OBJECT_ID( '[ssasload].[Is SSAS Object Changed Since Last Load]') ) IS NOT NULL
	PRINT 'CREATED SYNONYM [ssasload].[Is SSAS Object Changed Since Last Load]'
GO

CREATE SYNONYM [ssasload].[Create SSAS Rule Based Roles]
FOR [ssasload].[Create SSAS Rule Based Roles - NFL]
IF ( SELECT OBJECT_ID( '[ssasload].[Create SSAS Rule Based Roles]') ) IS NOT NULL
	PRINT 'CREATED SYNONYM [ssasload].[Create SSAS Rule Based Roles]'
GO

----------------------------------------------------------------------------
-- SYNONYMS refrenced from within stored procedures
----------------------------------------------------------------------------

CREATE SYNONYM [dw].[Prepare Data Mart Session]
FOR [dw].[Prepare Data Mart Session - NFL]
IF ( SELECT OBJECT_ID( '[dw].[Prepare Data Mart Session]') ) IS NOT NULL
	PRINT 'CREATED SYNONYM [dw].[Prepare Data Mart Session]'
GO

CREATE SYNONYM [dw].[Verify Fact Data]
FOR [dw].[Verify Fact Data - NFL]
IF ( SELECT OBJECT_ID( '[dw].[Verify Fact Data]') ) IS NOT NULL
	PRINT 'CREATED SYNONYM [dw].[Verify Fact Data]'
GO

PRINT 'Created synonyms in ' + DB_NAME() + ' database'
