/*
--	Copyright (c) 2007 - 2018 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_PADDING ON
GO

DECLARE @Statement					NVARCHAR(MAX);
DECLARE @ObjectSchema				NVARCHAR(300);
DECLARE @ObjectName					NVARCHAR(300);
DECLARE @ObjectType					NVARCHAR(30);
DECLARE @ObjectTypeForDelete		NVARCHAR(50);

PRINT 'Deleting [dw].[*] and [fact].[*] objects' -- the objects include stored procedures, views etc. When tables are recreated the procs etc should be recreated as well
DECLARE DbObjectsCursor CURSOR LOCAL FOR 
SELECT s.name, obj.name, obj.[type] FROM sys.objects AS obj INNER JOIN sys.schemas AS s ON obj.schema_id = s.schema_id
WHERE s.name IN ('dw', 'fact', 'dwprocess' ) AND obj.[type] IN ( 'U', 'P', 'SN', 'FN', 'IF', 'V' )
ORDER BY CASE obj.[type]  WHEN 'SN' THEN 0 WHEN 'V' THEN 1 WHEN 'U' THEN 2 ELSE 3 END  -- drop syns, then views as they can be defined WITH BINDDING
OPEN DbObjectsCursor

WHILE (1=1)
BEGIN
	
	FETCH FROM DbObjectsCursor INTO @ObjectSchema, @ObjectName, @ObjectType
	IF @@FETCH_STATUS != 0
		BREAK

	--PRINT @ObjectName + ' ' +  @ObjectType
	SET @ObjectName = '[' + RTRIM(@ObjectSchema)  + '].[' + RTRIM(@ObjectName)  + ']'
	SET @ObjectTypeForDelete = CASE @ObjectType WHEN 'U' THEN 'TABLE'
												WHEN 'P' THEN 'PROCEDURE'
												WHEN 'SN' THEN 'SYNONYM'
												WHEN 'FN' THEN 'FUNCTION'
												WHEN 'IF' THEN 'FUNCTION'
												WHEN 'V' THEN 'VIEW'
												ELSE NULL
												END
	IF @ObjectTypeForDelete IS NULL
		CONTINUE

	SELECT @Statement = 'DROP ' + @ObjectTypeForDelete + ' ' + @ObjectName
	EXEC  sp_executesql @Statement
	
	PRINT '    DELETED ' + @ObjectTypeForDelete + ' ' + @ObjectName
END
CLOSE DbObjectsCursor
DEALLOCATE DbObjectsCursor 
GO

-- Create schemas first. Check for correct DB
PRINT 'Creating schemas'
DECLARE	@ErrorMessage		VARCHAR(2000)
DECLARE	@DbNameExpected		VARCHAR(20)
DECLARE	@SchemaName			VARCHAR(20)
DECLARE	@Statement			NVARCHAR(200)

SET @DbNameExpected = '%Data%Mart%'

DECLARE @Schema TABLE
(
	[Schema Name]	VARCHAR(100)
)
INSERT @Schema VALUES ( 'dw' ), ( 'fact' ), ('dwprocess')

PRINT 'Creating schemas'
DECLARE SchemaCreateCursor CURSOR LOCAL FOR 
SELECT [Schema Name]
FROM @Schema
OPEN SchemaCreateCursor

WHILE (1=1)
BEGIN
	
	FETCH FROM SchemaCreateCursor INTO @SchemaName
	IF @@FETCH_STATUS != 0
		BREAK

	IF UPPER(DB_NAME()) NOT LIKE '%' + @DbNameExpected + '%'
	BEGIN
		SET @ErrorMessage = 'Wrong database ' + DB_NAME() + ' for objects.'
		RAISERROR ( @ErrorMessage, 16, 0 );
		BREAK
	END

	IF NOT EXISTS ( SELECT * FROM sys.schemas WHERE name = @SchemaName ) 
	BEGIN
		SET @Statement = N'CREATE SCHEMA ' + @SchemaName
		EXEC sp_executesql @Statement
			
		PRINT 'Created schema ' + @SchemaName + ' in database ' + DB_NAME()
	END
	ELSE
		PRINT 'Did not created schema ' + @SchemaName + ' in database ' + DB_NAME() + '. It already exists'

END
CLOSE SchemaCreateCursor
DEALLOCATE SchemaCreateCursor 
GO

PRINT 'Creating tables'

-- Fact tables
PRINT '--- Fact Tables ---'
GO

-- Some records are true duplicates - will remove them


CREATE VIEW [fact].[Fact Plays]
AS
SELECT
	CAST( 0 AS INT			)	AS [Fact Plays Key]		,
	CAST( 0 AS INT			)	AS [Play Date Key]		,				
	CAST( 0 AS TINYINT		)	AS [Data Source Key]	,					
	CAST( 0 AS INT			)	AS [Game Key]			,				
	CAST( 0 AS SMALLINT		)	AS [Offense Team Key]	, 
	CAST( 0 AS SMALLINT		)	AS [Defense Team Key]	, 
	CAST( 0 AS SMALLINT		)	AS [Penalty Team Key]	, 
	CAST( 0 AS SMALLINT		)	AS [Timeout Team Key]	, 
	CAST( 0 AS SMALLINT		)	AS [Play Type Key]		,				
	CAST( 0 AS SMALLINT		)	AS [Field Position Key]	,				
	CAST( 0 AS INT			)	AS [Play Time Key]		,				
	CAST( 0 AS SMALLINT		)	AS [Play Details Key]	,				
--	CAST( 0 AS INT			)	AS [Player Key]			, -- Player will be Many-to-Many, as a play can involve several players and a player is part of many plays
	
	CAST( NULL AS FLOAT )		AS [Score]				,
	CAST( NULL AS FLOAT )		AS [Yards]				,	
	CAST( NULL AS FLOAT )		AS [ToGo]				,
	CAST( NULL AS FLOAT )		AS [Penalty Yards]			
GO

IF ( SELECT OBJECT_ID( '[fact].[Fact Plays]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED PLACEHOLDER VIEW  [fact].[Fact Plays]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING PLACEHOLDER VIEW [fact].[Fact Plays]'
END
GO

-- There could timeout called by different teams on the same second - will create separate team record for Timeout team.
-- so there can be two records for the same team pointing to the same play record - one for offense/defense, the other for timeout
-- Team is Many-to-many for offense team, defense team, penalty team, Timeout team (timeout can be called by different teams on the same second)
CREATE VIEW [fact].[Bridge Teams-Plays]
AS
SELECT
	CAST( 0 AS INT			)	AS [Bridge Teams-Plays Key]		,
	CAST( 0 AS TINYINT		)	AS [Data Source Key]	,					
	CAST( 0 AS SMALLINT		)	AS [Team Key]			, 
	CAST( 0 AS TINYINT		)	AS [Team Role Key]		,					
	CAST( 0 AS INT			)	AS [Game Key]			,				
	CAST( 0 AS SMALLINT		)	AS [Play Type Key]		,				
	CAST( 0 AS SMALLINT		)	AS [Field Position Key]	,
	CAST( 0 AS INT			)	AS [Play Time Key]						
--	CAST( 0 AS SMALLINT		)	AS [Play Details Key]		-- do not need this key bas the other four are unique for each play			
GO

IF ( SELECT OBJECT_ID( '[fact].[Bridge Teams-Plays]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED PLACEHOLDER VIEW  [fact].[Bridge Teams-Plays]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING PLACEHOLDER VIEW [fact].[Bridge Teams-Plays]'
END
GO

---- etl objects
-- Tables
PRINT '--- Etl Tables ---'
GO

CREATE TABLE [dw].[Plays Source Data Template]
(
	-- Logical Keys and intermediate measures
	[Data Source Internal Id]				VARCHAR (50)	NOT NULL,	-- [dim].[Dim Data Source]
	[Play Date]								DATE			NULL,						-- for [dim].[Dim Single Column Junk] 
	[Game Id]								VARCHAR(200)	NULL,
	[Offense Team Abbrev]					VARCHAR(200)	NULL,		
	[Defense Team Abbrev]					VARCHAR(200)	NULL,		
	[Penalty Team Abbrev]					VARCHAR(200)	NULL,		
	[Timeout Team Abbrev]					VARCHAR(200)	NULL,		
	[Formation]								VARCHAR(200)	NULL,
	[Play Type]								VARCHAR(200)	NULL,
	[Pass Type]								VARCHAR(200)	NULL,
	[Penalty Type]							VARCHAR(200)	NULL,
	[Down]									VARCHAR(20)		NULL,
	[Yard Line]								VARCHAR(20)		NULL,
	[Rush Direction]						VARCHAR(200)	NULL,
	[Yard Line Fixed]						VARCHAR(20)		NULL,
	[Yard Line Direction]					VARCHAR(200)	NULL,
	[Quarter]								VARCHAR (10)	NULL,
	[Minute]								VARCHAR (10)	NULL,
	[Second]								VARCHAR (10)	NULL,
	[Is Rush]								VARCHAR(10) 	NULL,
	[Is Pass]								VARCHAR(10) 	NULL,
	[Is Incomplete]							VARCHAR(10) 	NULL,
	[Is Touchdown]							VARCHAR(10) 	NULL,
	[Is Sack]								VARCHAR(10) 	NULL,
	[Is Challenge]							VARCHAR(10) 	NULL,
	[Is Challenge Reversed]					VARCHAR(10) 	NULL,
	[Is Measurement]						VARCHAR(10) 	NULL,
	[Is Interception]						VARCHAR(10) 	NULL,
	[Is Fumble]								VARCHAR(10) 	NULL,
	[Is Penalty]							VARCHAR(10) 	NULL,
	[Is Two Point Conversion]				VARCHAR(10) 	NULL,
	[Is Two Point Conversion Successful]	VARCHAR(10) 	NULL,
	[Is Penalty Accepted]					VARCHAR(10) 	NULL,
	[Is No Play]							VARCHAR(10) 	NULL,
	
	-- Keys
	[Play Date Key]							INT				NULL ,				
	[Data Source Key]						TINYINT			NULL ,					
	[Game Key]								INT				NULL ,				
	[Offense Team Key]						SMALLINT		NULL ,	
	[Defense Team Key]						SMALLINT		NULL ,
	[Penalty Team Key]						SMALLINT		NULL ,	
	[Timeout Team Key]						SMALLINT		NULL ,
	[Play Type Key]							SMALLINT		NULL ,				
	[Field Position Key]					SMALLINT		NULL ,				
	[Play Time Key]							INT				NULL ,				
	[Play Details Key]						SMALLINT		NULL ,				

	[Score]									FLOAT			NULL,
	[Yards]									FLOAT			NULL,
	[ToGo]									FLOAT			NULL,
	[Penalty Yards]							FLOAT			NULL
)
IF ( SELECT OBJECT_ID( '[dw].[Plays Source Data Template]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED TABLE [dw].[Plays Source Data Template]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [dw].[Plays Source Data Template]'
END
GO

CREATE TABLE [dw].[Teams-Plays Source Data Template]
(
	-- Logical Keys and intermediate measures
	[Data Source Internal Id]				VARCHAR (50)	NOT NULL,	-- [dim].[Dim Data Source]
	[Play Date]								DATE			NULL,
	[Team Abbrev]							VARCHAR(200)	NULL,		
	[Team Role]								VARCHAR(200)	NULL,		-- Offense/Defense/Penalty/Timeout/
	[Game Id]								VARCHAR(200)	NULL,
	[Formation]								VARCHAR(200)	NULL,
	[Play Type]								VARCHAR(200)	NULL,
	[Pass Type]								VARCHAR(200)	NULL,
	[Penalty Type]							VARCHAR(200)	NULL,
	[Down]									VARCHAR(20)		NULL,
	[Yard Line]								VARCHAR(20)		NULL,
	[Rush Direction]						VARCHAR(200)	NULL,
	[Yard Line Fixed]						VARCHAR(20)		NULL,
	[Yard Line Direction]					VARCHAR(200)	NULL,
	[Quarter]								VARCHAR (10)	NULL,
	[Minute]								VARCHAR (10)	NULL,
	[Second]								VARCHAR (10)	NULL,
	
	-- Keys
	[Data Source Key]						TINYINT			NULL ,					
	[Team Key]								SMALLINT		NULL ,				
	[Team Role Key]							TINYINT			NULL ,				
	[Game Key]								INT				NULL ,				
	[Play Type Key]							SMALLINT		NULL ,				
	[Field Position Key]					SMALLINT		NULL ,				
	[Play Time Key]							INT				NULL ,				
)
IF ( SELECT OBJECT_ID( '[dw].[Teams-Plays Source Data Template]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED TABLE [dw].[Teams-Plays Source Data Template]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [dw].[Teams-Plays Source Data Template]'
END
GO


-- Can have configuration table spesific to Data Mart
--CREATE TABLE [dw].[Configuration - DM NFL]
--(
--...
--)

SET ANSI_PADDING OFF

PRINT 'Created tables in ' + DB_NAME() + ' database'
