/*
--	Copyright (c) 2007 - 2018 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_PADDING ON
GO

DECLARE @Statement					NVARCHAR(MAX);
DECLARE @ObjectSchema				NVARCHAR(300);
DECLARE @ObjectName					NVARCHAR(300);
DECLARE @ObjectType					NVARCHAR(30);
DECLARE @ObjectTypeForDelete		NVARCHAR(50);

PRINT 'Deleting [gen].[*] SYNONYMs, and [dimauth].[*], [etl].[*] objects' -- the objects include stored procedures, views etc. When tables are recreated the procs etc should be recreated as well
DECLARE DbObjectsCursor CURSOR LOCAL FOR 
SELECT s.name, obj.name, obj.[type] FROM sys.objects AS obj INNER JOIN sys.schemas AS s ON obj.schema_id = s.schema_id
WHERE s.name IN ('dimauth', 'etl', 'gen' )
ORDER BY obj.[type]
OPEN DbObjectsCursor

WHILE (1=1)
BEGIN
	
	FETCH FROM DbObjectsCursor INTO @ObjectSchema, @ObjectName, @ObjectType
	IF @@FETCH_STATUS != 0
		BREAK

	SET @ObjectName = '[' + RTRIM(@ObjectSchema)  + '].[' + RTRIM(@ObjectName)  + ']'
	SET @ObjectTypeForDelete = CASE @ObjectType WHEN 'U' THEN 'TABLE'
												WHEN 'P' THEN 'PROCEDURE'
												WHEN 'SN' THEN 'SYNONYM'
												ELSE NULL
												END
	IF @ObjectTypeForDelete IS NULL
		CONTINUE

	IF @ObjectSchema = 'gen' AND @ObjectType != 'SN' -- in gen schema only delete synonims
		CONTINUE

	SELECT @Statement = 'DROP ' + @ObjectTypeForDelete + ' ' + @ObjectName
	EXEC  sp_executesql @Statement
	
	PRINT '    DELETED ' + @ObjectTypeForDelete + ' ' + @ObjectName
END
CLOSE DbObjectsCursor
DEALLOCATE DbObjectsCursor 
GO

-- Create schemas first. Check for correct DB
PRINT 'Creating schemas'
DECLARE	@ErrorMessage		VARCHAR(2000)
DECLARE	@DbNameExpected		VARCHAR(20)
DECLARE	@SchemaName			VARCHAR(20)
DECLARE	@Statement			NVARCHAR(200)

SET @DbNameExpected = '%NFL%AUTH%'

DECLARE @Schema TABLE
(
	[Schema Name]	VARCHAR(100)
)
INSERT @Schema VALUES ( 'dimauth' ), ('etl'), ( 'gen' )

PRINT 'Creating schemas'
DECLARE SchemaCreateCursor CURSOR LOCAL FOR 
SELECT [Schema Name]
FROM @Schema
OPEN SchemaCreateCursor

WHILE (1=1)
BEGIN
	
	FETCH FROM SchemaCreateCursor INTO @SchemaName
	IF @@FETCH_STATUS != 0
		BREAK

	IF UPPER(DB_NAME()) NOT LIKE '%' + @DbNameExpected + '%'
	BEGIN
		SET @ErrorMessage = 'Wrong database ' + DB_NAME() + ' for objects.'
		RAISERROR ( @ErrorMessage, 16, 0 );
		BREAK
	END

	IF NOT EXISTS ( SELECT * FROM sys.schemas WHERE name = @SchemaName ) 
	BEGIN
		SET @Statement = N'CREATE SCHEMA ' + @SchemaName
		EXEC sp_executesql @Statement
			
		PRINT 'Created schema ' + @SchemaName + ' in database ' + DB_NAME()
	END
	ELSE
		PRINT 'Did not created schema ' + @SchemaName + ' in database ' + DB_NAME() + '. It already exists'

END
CLOSE SchemaCreateCursor
DEALLOCATE SchemaCreateCursor 
GO

PRINT 'Creating tables'

CREATE TABLE [dimauth].[Dim Date]
(
	[Date Key]				INT NOT NULL UNIQUE,		-- generated
	[Date] 					DATE NOT NULL, 
	[Date Desc] 			VARCHAR(25) NOT NULL,   -- 2001-11-03
	[Month Name]			VARCHAR(9) NOT NULL,	-- Nov
	[Year]					SMALLINT NOT NULL,		-- 2001
	[Quarter]				SMALLINT NOT NULL,		-- 20011,20012,20013,20014
	[Quarter Desc]			VARCHAR (8) NOT NULL,   -- e.g., 2001 Q1
	[Season Year]			DATE NOT NULL,		-- 2001-01-01
	[Season Description]	VARCHAR(200) NOT NULL,  -- 2001-2002
	
	---- End of data
	[Updated On]			DATETIME DEFAULT GETDATE(),
	[Updated By]			VARCHAR(255) DEFAULT SYSTEM_USER 
)
CREATE UNIQUE CLUSTERED INDEX [Dim Date Indx1] ON [dimauth].[Dim Date] ( [Date] )
IF ( SELECT OBJECT_ID( '[dimauth].[Dim Date]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED TABLE [dimauth].[Dim Date]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [dimauth].[Dim Date]'
END
GO

CREATE TABLE [dimauth].[Dim Data Source]
(
	[Data Source Key]					TINYINT IDENTITY(0,1) NOT NULL UNIQUE,
	[Data Source Internal Id]			VARCHAR (50) NOT NULL,	-- Internal name - not visible to the users, so we hard code it and don't need to change it
	[Data Source Name]					VARCHAR (50) NOT NULL,
	[Data Source Description]			VARCHAR (200) NOT NULL,
	[Primary Record Name]				VARCHAR (50) NOT NULL,
	[Transaction Name]					VARCHAR (50) NOT NULL,

	---- End of data
	[Updated On]						DATETIME NOT NULL DEFAULT GETDATE(),
	[Updated By]						VARCHAR(255) NOT NULL DEFAULT SYSTEM_USER, 
)
CREATE UNIQUE CLUSTERED INDEX [Dim Data Source Indx1] ON [dimauth].[Dim Data Source] ( [Data Source Internal Id] )
IF ( SELECT OBJECT_ID( '[dimauth].[Dim Data Source]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED TABLE [dimauth].[Dim Data Source]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [dimauth].[Dim Data Source]'
END
GO

CREATE TABLE [dimauth].[Dim Team Role]
(
	[Team Role Key]						TINYINT IDENTITY(0,1) NOT NULL UNIQUE,
	[Team Role]							VARCHAR (50) NOT NULL,	-- Offense/Defense/Penalty/Timeout/

	---- End of data
	[Updated On]						DATETIME NOT NULL DEFAULT GETDATE(),
	[Updated By]						VARCHAR(255) NOT NULL DEFAULT SYSTEM_USER, 
)
CREATE UNIQUE CLUSTERED INDEX [Dim Team Role Indx1] ON [dimauth].[Dim Team Role] ( [Team Role] )
IF ( SELECT OBJECT_ID( '[dimauth].[Dim Team Role]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED TABLE [dimauth].[Dim Team Role]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [dimauth].[Dim Team Role]'
END
GO

CREATE TABLE [dimauth].[Dim Team]
(
	[Team Key]							SMALLINT IDENTITY(0,1) NOT NULL UNIQUE,
	[Team Original Name]				VARCHAR(250) NOT NULL,  -- e.g., Oilers -> Texans, or if Redskins gets ever renamed
	[Team Name]							VARCHAR(250) NULL,
	[Team Alias]						VARCHAR(250) NULL,
	[Team City]							VARCHAR(250) NULL,
	[Team Conference]					VARCHAR(220) NULL,
	[Sport Abbrev]						VARCHAR(220) NULL   CHECK ( [Sport Abbrev] IN ( 'NFL' ) ),
	[Hist Team Name]					VARCHAR(250) NULL,

	---- End of data
	[Source System]	        			VARCHAR (100) NOT  NULL,
	[Source System Id]  				VARCHAR (40)   NULL,
	[Load Effective Date]				DATE NOT NULL ,
	[Row Effective Date]				DATE NOT NULL DEFAULT '20000101',
	[Row Expiration Date]				DATE NOT NULL DEFAULT '20500101',
	[Row Most Recent Indicator]			BIT NOT NULL DEFAULT 1,
	[Row Hash]							BINARY (16)   NULL,
	[Type 1 Column Hash]				BINARY (16)   NULL,
	[Type 2 Column Hash]				BINARY (16)   NULL,
	[Updated On]						DATETIME NOT NULL DEFAULT GETDATE(),
	[Updated By]						VARCHAR(255) NOT NULL DEFAULT SYSTEM_USER, 
	[Team Compound Key]					VARCHAR(250) NOT NULL,
)
CREATE UNIQUE CLUSTERED INDEX [Dim Team Indx1] ON [dimauth].[Dim Team] ( [Team Original Name], [Row Effective Date] )
CREATE INDEX [Dim Team Indx2] ON [dimauth].[Dim Team] ( [Team Compound Key] ) -- it is non-unique because of type 2 changes
IF ( SELECT OBJECT_ID( '[dimauth].[Dim Team]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED TABLE [dimauth].[Dim Team]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [dimauth].[Dim Team]'
END
GO

-- SELECT * FROM [dimauth].[Dim Player]
CREATE TABLE [dimauth].[Dim Player]
(
	[Player Key]				INT IDENTITY(0,1) NOT NULL  UNIQUE,
	[Player Full Name]			VARCHAR(200) NOT NULL,
	[Player First Name]			VARCHAR(200) NULL,
	[Player Last Name]			VARCHAR(200) NULL,
	[Player Birthday]			DATE NULL,
	[Player Alias]				VARCHAR(200) NULL,
	
	---- End of data
	[Source System]	        		VARCHAR (100) NOT NULL,
	[Source System Id]  			VARCHAR (40)   NULL,
	[Load Effective Date]			DATE NOT NULL ,
	[Row Effective Date]			DATE NOT NULL DEFAULT '20000101',
	[Row Expiration Date]			DATE NOT NULL DEFAULT '20500101',
	[Row Most Recent Indicator]		BIT NOT NULL DEFAULT 1,
	[Row Hash]						BINARY (16)   NULL,
	[Type 1 Column Hash]			BINARY (16)   NULL,
	[Type 2 Column Hash]			BINARY (16)   NULL,
	[Updated On]					DATETIME NOT NULL DEFAULT GETDATE(),
	[Updated By]					VARCHAR(255) NOT NULL DEFAULT SYSTEM_USER, 
	[Player Compound Key]			VARCHAR(250) NOT NULL,
)
CREATE UNIQUE CLUSTERED INDEX [Dim Player Indx1] ON [dimauth].[Dim Player] ( [Player Full Name], [Player Birthday], [Row Effective Date] )
CREATE INDEX [Dim Player Indx2] ON [dimauth].[Dim Player] ( [Player Compound Key] ) -- it is non-unique because of type 2 changes
IF ( SELECT OBJECT_ID( '[dimauth].[Dim Player]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED TABLE [dimauth].[Dim Player]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [dimauth].[Dim Player]'
END
GO

-- For uniformity each dimension will have compoound key even if it is a singlr field
-- SELECT * FROM [dimauth].[Dim Game]
CREATE TABLE [dimauth].[Dim Game]
(
	[Game Key]					INT IDENTITY(0,1) NOT NULL  UNIQUE,
	[Game Id]					VARCHAR(200) NOT NULL,
	[Game Teams]				VARCHAR(200) NOT NULL,	-- e.g., NYG/NYJ
	
	---- End of data
	[Source System]	        		VARCHAR (100) NOT NULL,
	[Source System Id]  			VARCHAR (40)   NULL,
	[Load Effective Date]			DATE NOT NULL ,
	[Row Effective Date]			DATE NOT NULL DEFAULT '20000101',
	[Row Expiration Date]			DATE NOT NULL DEFAULT '20500101',
	[Row Most Recent Indicator]		BIT NOT NULL DEFAULT 1,
	[Row Hash]						BINARY (16)   NULL,
	[Type 1 Column Hash]			BINARY (16)   NULL,
	[Type 2 Column Hash]			BINARY (16)   NULL,
	[Updated On]					DATETIME NOT NULL DEFAULT GETDATE(),
	[Updated By]					VARCHAR(255) NOT NULL DEFAULT SYSTEM_USER, 
	[Game Compound Key]				VARCHAR(250) NOT NULL,
)
CREATE UNIQUE CLUSTERED INDEX [Dim Game Indx1] ON [dimauth].[Dim Game] ( [Game Id], [Row Effective Date] )
CREATE INDEX [Dim Game Indx2] ON [dimauth].[Dim Game] ( [Game Compound Key] ) -- it is non-unique because of type 2 changes
IF ( SELECT OBJECT_ID( '[dimauth].[Dim Game]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED TABLE [dimauth].[Dim Game]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [dimauth].[Dim Game]'
END
GO

-- SELECT * FROM [dimauth].[Dim Field Position]
CREATE TABLE [dimauth].[Dim Field Position]
(
	[Field Position Key]			SMALLINT IDENTITY(0,1) NOT NULL  UNIQUE,
	[Down]							VARCHAR(20) NOT NULL,
	[Yard Line]						VARCHAR(20) NOT NULL,
	[Rush Direction]				VARCHAR(200) NOT NULL,
	[Yard Line Fixed]				VARCHAR(20) NOT NULL,
	[Yard Line Direction]			VARCHAR(200) NOT NULL,
	
	---- End of data
	[Source System]	        		VARCHAR (100) NOT NULL,
	[Source System Id]  			VARCHAR (40)   NULL,
	[Load Effective Date]			DATE NOT NULL ,
	[Row Hash]						BINARY (16)   NULL,
	[Updated On]					DATETIME NOT NULL DEFAULT GETDATE(),
	[Updated By]					VARCHAR(255) NOT NULL DEFAULT SYSTEM_USER, 
	[Field Position Compound Key]	VARCHAR(500) NOT NULL,
)
CREATE UNIQUE CLUSTERED INDEX [Dim Field Position Indx1] ON [dimauth].[Dim Field Position] ( [Down], [Yard Line], [Rush Direction], [Yard Line Fixed], [Yard Line Direction] )
CREATE UNIQUE INDEX [Dim Field Position Indx2] ON [dimauth].[Dim Field Position] ( [Field Position Compound Key] )
IF ( SELECT OBJECT_ID( '[dimauth].[Dim Field Position]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED TABLE [dimauth].[Dim Field Position]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [dimauth].[Dim Field Position]'
END
GO


-- SELECT * FROM [dimauth].[Dim Play Type]
CREATE TABLE [dimauth].[Dim Play Type]
(
	[Play Type Key]					SMALLINT IDENTITY(0,1) NOT NULL  UNIQUE,
	[Formation]						VARCHAR(200) NOT NULL,
	[Play Type]						VARCHAR(200) NOT NULL,
	[Pass Type]						VARCHAR(200) NOT NULL,
	[Penalty Type]					VARCHAR(200) NOT NULL,
	
	---- End of data
	[Source System]	        		VARCHAR (100) NOT NULL,
	[Source System Id]  			VARCHAR (40)   NULL,
	[Load Effective Date]			DATE NOT NULL ,
	[Row Hash]						BINARY (16)   NULL,
	[Updated On]					DATETIME NOT NULL DEFAULT GETDATE(),
	[Updated By]					VARCHAR(255) NOT NULL DEFAULT SYSTEM_USER, 
	[Play Type Compound Key]		VARCHAR(500) NOT NULL,
)
CREATE UNIQUE CLUSTERED INDEX [Dim Play Type Indx1] ON [dimauth].[Dim Play Type] ( [Formation], [Play Type], [Pass Type], [Penalty Type] )
CREATE UNIQUE INDEX [Dim Play Type Indx2] ON [dimauth].[Dim Play Type] ( [Play Type Compound Key] )
IF ( SELECT OBJECT_ID( '[dimauth].[Dim Play Type]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED TABLE [dimauth].[Dim Play Type]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [dimauth].[Dim Play Type]'
END
GO

-- SELECT * FROM [dimauth].[Dim Play Time]
CREATE TABLE [dimauth].[Dim Play Time]
(
	[Play Time Key]					INT IDENTITY(0,1) NOT NULL  UNIQUE,
	[Quarter]						VARCHAR (10)	NOT NULL,
	[Minute]						VARCHAR (10)	NOT NULL,
	[Second]						VARCHAR (10)	NOT NULL,
	
	---- End of data
	[Source System]	        		VARCHAR (100) NOT NULL,
	[Source System Id]  			VARCHAR (40)   NULL,
	[Load Effective Date]			DATE NOT NULL ,
	[Row Hash]						BINARY (16)   NULL,
	[Updated On]					DATETIME NOT NULL DEFAULT GETDATE(),
	[Updated By]					VARCHAR(255) NOT NULL DEFAULT SYSTEM_USER, 
	[Play Time Compound Key]		VARCHAR(500) NOT NULL,
)
CREATE UNIQUE CLUSTERED INDEX [Dim Play Time Indx1] ON [dimauth].[Dim Play Time] ( [Quarter], [Minute], [Second] )
CREATE UNIQUE INDEX [Dim Play Time Indx2] ON [dimauth].[Dim Play TIme] ( [Play Time Compound Key] )
IF ( SELECT OBJECT_ID( '[dimauth].[Dim Play Time]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED TABLE [dimauth].[Dim Play Time]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [dimauth].[Dim Play Time]'
END
GO

-- SELECT * FROM [dimauth].[Dim Play Details]
CREATE TABLE [dimauth].[Dim Play Details]
(
	[Play Details Key]						SMALLINT IDENTITY(0,1) NOT NULL  UNIQUE,
	[Is Rush]								VARCHAR(10) NOT NULL CHECK ( [Is Rush] IN ( 'Y', 'N', '', 'Unknown' ) ),
	[Is Pass]								VARCHAR(10) NOT NULL CHECK ( [Is Pass] IN ( 'Y', 'N', '', 'Unknown' ) ),
	[Is Incomplete]							VARCHAR(10) NOT NULL CHECK ( [Is Incomplete] IN ( 'Y', 'N', '', 'Unknown' ) ),
	[Is Touchdown]							VARCHAR(10) NOT NULL CHECK ( [Is Touchdown] IN ( 'Y', 'N', '', 'Unknown' ) ),
	[Is Sack]								VARCHAR(10) NOT NULL CHECK ( [Is Sack] IN ( 'Y', 'N', '', 'Unknown' ) ),
	[Is Challenge]							VARCHAR(10) NOT NULL CHECK ( [Is Challenge] IN ( 'Y', 'N', '', 'Unknown' ) ),
	[Is Challenge Reversed]					VARCHAR(10) NOT NULL CHECK ( [Is Challenge Reversed] IN ( 'Y', 'N', '', 'Unknown' ) ),
	[Is Measurement]						VARCHAR(10) NOT NULL CHECK ( [Is Measurement] IN ( 'Y', 'N', '', 'Unknown' ) ),
	[Is Interception]						VARCHAR(10) NOT NULL CHECK ( [Is Interception] IN ( 'Y', 'N', '', 'Unknown' ) ),
	[Is Fumble]								VARCHAR(10) NOT NULL CHECK ( [Is Fumble] IN ( 'Y', 'N', '', 'Unknown' ) ),
	[Is Penalty]							VARCHAR(10) NOT NULL CHECK ( [Is Penalty] IN ( 'Y', 'N', '', 'Unknown' ) ),
	[Is Two Point Conversion]				VARCHAR(10) NOT NULL CHECK ( [Is Two Point Conversion] IN ( 'Y', 'N', '', 'Unknown' ) ),
	[Is Two Point Conversion Successful]	VARCHAR(10) NOT NULL CHECK ( [Is Two Point Conversion Successful] IN ( 'Y', 'N', '', 'Unknown' ) ),
	[Is Penalty Accepted]					VARCHAR(10) NOT NULL CHECK ( [Is Penalty Accepted] IN ( 'Y', 'N', '', 'Unknown' ) ),
	[Is No Play]							VARCHAR(10) NOT NULL CHECK ( [Is No Play] IN ( 'Y', 'N', '', 'Unknown' ) ),
	
	---- End of data
	[Source System]	        				VARCHAR (100) NOT NULL,
	[Source System Id]  					VARCHAR (40)   NULL,
	[Load Effective Date]					DATE NOT NULL ,
	[Row Hash]								BINARY (16)   NULL,
	[Updated On]							DATETIME NOT NULL DEFAULT GETDATE(),
	[Updated By]							VARCHAR(255) NOT NULL DEFAULT SYSTEM_USER, 
	[Play Details Compound Key]		VARCHAR(500) NOT NULL,
)
CREATE UNIQUE CLUSTERED INDEX [Dim Play Details Indx1] ON [dimauth].[Dim Play Details] 
																	(	[Is Rush]							,
																		[Is Pass]							,
																		[Is Incomplete]						,
																		[Is Touchdown]						,
																		[Is Sack]							,
																		[Is Challenge]						,
																		[Is Challenge Reversed]				,
																		[Is Measurement]					,
																		[Is Interception]					,	
																		[Is Fumble]							,	
																		[Is Penalty]						,
																		[Is Two Point Conversion]			,	
																		[Is Two Point Conversion Successful],
																		[Is Penalty Accepted]				,
																		[Is No Play]								)
CREATE UNIQUE INDEX [Dim Play Details Indx2] ON [dimauth].[Dim Play Details] ( [Play Details Compound Key] )
IF ( SELECT OBJECT_ID( '[dimauth].[Dim Play Details]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED TABLE [dimauth].[Dim Play Details]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [dimauth].[Dim Play Details]'
END
GO

CREATE TABLE [etl].[ETLParallelLoadController]
(
	UniqueSessionId				VARCHAR(200) NOT NULL,
)
CREATE UNIQUE CLUSTERED INDEX etlETLParallelLoadController_Indx ON [etl].[ETLParallelLoadController] ( UniqueSessionId )

IF ( SELECT OBJECT_ID( '[etl].[ETLParallelLoadController]') ) IS NOT NULL
BEGIN
	PRINT 'CREATED TABLE [etl].[ETLParallelLoadController]'
END
ELSE
BEGIN
	PRINT 'FAILED CREATING TABLE [etl].[ETLParallelLoadController]'
END
GO

CREATE TABLE [etl].[ETLDimensionsLog]
(
	LogKey					INT NOT NULL IDENTITY(0,1) UNIQUE ,
	EtlStartedOn			DATETIME NULL,
	StartDatePassed			DATE NULL,
	EndDatePassed			DATE NULL,
	LastEffectiveDateLoaded	DATE NULL,
	EtlCompletedOn			DATETIME NULL
)
GO
IF ( SELECT OBJECT_ID( '[etl].[ETLDimensionsLog]') ) IS NOT NULL
	PRINT 'CREATED TABLE [etl].[ETLDimensionsLog]'
ELSE
	PRINT 'FAILED CREATING TABLE [etl].[ETLDimensionsLog]'

CREATE TABLE [etl].[Dimensions Source Table]
(
	SourceTableKey								TINYINT IDENTITY,
	TableServer									VARCHAR (255) NULL, -- If set it is a Linked server, NULL when the same Server as dimensions
	TableDatabase								VARCHAR (255) NULL, -- NULL when the same DB as dimensions
	TableSchema									VARCHAR (255) NOT NULL,
	TableName									VARCHAR (255) NOT NULL UNIQUE,
	IsTrackedForChanges							BIT NOT NULL,
	EffectiveDateColumn							VARCHAR (255) NULL,
	Flags										VARCHAR (20)   NOT NULL,
	IsUsedForEarliestEffectiveDate				BIT NOT NULL,
	IsUsedForLatestEffectiveDate				BIT NOT NULL,
	EarliestEffectiveDateChanged				DATE NULL,
	LastEffectiveDate							DATE NULL,
	LastLoadedTableVersion						BIGINT NULL,
	EarliestEffectiveDateChangedBeingProcessed	DATE NULL,
	LastEffectiveDateBeingProcessed				DATE NULL,
	LastLoadedTableVersionBeingProcessed		BIGINT NULL,
	EtlRanOn									DATETIME NOT NULL DEFAULT GETDATE(),
)
GO
IF ( SELECT OBJECT_ID( '[etl].[Dimensions Source Table]') ) IS NOT NULL
	PRINT 'CREATED TABLE [etl].[Dimensions Source Table]'
ELSE
	PRINT 'FAILED CREATING TABLE [etl].[Dimensions Source Table]'	
	


SET ANSI_PADDING OFF

PRINT 'Created Dimension Authority tables in ' + DB_NAME() + ' database'