﻿/*
--	Copyright (c) 2007 - 2021 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
using System;
using System.Collections.Generic;
using System.Globalization;
using System.Text;
using System.Configuration;
using System.Collections;
using System.IO;
using System.Security.Principal;

using DWJobs.Common.AppEnvironment;

// from http://logging.apache.org/log4net/
using log4net;
using log4net.Config;
using log4net.Repository;

namespace DWJobs.Common.AppEnvironment
{
    /// <summary>
    ///  Configuration
    /// </summary>
    public partial class AppConfiguration
    {
        private static ILog log = LogManager.GetLogger("DWJobs.Common");
        private static string environment = "DEFAULT";
        private static string jobExplicit = null;

        static AppConfiguration()
        {
            // configure log4net
            XmlConfigurator.Configure();
            AppConfiguration.Environment = AppSettingsPerEnvironmentSection.Current.DefaultEnvironment;
        }

        public static void SetJob(string job)
        {
            AppConfiguration.jobExplicit = job;
            AppConfiguration.Job = (JobType)Enum.Parse(typeof(JobType), job, true);

            AppConfiguration.SetLogFile();
        }

        public static void SetOverrides(string[] args)
        {
            List <string> envKeyValues = new List<string>();

            envKeyValues.AddRange( args );

            for ( int indx = 0; indx < envKeyValues.Count; indx++ )
            {
                string envKeyValue = envKeyValues[indx];


                if ( indx == 0 && !envKeyValue.Contains( "=" ) )
                {
                    AppConfiguration.Environment = envKeyValue;
                }
                else if ( indx == 1 && !envKeyValue.Contains( "=" ) )
                {
                    AppConfiguration.SetJob(envKeyValue);
                }
                else if ( envKeyValue.Contains( "=" ) )
                {
                    string attributeName = envKeyValue.Substring(0, envKeyValue.IndexOf( '=' ) );
                    string attributeValue = envKeyValue.Substring(envKeyValue.IndexOf( '=' ) +1 );

                    if ( attributeName == "mailToList" )
                        AppConfiguration.MailToList = attributeValue;
                    else if ( attributeName == "ssasGoBackDaysForStartDateToLoad" )
                        AppConfiguration.SSASGoBackDaysForStartDateToLoad = Int32.Parse( attributeValue );
                    else if ( attributeName == "ssasMaxParallel" )
                        AppConfiguration.SSASMaxParallel = Int32.Parse( attributeValue );
                    else if ( attributeName == "timeoutMinutesOnParallelProcesses" )
                        AppConfiguration.TimeoutMinutesOnParallelProcesses = Int32.Parse( attributeValue );
                    else if ( attributeName == "floatCompareTheshold" )
                        AppConfiguration.FloatCompareTheshold = Double.Parse(attributeValue);
                    else if ( attributeName == "goBackDaysForStartDateToLoad" )
                        AppConfiguration.GoBackDaysForStartDateToLoad = Int32.Parse(attributeValue);
                    else if ( attributeName == "startDateToLoad" )
                        AppConfiguration.StartDateToLoad = DateTime.ParseExact(attributeValue, "d", CultureInfo.InvariantCulture);
                    else if ( attributeName == "ssasStartDateToLoad" )
                        AppConfiguration.SSASStartDateToLoad = DateTime.ParseExact(attributeValue, "d", CultureInfo.InvariantCulture);
                    else if ( attributeName == "dimStartDateToLoad" )
                        AppConfiguration.DIMStartDateToLoad = DateTime.ParseExact(attributeValue, "d", CultureInfo.InvariantCulture);
                    else if ( attributeName == "endDateToLoad" )
                        AppConfiguration.EndDateToLoad = DateTime.ParseExact(attributeValue, "d", CultureInfo.InvariantCulture);
                    else if (attributeName == "ssasServerName")
                        AppConfiguration.SSASServerName = attributeValue;
                    else if (attributeName == "ssasDBName")
                        AppConfiguration.SSASDBName = attributeValue;
                    else if (attributeName == "ssasFullLoad")
                        AppConfiguration.IsSSASFullLoad = (bool)(attributeValue != "Y" ? false : true);
                    else if (attributeName == "nonInitialFullLoad")
                        AppConfiguration.IsNonInitialFullLoad = (bool)(attributeValue != "Y" ? false : true);
                    else
                        throw new ApplicationException("Unsupported command line attribute - " + attributeName );
                }
            }
        }

        private static void SetLogFile()
        {
            // set log file name
            string fileName = "etl_log.txt";
            string loginName = WindowsIdentity.GetCurrent().Name;
            string loginNamePart = "";
            if (loginName != null)
            {
                if (loginName.LastIndexOf('.') >= 0 && loginName.LastIndexOf('.') < loginName.Length - 1)
                    loginNamePart = loginName.Substring(loginName.LastIndexOf('.') + 1).ToUpper() + "_";
                else if (loginName.LastIndexOf('\\') >= 0 && loginName.LastIndexOf('\\') < loginName.Length - 1)
                    loginNamePart = loginName.Substring(loginName.LastIndexOf('\\') + 1).ToUpper() + "_";

            }
            DateTime now = DateTime.Now;
            fileName = String.Format(@"{0}_{1}_{2}{3:D4}{4:D2}{5:D2}_{6:D2}{7:D2}{8:D2}_{9}", AppConfiguration.Environment, AppConfiguration.Job.ToString(), loginNamePart, now.Year, now.Month, now.Day, now.Hour, now.Minute, now.Second, fileName);

            AppConfiguration.ChangeLogFileName("FileAppender", Path.Combine(AppConfiguration.LogFileDir, fileName));
        }

        public static string Environment
        {
            get
            {
                return AppConfiguration.environment;
            }
            set
            {
                AppConfiguration.environment = value;
                AppConfiguration.SetLogFile();
            }
        }

        public static string Description
        {
            get
            {
                return (string)AppConfiguration.GetEnvironmentSettings().Description;
            }
        }

        public static ILog Log
        {
            get
            {
                return AppConfiguration.log;
            }
        }

        private static string GetAppSettingsValue(string key)
        {
            try
            {
                string val = ConfigurationManager.AppSettings[key];
                if (val == null)
                    throw new ApplicationException("");

                return val;
            }
            catch
            {
                AppConfiguration.Log.ErrorFormat("Cannot find application settings for key {0} in configuration file.", key);
                throw new ApplicationException("Undefined application settings");
            }
        }

        internal static AppEnvironmentSettingsElement GetEnvironmentSettings()
        {
            try
            {
                AppEnvironmentSettingsElement appEnvironmentSettings = AppSettingsPerEnvironmentSection.Current.AppEnvironmentSettings[AppConfiguration.Environment];
                return appEnvironmentSettings;
            }
            catch
            {
                string error = String.Format("Cannot find settings for \"{0}\" environment.", AppConfiguration.Environment);
                AppConfiguration.Log.Error(error);
                throw new ApplicationException(error);
            }
        }

        private static string LogFileDir
        {
            get
            {
                return (string)AppConfiguration.GetEnvironmentSettings().LogFileDir;
            }
        }

        public static new string ToString()
        {
            return AppConfiguration.GetEnvironmentSettings().ToString();
        }


        private static bool ChangeLogFileName(string appenderName, string newFilename)
        {
            log4net.Repository.ILoggerRepository rootRep;
            rootRep = log4net.LogManager.GetRepository();

            foreach (log4net.Appender.IAppender appender in rootRep.GetAppenders())
            {
                if (appender.Name.CompareTo(appenderName) == 0 && appender is log4net.Appender.FileAppender)
                {
                    log4net.Appender.FileAppender fileAppender = (log4net.Appender.FileAppender)appender;

                    string oldFileName = fileAppender.File;

                    fileAppender.File = newFilename;
                    fileAppender.ActivateOptions();

                    // delete old file if empty
	                FileInfo fi = new FileInfo(oldFileName);
                    if ( fi.Length == 0 )
                    {
                        try
                        {
                            File.Delete( oldFileName );
                        }
                        catch
                        {
                        }
                        
                    }

                    return true;  // Appender found and name changed to newFilename
                }
            }
            return false; // appender not found
        }

        public static string LogFileName
        {
            get
            {
                const string appenderName = "FileAppender";
                ILoggerRepository rootRep = log4net.LogManager.GetRepository();
                string fileName = "Unknown";

                foreach (log4net.Appender.IAppender appender in rootRep.GetAppenders())
                {
                    if (appender.Name.CompareTo(appenderName) == 0 && appender is log4net.Appender.FileAppender)
                    {
                        var fileAppender = (log4net.Appender.FileAppender)appender;

                        fileName = fileAppender.File;
                        break;
                    }
                }

                return fileName;
            }

        }

        public static void Usage()
        {
            AppConfiguration.Log.Error("This script takes one optional parameter\n\n");
            AppConfiguration.Log.Info("Usage: RunPositionsETL [env]\n"
                              + "        Valid environments can be found in App.config in appEnvironments section\n");
        }

    }
}
