﻿/*
--	Copyright (c) 2007 - 2021 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
using System;
using System.Collections.Generic;
using System.Text.RegularExpressions;
using System.IO;
using System.Net.NetworkInformation;

using log4net;

namespace DWJobs.Common
{
    public static class FileHelper
    {
        public static string SanitizeFileName(string fileName)
        {
            foreach (char c in Path.GetInvalidFileNameChars())
                fileName = fileName.Replace(c.ToString(), "");

            // also remove comma
            fileName = fileName.Replace(",", "");

            return fileName;
        }

        private static string GetUncDataDir(string filesShareOrExternalDir)
        {
            string uncDataDir = filesShareOrExternalDir;

            if (!(filesShareOrExternalDir.Contains(":") || filesShareOrExternalDir.Substring(0,2)== "\\\\")) // This is share as opposed to external directory name
            {
                IPGlobalProperties properties = IPGlobalProperties.GetIPGlobalProperties();
                uncDataDir = "\\\\" + properties.HostName + "." + properties.DomainName + "\\" + filesShareOrExternalDir;
                if (!Directory.Exists(uncDataDir))
                {
                    string error = String.Format("Cannot find {0} share on current computer.", filesShareOrExternalDir);
                    throw new ApplicationException(error);
                }
            }

            return uncDataDir;
        }

        public static string GetFilePathForTimestampFile(string filePath, string fileNamePrefix, string fileExtension)
        {
            return FileHelper.GetFilePathForTimestampFile( filePath, fileNamePrefix, fileExtension, true );
        }

        public static string GetFilePathForTimestampFile(string filePath, string fileNamePrefix, string fileExtension, bool appendTempFileNameForUniqueness ) // = true)
        {
            if (filePath == null)
                filePath = Path.GetDirectoryName(Path.GetTempFileName());
            if (fileNamePrefix == null)
                fileNamePrefix = "";
            if (fileExtension == null)
                fileExtension = "";
            else if (fileExtension != "" && fileExtension[0] != '.')
                fileExtension = fileExtension.Insert(0, ".");

            DateTime now = DateTime.Now;
            string fileName = String.Format("{0} as of {1:D4}{2:D2}{3:D2}_{4:D2}{5:D2}{6:D2}{7}{8}", fileNamePrefix, now.Year, now.Month, now.Day, now.Hour, now.Minute, now.Second, appendTempFileNameForUniqueness ? "_" + Path.GetFileNameWithoutExtension(Path.GetTempFileName()) : "", fileExtension);
            string outputFilePath = Path.Combine(filePath, fileName);
            if (File.Exists(outputFilePath))
                File.Delete(outputFilePath);

            return outputFilePath;
        }

        public static string GetUncFilePathFromFilePath(string filePath, string filesShareOrExternalDir)
        {
            string fileName = Path.GetFileName(filePath);
            return Path.Combine(FileHelper.GetUncDataDir(filesShareOrExternalDir), fileName);
        }

        public static string SaveReportResult(IEnumerable<string> result, string reportResultFilesPath, string filesShareOrExternalDir)
        {
            string outputFilePath = FileHelper.GetFilePathForTimestampFile(reportResultFilesPath,"","txt");

            DateTime start = DateTime.Now;
            using (StreamWriter streamWriter = File.CreateText(outputFilePath ))
            {
                if ( result != null )
                    foreach (string resultLine in result)
                    {
                        streamWriter.WriteLine(resultLine);
                    }

                streamWriter.Close();
            }
            File.SetCreationTime(outputFilePath, start); // Otherwize the other thread can see File Info for old file with the same name

            string uncOutputFilePath = FileHelper.GetUncFilePathFromFilePath(outputFilePath, filesShareOrExternalDir);

            return uncOutputFilePath;
        }

        public static void AppendToFile(string fileMainPath, string fileToAppendPath )
        {
            if (!File.Exists(fileToAppendPath))
                return;

            if (!File.Exists(fileMainPath))
            {
                File.Move(fileToAppendPath, fileMainPath);
                return;
            }

            int bufSize = 1024 * 64;
            byte[] buf = new byte[bufSize];
            using (FileStream outFile =  new FileStream(fileMainPath, FileMode.Append, FileAccess.Write, FileShare.None, bufSize))
            {
                using (FileStream inFile = new FileStream(fileToAppendPath, FileMode.Open, FileAccess.Read, FileShare.Read, bufSize))
                {
                    int br = 0;
                    while ((br = inFile.Read(buf, 0, buf.Length)) > 0)
                    {
                        outFile.Write(buf, 0, br);
                    }
                }
            }
            File.Delete(fileToAppendPath);
        }

        public static string MoveProcessedFile(string filePath, string processedFilesDir, string fileName, ILog log)
        {
            return FileHelper.MoveProcessedFile(filePath, processedFilesDir, fileName, null, true, log);
        }

        public static string MoveProcessedFile(string filePath, string processedFilesDir, string fileName, string extension, ILog log)
        {
            return FileHelper.MoveProcessedFile(filePath, processedFilesDir, fileName, extension, true, log);
        }

        public static string MoveProcessedFile(string filePath, string processedFilesDir, string fileName, bool appendTempFileNameForUniqueness, ILog log)
        {
            return FileHelper.MoveProcessedFile(filePath, processedFilesDir, fileName, null, appendTempFileNameForUniqueness, log);
        }

        public static string MoveProcessedFile(string filePath, string processedFilesDir, string fileName, string extension, bool appendTempFileNameForUniqueness, ILog log)
        {
            if (!Directory.Exists(processedFilesDir))
                FileHelper.CreateFolder(processedFilesDir);

            string processedFilePath = FileHelper.GetFilePathForTimestampFile(processedFilesDir,
                (String.IsNullOrEmpty(fileName) ? Path.GetFileNameWithoutExtension(filePath) : fileName ),
                String.IsNullOrEmpty(extension) ? Path.GetExtension(filePath) : extension, appendTempFileNameForUniqueness);

            try
            {
                if (File.Exists(processedFilePath))
                    File.Delete(processedFilePath);
                //Thread.Sleep(5000); // Let excell release a file handle. There was a problem moving the second file.
                File.Move(filePath, processedFilePath);

                return processedFilePath;
            }
            catch (Exception e)
            {
                log.Error("File move failed", e);
                throw new ApplicationException(String.Format("Cannot move processed file {0} to {1} \n", filePath, processedFilePath));
            }
        }

        public static void CopyFile(string filePath, string destinationDir, string destinationFileName, ILog log)
        {
            string destinationFilePath = Path.Combine(destinationDir, destinationFileName);
            try
            {
                File.Copy(filePath, destinationFilePath,false);
            }
            catch (Exception e)
            {
                log.Error("File copy failed", e);
                throw new ApplicationException(String.Format("Cannot copy file {0} to {1} \n", filePath, destinationFilePath));
            }
        }

        public static void DeleteOldResultFiles(string reportResultFilesPath, ILog log)
        {
            Regex reportResultFileNameRegEx = new Regex(@"^20[0-9][0-9][01][0-9][0-3][0-9]_[0-9][0-9][0-9][0-9][0-9][0-9]_[A-Z0-9]+\.TMP$");

            // Get a list of all the files in the directory and delete them
            try
            {
                // Create a reference to the current directory.
                DirectoryInfo di = new DirectoryInfo(reportResultFilesPath);

                // Create an array representing the files in the result files directory.
                FileInfo[] fileinfos = di.GetFiles();
                foreach (FileInfo fileinfo in fileinfos)
                {
                    Match m = reportResultFileNameRegEx.Match(fileinfo.Name.ToUpper());
                    if (m.Success)
                    {
                        log.InfoFormat("Deleted old result file {0}", fileinfo.FullName);
                        File.Delete(fileinfo.FullName);
                    }
                }
            }
            catch (Exception e)
            {
                log.Warn("Cannot delete old result files\n", e);
            }
        }

        public static void CreateFolder(string path)
        {
            if (!Directory.GetParent(path).Exists)
            {
                CreateFolder(Directory.GetParent(path).FullName);
            }
            if (!System.IO.Directory.Exists(path))
            {
                System.IO.Directory.CreateDirectory(path);
            }
        }

    }
}
