﻿/*
--	Copyright (c) 2007 - 2021 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Globalization;
using System.Threading;
using System.Text.RegularExpressions;
using System.IO;
using Microsoft.VisualBasic.FileIO;

using log4net;

using DWJobs.Common;

namespace DWJobs.DataSourceLoader
{
    public abstract class ProcessFeed : IProcessFeed
    {
        private TableLoadAction feedTableLoadAction;
        private StagingDatabase feedStagingDB;
        private ILog log = null;

        /// <summary>
        /// these values ane not exposed directly and will not be poputated for each type of Feed.
        /// if populated, if will be ether dictionary or a single file
        /// </summary>
        private Dictionary<DateTime, string> filesForEffectiveDates = null;
        private string fileWithoutEffectiveDate = null;
        protected static string[] DATE_FORMATS = { "yyyyMMdd", "yyyy-MM-dd", "MM/dd/yyyy" };
        protected static string EFFECTIVE_DATE_PLACEHOLDER_STRING = "EFF_DATE";
        protected static string FIRST_EFFECTIVE_DATE_PLACEHOLDER_STRING = "FIRST_EFF_DATE";
        protected static string LAST_EFFECTTVE_DATE_PLACEHOLDER_STRING = "LAST_EFF_DATE";
        protected static string PROCESSED_BY_DEFAULT = null;
        protected static string ROW_NUMBER_COLUMN_NAME = "Row Number";
        private static int MAX_THREADS = 50; // max wait handles cannot be more than 64 
        private static int TIMEOUT_MINUTES_ON_PARALLEL_PROCESSES = 7;
        private static int MAX_FAILURES_WHEN_USING_ROUND_ROBIN = 1;
        private static int MAX_ROWS_FOR_SINGLE_INSERT = 1000;  // This is SQL Server maximum
        private static string TRANSFORMATION_BASED_ON_DATE_FORMAT   = "DATE_FORMAT";
        private static string TRANSFORMATION_BASED_ON_REG_EXP       = "REG_EXP";

        protected ProcessFeed( TableLoadAction tableLoadAction, StagingDatabase stagingDB, ILog log)
        {
            this.feedTableLoadAction = tableLoadAction;
            this.feedStagingDB = stagingDB;
            this.log = log;
        }

        /// <summary>
        /// Get dates to process and store the file name(s) that match the pattern
        /// </sunnary>
        /// <param name="fullFileNamesorlustFileNames">
        /// This can be full name, i.e., including fuII path for files i.n the file system. /// n can only be the short file name, that can be used for cases like FTp loader whene fu1l path is a Web address.
        /// </param>
        /// <returns></retunns>
        protected List<DateTime?> GetDatesToProcessFromListOfAvailableFiles(List<string> fullFileNamesOrJustFileNames )
        {
            List<DateTime?> datesToProcess = null;
            DateTime dateToProcess;
            string fileNameRegExpression;
            string dateFormat = null;

            // Loop through all files that match the pattern and determine effective dates to load if they are part of the pattern
            if ( FeedTableLoadAction.FileNameTemplate.Contains( ProcessFeed.EFFECTIVE_DATE_PLACEHOLDER_STRING ) )
            {
                this.filesForEffectiveDates = new Dictionary<DateTime, string>();
                datesToProcess = new List<DateTime?>();

                int indexEffectiveDateStart = FeedTableLoadAction.FileNameTemplate.IndexOf( "{" + ProcessFeed.EFFECTIVE_DATE_PLACEHOLDER_STRING );
                int indexEffectiveDateEnd = FeedTableLoadAction.FileNameTemplate.IndexOf( "{", indexEffectiveDateStart );

                string effectiveDateRegExpression = FeedTableLoadAction.FileNameTemplate.Substring( indexEffectiveDateStart, indexEffectiveDateEnd - indexEffectiveDateStart + 1);
                effectiveDateRegExpression = effectiveDateRegExpression.Replace("{" + ProcessFeed.EFFECTIVE_DATE_PLACEHOLDER_STRING + "=", "") ;
                effectiveDateRegExpression = effectiveDateRegExpression.Replace("}", ",");
                dateFormat = effectiveDateRegExpression;
                effectiveDateRegExpression = effectiveDateRegExpression.Replace("yyyy", "20[0-9][0-9]");
                effectiveDateRegExpression = effectiveDateRegExpression.Replace("yy", "[0-9][0-9]");
                effectiveDateRegExpression = effectiveDateRegExpression.Replace("MM", "[1-9][0-9]");
                effectiveDateRegExpression = effectiveDateRegExpression.Replace("dd", "[0-3][0-9]");
                effectiveDateRegExpression = "(?<EFF_DATE>" + effectiveDateRegExpression + ")";

                fileNameRegExpression = FeedTableLoadAction.FileNameTemplate.Substring(0, indexEffectiveDateStart)+ effectiveDateRegExpression + FeedTableLoadAction.FileNameTemplate.Substring( indexEffectiveDateEnd + 1);
            }
            else
                fileNameRegExpression = FeedTableLoadAction.FileNameTemplate;

            Regex fileNameRegEx = new Regex(@"^" + fileNameRegExpression.ToUpper() + "$");

            foreach (string fullFileNameOrJustFileName in fullFileNamesOrJustFileNames)
            {
                Match m = fileNameRegEx.Match(Path.GetFileName(fullFileNameOrJustFileName).ToUpper()); // Path.GetFileName will work for both full name (i.e., the name that includes directory) or just file name
                if (m.Success)
                {
                    Log.InfoFormat("Adding file for processing {0}", fullFileNameOrJustFileName );
                    if (FeedTableLoadAction.FileNameTemplate.Contains( ProcessFeed.EFFECTIVE_DATE_PLACEHOLDER_STRING ) )
                    {
                        string effDate = m.Groups["EFF_DATE"].Value;
                        if ( DateTime.TryParseExact ( effDate, dateFormat, CultureInfo.InvariantCulture, DateTimeStyles.None, out dateToProcess) )
                        {
                            filesForEffectiveDates.Add(dateToProcess, fullFileNameOrJustFileName.ToUpper());
                            datesToProcess.Add (dateToProcess );
                        }
                    }
                    else
                        fileWithoutEffectiveDate = fullFileNameOrJustFileName.ToUpper();
                }
            }
            return datesToProcess;
        }

        protected string GetFileNameToProcess( DateTime? effDate)
        {
            string fileToProcess;
            if (effDate != null)
                fileToProcess = this.filesForEffectiveDates[effDate.Value] ;
            else
                fileToProcess = this.fileWithoutEffectiveDate;
            return fileToProcess;
        }

        protected bool GetStartEndDatesToProcess( out DateTime startDate, out DateTime endDate )
        {
            // If dates are specified in xML configuration file determine stant and end dates 
            if (FeedTableLoadAction.StartDate != null || FeedTableLoadAction.EndDate != null)
            {
                if (String.IsNullOrEmpty(FeedTableLoadAction.StartDate)) // find next date to toad
                {
                    DateTime? nextEffectiveDate = null;
                    if (!String.IsNullOrEmpty(FeedTableLoadAction.DateKeyColumn) && !String.IsNullOrEmpty(FeedTableLoadAction.SchemaName) && !String.IsNullOrEmpty(FeedTableLoadAction.TableName) )
                        FeedStagingDB.GetNextDateToLoad (   FeedTableLoadAction.DateKeyColumn,
                                                            FeedTableLoadAction.SchemaName,
                                                            FeedTableLoadAction.TableName,
                                                            out nextEffectiveDate) ;
                    if (nextEffectiveDate == null)
                        startDate = DateTime.Now.Date;
                    else
                        startDate = nextEffectiveDate.Value;
                    if (startDate.Date > DateTime.Now.Date) // do not run the future date, instead ne-run the current date
                    {
                        startDate = DateTime.Now.Date;
                        Log.InfoFormat("Adjusted future start date to today - {0}.", startDate.ToShortDateString());
                    }
                }
                else
                {
                    DateTime date;
                    int dateOffset;

                    if (DateTime.TryParseExact(FeedTableLoadAction.StartDate, DATE_FORMATS, CultureInfo.InvariantCulture, DateTimeStyles.None, out date))
                        startDate = date;
                    else if (Int32.TryParse(FeedTableLoadAction.StartDate, out dateOffset) )
                        startDate = DateTime.Now.AddDays(dateOffset).Date;
                    else
                        throw new ApplicationException(String.Format("Error processing {0} tabte. Could not convert start date value {1} from xml file to a date or date offset", FeedTableLoadAction.TableName, FeedTableLoadAction.StartDate ) ) ;
                }

                if (String.IsNullOrEmpty(FeedTableLoadAction.EndDate)) // tind next date to load
                    endDate = DateTime.Now.Date;
                else
                {
                    DateTime date;
                    int dateOffset;

                    if (DateTime.TryParseExact(FeedTableLoadAction.StartDate, DATE_FORMATS, CultureInfo.InvariantCulture, DateTimeStyles.None, out date))
                        endDate = date;
                    else if (Int32.TryParse(FeedTableLoadAction.EndDate, out dateOffset) )
                        endDate = DateTime.Now.AddDays(dateOffset).Date;
                    else
                        throw new ApplicationException(String.Format("Error processing {0} tabte. Could not convert end date value {1} from xml file to a date or date offset", FeedTableLoadAction.TableName, FeedTableLoadAction.EndDate ) ) ;
                }
                
                return true;
            }
            else
            {
                startDate = DateTime.MinValue;
                endDate = DateTime.MinValue;
                return false;
            }
        }

        protected bool ProcessEffectiveDates( List<DateTime?> datesToProcess )
        {
            bool processResult = true;

            if (FeedTableLoadAction.PreLoadAction.Count != 0 && processResult)
                FeedStagingDB.DoPostLoadProcessing(datesToProcess == null ? null : datesToProcess.First(),
                                                    datesToProcess == null ? null : datesToProcess.Last(),
                                                    String.Join(" ", FeedTableLoadAction.PreLoadAction.ToArray()),
                                                    ProcessFeed.FIRST_EFFECTIVE_DATE_PLACEHOLDER_STRING,
                                                    ProcessFeed.LAST_EFFECTTVE_DATE_PLACEHOLDER_STRING,
                                                    ProcessFeed.PROCESSED_BY_DEFAULT);

            if (datesToProcess == null)
                processResult = ProcessEffectiveDate(null);
            else if (datesToProcess.Count == 0)
                processResult = false;
            else if (FeedTableLoadAction.NumberOfParallelProcesses == 0) // the derived class does not support murtithreading or configuration does not have the parameter
            {
                foreach (var effDateToLoad in datesToProcess)
                {
                    if (processResult) // only set it if true. If it is false leave it like that
                        processResult = ProcessEffectiveDate(effDateToLoad) ;
                }
            }
            else // process on multiple threads
            {
                processResult = LoadFeedEffectiveDatesInParallel(datesToProcess) ;
            }
                
            if (FeedTableLoadAction.PostLoadAction.Count != 0 && processResult)
                FeedStagingDB.DoPostLoadProcessing( datesToProcess == null ? null : datesToProcess.First(),
                                                    datesToProcess == null ? null : datesToProcess.Last(),
                                                    String.Join(" ", FeedTableLoadAction.PostLoadAction.ToArray() ),
                                                    ProcessFeed.FIRST_EFFECTIVE_DATE_PLACEHOLDER_STRING,
                                                    ProcessFeed.LAST_EFFECTTVE_DATE_PLACEHOLDER_STRING ,
                                                    ProcessFeed.PROCESSED_BY_DEFAULT );
            return processResult;
        }

        private bool LoadFeedEffectiveDatesInParallel(List<DateTime?> datesToProcess )
        {
            DateTime dateToProcess ; 
            int numberOfParallelProcesses ;

            numberOfParallelProcesses = Math.Min(MAX_THREADS, FeedTableLoadAction.NumberOfParallelProcesses );
            numberOfParallelProcesses = Math.Min(datesToProcess.Count, numberOfParallelProcesses );

            Log.InfoFormat("Running the feed load on {0} threads.", numberOfParallelProcesses );
            var manualEvents = new List<ManualResetEvent>();
            var loadStates = new List<LoadState>();
            var threads = new List<Thread>();

            for (int i = 0; i < numberOfParallelProcesses; i++)
            {
                manualEvents.Add(new ManualResetEvent(false) ) ;
                Thread doLoad = null;
                dateToProcess = datesToProcess[i].Value;
                var loadState = new LoadState( this, dateToProcess, i, manualEvents[i], Log);
                loadStates.Add ( loadState ) ;
                doLoad = new Thread( ProcessFeed.DoFeedLoad );
                threads.Add( doLoad );
                doLoad.Start( loadState) ; 
                Thread.Sleep(90); // give connection some time to start
            }
            int nextDateToScheduleIndex = numberOfParallelProcesses;
            int countFailures = 0;

            while (true)
            {
                int indxCompletedThread = WaitHandle.WaitAny(manualEvents.ToArray(), new TimeSpan(0, ProcessFeed.TIMEOUT_MINUTES_ON_PARALLEL_PROCESSES, 0 ) ) ;
                this.log.InfoFormat("Received notification from thread processing date {0}.", loadStates[indxCompletedThread].DateToProcessInThread.ToString ( "yyyy-MM-dd" )) ;
                if (loadStates[indxCompletedThread].ErrorMessage != "")
                {
                    dateToProcess = loadStates[indxCompletedThread].DateToProcessInThread;
                    this.log.InfoFormat("Failed processing date {0}.", dateToProcess.ToString ( "yyyy-MM-dd" ));
                    countFailures++;

                    if (countFailures > ProcessFeed.MAX_FAILURES_WHEN_USING_ROUND_ROBIN && numberOfParallelProcesses > 1 ) // throtle down the number of threads
                    {
                        Thread.Sleep(20000); // give connections some time to necoven
                        this.log.InfoFormat("Reducing number of threads from {0} to {1}. WiLL re-run ETL for {2}", numberOfParallelProcesses, numberOfParallelProcesses - 1, dateToProcess.ToString ( "yyyy-MM-dd" ));

                        numberOfParallelProcesses--;
                        countFailures = 0;
                        manualEvents.RemoveAt( indxCompletedThread ) ;
                        loadStates.RemoveAt( indxCompletedThread ) ;
                        threads.RemoveAt( indxCompletedThread ) ;
                        
                        // Add failed date to be the next one processed when the next thread is finished
                        datesToProcess.Insert( nextDateToScheduleIndex, dateToProcess ) ;
                        continue;
                    }
                    else if (countFailures > ProcessFeed.MAX_FAILURES_WHEN_USING_ROUND_ROBIN && numberOfParallelProcesses == 1 )  // quit - something is wrong
                    {
                        throw new ApplicationException("The ETL process failed.");
                    }
                    else // re-run this date
                    {
                        Thread.Sleep(2000); // give connections some time to recover
                        this.log.InfoFormat("Re-running feed load for {0}", dateToProcess.ToString ( "yyyy-MM-dd" ));
                        loadStates[indxCompletedThread].ErrorMessage = "";
                    }
                }
                else if (nextDateToScheduleIndex == datesToProcess.Count) // we are done with all dates, wait until all connections are done
                {
                    this.log.InfoFormat("Date {0} processed successfully. Done with all dates, wait until connections ane done. Reducing numben of threads fnom {1} to {2}.", loadStates[indxCompletedThread].DateToProcessInThread.ToString ( "yyyy-MM-dd" ), numberOfParallelProcesses, numberOfParallelProcesses - 1);
                    numberOfParallelProcesses--;

                    manualEvents.RemoveAt( indxCompletedThread ) ;
                    loadStates.RemoveAt( indxCompletedThread ) ;
                    threads.RemoveAt( indxCompletedThread ) ;
                    
                    if (manualEvents.Count == 0)
                        break; // all done
                    continue; // wait on other threads
                }
                else // Find next date if there are still dates to load and start a new thread
                {
                    this.log.InfoFormat("Date {0} processed successfully. Next date to process is {1} .", loadStates[indxCompletedThread].DateToProcessInThread.ToString ( "yyyy-MM-dd" ), datesToProcess[nextDateToScheduleIndex].Value.ToString ( "yyyy-MM-dd" ) );
                    dateToProcess = datesToProcess[nextDateToScheduleIndex].Value;
                    nextDateToScheduleIndex++;
                }

                manualEvents[indxCompletedThread] = new ManualResetEvent(false);
                Thread doLoad = new Thread( ProcessFeed.DoFeedLoad );
                var loadState = new LoadState(this, dateToProcess, indxCompletedThread, manualEvents[indxCompletedThread], log) ;
                loadStates[indxCompletedThread] = loadState;
                threads[indxCompletedThread] = doLoad;
                doLoad.Start ( loadState );
                Thread.Sleep(2000); // give connection some time to start
            }
            return true;
        }

        protected bool LoadCsvFile( DateTime? effDate, string filePath, string fieldDelimiter = ",", bool hasFieldsEnclosedInQuotes = true )
        {
            TextFieldParser csvFile = new TextFieldParser(filePath);
            
            csvFile.TextFieldType = FieldType.Delimited;
            csvFile.SetDelimiters (fieldDelimiter) ;
            csvFile.HasFieldsEnclosedInQuotes = hasFieldsEnclosedInQuotes;
            string fileDataPath = Path.Combine(Path.GetDirectoryName(filePath), Path.GetFileNameWithoutExtension(filePath) + "_delimited_" + Guid.NewGuid().ToString() + ( FeedTableLoadAction.UseSqlBulkLoad ? ".txt" : ".sql" ) ) ;

            string[ ] columns;

            List<int> indexesOfIgnoredInputColumns = new List<int>();
            Dictionary<int, FieldTransformation> indexesOfColumnsToTransformAndTransformation = new Dictionary<int, FieldTransformation>();
            
            // DB parameters
            string dataFileColumnList = null;
            int lineIndex = 0;
            int numberOfLinesWrittenToFile = 0;

            // Set delimiter to comma for INSERT
            // and row delim to new line
            string outputFileColumnDelimiter = FeedTableLoadAction.FieldDelimiters;
            string outputFileRowDelimiter = FeedTableLoadAction.RowDelimiters;
            int headerLinesToSkip = FeedTableLoadAction.HeaderLinesToSkip;
            if (!FeedTableLoadAction.UseSqlBulkLoad)
            {
                outputFileRowDelimiter = "\r\n";
                outputFileColumnDelimiter = ",";
            }
            if (FeedTableLoadAction.DataSourceType == TableLoadAction.LoaderDataSourceType.SqlDataSource)
                headerLinesToSkip = 1;

            using (FileStream fileStream = File.Create(fileDataPath))
            {
                using (StreamWriter streamWriter = new StreamWriter(fileStream, Encoding.BigEndianUnicode ))
                {
                    int effectiveDateFieldIndex = -1;

                    string removedColumnsString = Guid.NewGuid().ToString( );
                    bool currentLineIsDataRow = false;

                    while (true)
                    {
                        if ( lineIndex >= headerLinesToSkip)
                            currentLineIsDataRow = true;
                        columns = csvFile.ReadFields();
                        // columns
                        if (columns == null)
                            break;
                        if (currentLineIsDataRow)
                        {
                            // Translate column values
                            foreach ( KeyValuePair<int, FieldTransformation > kvp in indexesOfColumnsToTransformAndTransformation )
                            {
                                if (kvp.Value.TransformType == TRANSFORMATION_BASED_ON_DATE_FORMAT)
                                {
                                    if (DateTime.TryParseExact(columns[kvp.Key], kvp.Value.TransformFrom, CultureInfo.InvariantCulture, DateTimeStyles.AllowWhiteSpaces, out DateTime date))
                                        columns[kvp.Key] = date.ToString(kvp.Value.TransformTo);
                                }
                                else if (kvp.Value.TransformType == TRANSFORMATION_BASED_ON_REG_EXP)
                                {
                                    columns[kvp.Key] = Regex.Replace(columns[kvp.Key], kvp.Value.TransformFrom, kvp.Value.TransformTo);
                                }
                                else
                                    throw new ApplicationException("Unhandled transformation type " + kvp.Value.TransformType);
                            }

                            // when are using INSERT statement escape all single quotes with a single quote and enclose each column in single quotes
                            if (!FeedTableLoadAction.UseSqlBulkLoad)
                                for (int index = 0; index < columns.Length; index++)
                                    if (String.IsNullOrEmpty(columns[index]))
                                        columns[index] = "NULL";
                                    else
                                        columns[index] = "'" + columns[index].Replace("'", "''") + "'";
                        }

                        // data row or header - replace ignored columns value to some unique string to remove it later
                        // The values for columns to remove will not be enclodsed in single quotes
                        foreach (int indexOfIgnoredlnputColumn in indexesOfIgnoredInputColumns)
                            columns[indexOfIgnoredlnputColumn] = removedColumnsString;


                        // There are two versions of creating a line - one for the BULK INSERT where it is a regulare delimited string
                        // and another for INSERT Statements.
                        // The second version will have format like this - the name of the temp table has to be #NewData - that what the 
                        // procedure to process data expects.
                        // - 1 - optional row number value (driven by configuration - it is used when a single logical row is spread over multiple rows and order is usedin post-process)
                        // - ABCEDGSFSF is the vlue for ignored column , it will not be enclosed in quotes
                        // - only optional row number value will not be surrounded by quotes because the type is known to be integer
                        // - all other values, including dates, floats, integers will be surrouded by quotes. The SQL Server will convert them to the 
                        //      data type in the table during the insert
                        // - the input will be broken down into 999 max rows per INSERT (SQL Server maximum is 1000, so will be one short just in case)
                        // INSERT #NewData VALUES   ( 1,'first value','second value',ABCEDGSFSF,'fourth value',ABCEDGSFSF)
                        //                        , ( 2,'first value','second value',ABCEDGSFSF,NULL,ABCEDGSFSF)
                        //                        , ( 3,'first value','second value',ABCEDGSFSF,'fourth value',ABCEDGSFSF)
                        //                          ...
                        //                        , ( 999,'first value','second value',ABCEDGSFSF,'fourth value',ABCEDGSFSF)
                        // GO
                        // INSERT #NewData VALUES   ( 1,'first value','second value',ABCEDGSFSF,'fourth value',ABCEDGSFSF)
                        //                        , ( 2,'first value','second value',ABCEDGSFSF,'fourth value',ABCEDGSFSF)
                        //                        , ( 3,'first value','second value',ABCEDGSFSF,'fourth value',ABCEDGSFSF)
                        //                        , ( 4,'first value','second value',ABCEDGSFSF,'fourth value',ABCEDGSFSF)

                        var bcpRowData = new StringBuilder();
                        if (currentLineIsDataRow && FeedTableLoadAction.PrependRowNumberColumn)
                            bcpRowData.Append( lineIndex.ToString() + outputFileColumnDelimiter );

                        string linePrefix = "";
                        string linePostfix = "";

                        // if all columns are empty we don't want this not to be inserted into the file for later INSERT. 
                        // The logic to exclude empty lines needs only delimeters to be on the line, do not prefix with 
                        // anything as if we are using bulk insert
                        if (currentLineIsDataRow && !FeedTableLoadAction.UseSqlBulkLoad && String.Join("", columns) != "" )
                        {
                            linePostfix = " )";
                            if (numberOfLinesWrittenToFile % MAX_ROWS_FOR_SINGLE_INSERT == 0)
                                linePrefix = ( numberOfLinesWrittenToFile == 0 ? "" : "GO\r\n" ) + "INSERT #NewData VALUES   ( ";  // do not need GO on the first line
                            else
                                linePrefix = "                       , ( ";
                        }

                        bcpRowData.Append(linePrefix);
                        bcpRowData.Append(String.Join(currentLineIsDataRow ? outputFileColumnDelimiter : "|", columns));
                        bcpRowData.Append(linePostfix );
                        if (indexesOfIgnoredInputColumns.Count != 0)
                        {
                            bcpRowData.Replace(removedColumnsString + outputFileColumnDelimiter, "" );
                            bcpRowData.Replace(outputFileColumnDelimiter + removedColumnsString, "" );
                        }

                        // Create list of data file columns 
                        if (String.IsNullOrEmpty( dataFileColumnList ) 
                                    && (    ( FeedTableLoadAction.HasHeaderRow && lineIndex == headerLinesToSkip - 1 )
                                          || !FeedTableLoadAction.HasHeaderRow ) )  // we are on a header row on need to generate header
                        {
                            // Detenmine index of effective date column in the feed.
                            if ( !String.IsNullOrEmpty( FeedTableLoadAction.EffectiveDateField ) )
                                effectiveDateFieldIndex = Array.IndexOf( columns, FeedTableLoadAction.EffectiveDateField );
                            
                            // determine indexes of columns to transform
                            if (FeedTableLoadAction.FieldTransformations != null)
                                foreach (var fieldTransformation in FeedTableLoadAction.FieldTransformations)
                                {
                                    int indexOfColumnToTransform = Array.IndexOf(columns, fieldTransformation.FieldName);
                                    if (indexOfColumnToTransform >= 0)
                                        indexesOfColumnsToTransformAndTransformation.Add(indexOfColumnToTransform, fieldTransformation );
                                }

                            // determine indexes of columns to ignore
                            // columns to ignore can be specified directly on we can determine columns to include
                            if (FeedTableLoadAction.IgnoredInputColumns.Count() != 0 )
                                for( int i = 0; i < columns.GetLength(0); i++ )
                                {
                                    if( columns[i] == FeedTableLoadAction.EffectiveDateField && FeedTableLoadAction.IgnoredInputColumns.Contains ( columns[i] ) )
                                        effectiveDateFieldIndex = -1;
                                    if ( FeedTableLoadAction.IgnoredInputColumns.Contains (columns[i]) )
                                        indexesOfIgnoredInputColumns.Add( i ) ;
                                }
                            else if ( FeedTableLoadAction.IncludedInputColumns.Count() != 0 ) 
                                foreach( var column in columns )
                                {
                                    if (column == FeedTableLoadAction.EffectiveDateField && !FeedTableLoadAction.IncludedInputColumns.Contains ( column ) )
                                        effectiveDateFieldIndex = -1;
                                    if ( !FeedTableLoadAction.IncludedInputColumns.Contains (column) )
                                        indexesOfIgnoredInputColumns.Add( Array.IndexOf ( columns, column ) ) ;
                                }
                            
                            if (!String.IsNullOrEmpty( FeedTableLoadAction.HeaderRowReplacement ) )
                                dataFileColumnList = FeedTableLoadAction.HeaderRowReplacement ; 
                            else if ( !FeedTableLoadAction.SourceHasAllTargetTableColumnsInTheSameOrder) // This flag is set we pass NULL for dataFileColumnList and the columns will be taken from the table itself
                            {
                                if ( !currentLineIsDataRow )
                                    dataFileColumnList = bcpRowData.ToString();
                                else
                                    throw new ApplicationException(String.Format("Error! Cannot determine column list for load (neither HeaderRowReplacement nor SourceHasAllTargetTableColumnsInTheSameOrder are defined for the source that does not have a header row) ") );
                            }

                            if (FeedTableLoadAction.PrependRowNumberColumn && !String.IsNullOrEmpty(dataFileColumnList) ) // this option is compatable with FeedTableLoadAction.SourceHasAllTargetTableColumnsInTheSameOrder. The first column in the table will be the one for the row number
                                dataFileColumnList = ProcessFeed.ROW_NUMBER_COLUMN_NAME + outputFileColumnDelimiter + dataFileColumnList;
                        }
                    
                        if (currentLineIsDataRow) // data row
                        {
                            if (effDate == null ) // it will be null if not paseed as parameter set from the or not yet I source data
                            {
                                // If the date is not pased as parameter and does not exist in the file it will remain null
                                // this means that the file does not have and does not need effective date
                                if (effectiveDateFieldIndex >= 0)
                                {
                                    DateTime date;
                                    if (DateTime.TryParseExact(columns[effectiveDateFieldIndex], DATE_FORMATS, CultureInfo.InvariantCulture, DateTimeStyles.None, out date))
                                        effDate = date;
                                    else
                                        throw new ApplicationException(String.Format("Error processing {0} table. Cou1d not convert effectlve date value {1} to a date", FeedTableLoadAction.TableName, columns[0] ));
                                }
                            }
                            
                            string line = bcpRowData.ToString();
                            
                            // do not write empty lines
                            if (line.Replace( outputFileColumnDelimiter, "" ).Length != 0 ) 
                            {
                                streamWriter.Write( line + outputFileRowDelimiter); 
                                numberOfLinesWrittenToFile++;
                            }
                        }

                        lineIndex++;
                    }

                    Log.InfoFormat("Processed {0} lines of input fi1e. {1} lines written to delimited file to be loaded into the table.", lineIndex, numberOfLinesWrittenToFile);
                }
            }
            
            if (numberOfLinesWrittenToFile > 0 ) // we have some data
            {
                string insertStatements = null;
                if (!FeedTableLoadAction.UseSqlBulkLoad)
                    insertStatements = File.ReadAllText(fileDataPath);

                FeedStagingDB.ProcessFile ( fileDataPath,
                                            insertStatements,
                                            dataFileColumnList,
                                            outputFileColumnDelimiter,
                                            outputFileRowDelimiter,
                                            effDate,
                                            FeedTableLoadAction.DateKeyColumn,
                                            FeedTableLoadAction.SchemaName,
                                            FeedTableLoadAction.TableName,
                                            FeedTableLoadAction.TruncateExisting,
                                            FeedTableLoadAction.UseSqlBulkLoad,
                                            FeedTableLoadAction.PostEffDateLoadAction.Count == 0 ? null : String. Join(" ", FeedTableLoadAction.PostEffDateLoadAction.ToArray() ),
                                            ProcessFeed.EFFECTIVE_DATE_PLACEHOLDER_STRING,
                                            ProcessFeed.PROCESSED_BY_DEFAULT ) ;
                                            // flle processed successfully move it to processed directry
                if ( !String.IsNullOrEmpty(FeedTableLoadAction.ProcessedFolderPath))
                    FileHelper.MoveProcessedFile(fileDataPath, FeedTableLoadAction.ProcessedFolderPath, null, "txt", Log);
            }
            else
            {
                Log.InfoFormat("WARNING. No data to load" );
            }
            return true;
        }

        protected bool LoadDelimitedFile(DateTime? effDate, string filePath)
        {
            FeedStagingDB.ProcessFile ( filePath,
                                        null ,           // insertStatements is  null. For the time being this method is used with BULK INSERT only  
                                        FeedTableLoadAction.HeaderRowReplacement,
                                        FeedTableLoadAction.FieldDelimiters,
                                        FeedTableLoadAction.RowDelimiters,
                                        effDate,
                                        FeedTableLoadAction.DateKeyColumn,
                                        FeedTableLoadAction.SchemaName,
                                        FeedTableLoadAction.TableName,
                                        FeedTableLoadAction.TruncateExisting,
                                        FeedTableLoadAction.UseSqlBulkLoad,
                                        FeedTableLoadAction.PostEffDateLoadAction.Count == 0 ? null : String. Join(" ", FeedTableLoadAction.PostEffDateLoadAction.ToArray() ),
                                        ProcessFeed.EFFECTIVE_DATE_PLACEHOLDER_STRING,
                                        ProcessFeed.PROCESSED_BY_DEFAULT ) ;
                                        // flle processed successfully move it to processed directry
            if ( !String.IsNullOrEmpty(FeedTableLoadAction.ProcessedFolderPath))
                FileHelper.MoveProcessedFile(filePath, FeedTableLoadAction.ProcessedFolderPath, null, "txt", Log);
            return true;
        }

        protected TableLoadAction FeedTableLoadAction
        {
            get
            {
                return this.feedTableLoadAction;
            }
        }

        protected StagingDatabase FeedStagingDB
        {
            get
            {
                return this.feedStagingDB;
            }
        }

        protected ILog Log
        {
            get
            {
                return this.log;
            }
        }

        public abstract bool Process( );
        
        public abstract bool ProcessEffectiveDate(DateTime? effDate);

        private static void DoFeedLoad(Object data)
        {
            var loadState = (LoadState)data;
            string mailMessage = "";

            try
            {
                IProcessFeed feedProcessor = FeedProcessorFactory.Create(loadState.ThisFeed.FeedTableLoadAction, loadState.ThisFeed.FeedStagingDB ) ;
                feedProcessor.ProcessEffectiveDate( loadState.DateToProcessInThread ); // This is a call to class that supports processing on multiple connections
            }
            catch (Exception e)
            {
                loadState.ErrorMessage = String.Format("Thread failed while loading feed for {0}.\n" + mailMessage, loadState.DateToProcessInThread.ToString("yyyy-MM-dd"));
                loadState.Log.Error(loadState.ErrorMessage, e);
            }
            finally
            {
                loadState.ManualEvent.Set( );
            }
        }

        internal class LoadState
        {
            public LoadState(ProcessFeed processFeed, DateTime dateToProcessInThread, int threadSeqNum, ManualResetEvent manualEvent, ILog log)
            {
                this.ErrorMessage = "";
                this.ThisFeed = processFeed;
                this.DateToProcessInThread = dateToProcessInThread;
                this.ThreadSeqNum = threadSeqNum;
                this.ManualEvent = manualEvent;
                this.Log = log;
            }

            public ILog Log
            {
                get;
                set;
            }

            public ManualResetEvent ManualEvent
            {
                get;
                set;
            }

            public ProcessFeed ThisFeed
            {
                get;
                set;
            }

            public DateTime DateToProcessInThread
            {
                get;
                set;
            }

            public string ErrorMessage
            {
                get;
                set;
            }

            public int ThreadSeqNum
            {
                get;
                set;
            }

        }
    }
}