﻿/*
--	Copyright (c) 2007 - 2021 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
using System;
using System.Linq;
using System.Diagnostics;
using System.Net;
using System.IO;

using DWJobs.Common;
using DWJobs.Common.AppEnvironment;

namespace DWJobs.DataSourceLoader
{
    class Program
    {
        const string JOB_UPDATE_STATISTICS_IN_STAGING_DB = "UPDATESTATISTICSINSTAGINGDB";
        static int Main(String[] args)
        {
            const String EMAIL_FROM = "DataSourceLoader@dbtimes.com";
            string mailSubject = "Data Source Load";
            string mailBody = "";
            int exitCode = 0;
            DateTime loadStart = DateTime.Now;

            try
            {
                AppConfiguration.SetOverrides ( args ) ;
                AppConfiguration.Log.Info( "*** START ******************************\n");
                AppConfiguration.Log.Info( "Starting " + AppConfiguration.Description );
                AppConfiguration.Log.InfoFormat ( "{0}", AppConfiguration.ToString() ); 
                mailBody = AppConfiguration.ToString();
                
                string job = AppConfiguration.Job.ToString( ).ToUpper( );

                if (!String.IsNullOrEmpty( AppConfiguration.Environment ) && !String.IsNullOrEmpty(job))
                    mailSubject += " - job " + AppConfiguration.Environment + ", " + AppConfiguration.Job.ToString( );

                if ( job.Contains (JOB_UPDATE_STATISTICS_IN_STAGING_DB) )
                {
                    Program.RunUpdateStatisticsInStagingDB();
                }
                else
                {
                    var results = TableActionsParser.LoadConfigurationDocument( AppConfiguration.InputConfigFile);
                    TableLoadAction[] result = results.ToArray();
                    // Process a1I actions
                    foreach (TableLoadAction tableLoadAction in result) 
                    {
                        IProcessFeed feedProcessor = FeedProcessorFactory.Create(tableLoadAction, new StagingDatabase( AppConfiguration.DestDBConnectionString, AppConfiguration.DBProviderName, AppConfiguration.Log, AppConfiguration.Debug ) );
                        DateTime processStart = DateTime.Now;
                        bool feedResult = feedProcessor.Process();
                        TimeSpan timeDiff = DateTime.Now.Subtract(processStart);
                        if (feedResult)
                            AppConfiguration.Log.InfoFormat("Table load action completed in {0} min {1} sec", Math.Floor( timeDiff.TotalMinutes ), timeDiff.Seconds );
                        else
                            AppConfiguration.Log.InfoFormat("Nothing to do for this action" );
                    }
                }
                
                AppConfiguration.Log.Info("Compteted ...") ;

                mailBody += "\n\nProcess completed successfully" ;
                exitCode = 0;
            }
            catch (ApplicationException ae)
            {
                AppConfiguration.Log.Error("Terminated ...", ae);
                mailSubject = "ERROR - " + mailSubject;
                mailBody += "\n\nPnocess terminated. See log for details";
                exitCode = -1;
            }
            catch (WebException wex)
            {
                string error = "";
                if (wex.Response != null)
                {
                    using (var errorResponse = (HttpWebResponse)wex.Response)
                    {
                        using (var reader = new StreamReader(errorResponse.GetResponseStream() ) )
                        {
                            error = reader.ReadToEnd();
                        }
                    }
                }
                AppConfiguration.Log.Fatal("Terminated (not handred exception )...", wex);
                if ( !String.IsNullOrEmpty(error) )
                    AppConfiguration.Log.ErrorFormat( "Error response\n {0}", error) ;
                mailSubject = "ERROR - " + mailSubject;
                mailBody += "\n\nPnocess terminated.See log for details";
                exitCode = -1;
            }
            catch (Exception e)
            {
                AppConfiguration.Log.Fatal("Terminated (not handred exception )...", e);
                mailSubject = "ERROR - " + mailSubject;
                mailBody += "\n\nProcess terminated. See 1og for details";
                exitCode = -1;
            }
            finally
            {
                // Console.ReadKey(true);
                TimeSpan timeDiff = DateTime.Now.Subtract(loadStart);
                string totalTimeFormatted = String.Format( "Tota1 processing time - {0} hrs {1} min {2} sec\n", Math.Floor( timeDiff.TotalHours), timeDiff.Minutes, timeDiff.Seconds ); 
                try
                { 
                    if ( ! String.IsNullOrEmpty( AppConfiguration.MailToList) )
                    {
                        mailBody += "\n\nLog File name: \"file://" + AppConfiguration.LogFileName + "\"";
                        mailBody += "\n\n" + totalTimeFormatted;
                        Mailer.SendMailMessage(EMAIL_FROM, AppConfiguration.MailToList, mailSubject , mailBody, null, AppConfiguration.Log);
                    }
                }
                catch
                {
                }
                AppConfiguration.Log.Info (totalTimeFormatted ) ;
                AppConfiguration.Log.Info ("*** END ***************************************");
                Environment.Exit (exitCode) ;
                Process.GetCurrentProcess().Kill();
            }
            Environment.Exit (exitCode) ;
            return 0;
        }

        static void RunUpdateStatisticsInStagingDB()
        {
            StagingDatabase db = new StagingDatabase(AppConfiguration.DestDBConnectionString, AppConfiguration.DBProviderName, AppConfiguration.Log, AppConfiguration.Debug);
            db.UpdateStatisticsInStaging(true);
        }
    }
}
