﻿/*
--	Copyright (c) 2007 - 2021 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
using System;
using System.IO;

using log4net;

using DWJobs.Common;

namespace DWJobs.DataSourceLoader
{
    partial class SqlServiceLoader : ProcessFeed
    {
        private bool isNonInitialFullLoad;
        public SqlServiceLoader(TableLoadAction tableLoadAction, StagingDatabase db, bool isNonInitialFullLoad, ILog log) 
            : base( tableLoadAction, db, log )
        {
            this.isNonInitialFullLoad = isNonInitialFullLoad;
        }

        public override bool Process()
        {
            Log.InfoFormat("Executing sql: {0}", FeedTableLoadAction.Sql );
            DateTime processStart = DateTime.Now;

            if (FeedTableLoadAction.PreLoadAction.Count != 0)
                FeedStagingDB.DoPostLoadProcessing( null,
                                                    null,
                                                    String.Join(" ", FeedTableLoadAction.PreLoadAction.ToArray()),
                                                    ProcessFeed.FIRST_EFFECTIVE_DATE_PLACEHOLDER_STRING,
                                                    ProcessFeed.LAST_EFFECTTVE_DATE_PLACEHOLDER_STRING,
                                                    ProcessFeed.PROCESSED_BY_DEFAULT);

            string resultFileName = Path.Combine(FeedTableLoadAction.DropFolderPath, FeedTableLoadAction.TableName + "_sql_" + Guid.NewGuid().ToString() + ".txt" );
            
            string sql = FeedTableLoadAction.Sql;
            if (FeedTableLoadAction.Sql.Contains( "<" + ProcessFeed.EFFECTIVE_DATE_PLACEHOLDER_STRING + ">") )
            {
                DateTime? nextEffectiveDate = null;
                if (this.isNonInitialFullLoad)
                    nextEffectiveDate = new DateTime?(new DateTime(1900, 01, 01)); // default to 19000101
                else
                    FeedStagingDB.GetNextDateToLoad(    FeedTableLoadAction.DateKeyColumn,
                                                        FeedTableLoadAction.SchemaName,
                                                        FeedTableLoadAction.TableName,
                                                        out nextEffectiveDate);
                sql = sql.Replace("<" + ProcessFeed.EFFECTIVE_DATE_PLACEHOLDER_STRING + ">", nextEffectiveDate.Value.ToString("yyyy-MM-dd HH:mm:ss"));
            }

            if (FeedTableLoadAction.TruncateExisting)
                FeedStagingDB.TruncateTable(    FeedTableLoadAction.SchemaName,
                                                FeedTableLoadAction.TableName);

            Log.InfoFormat("Executing sql: {0}", sql);


            if (FeedTableLoadAction.UseDirectSql)
            {
                FeedStagingDB.ExecuteStoredProcedure(FeedTableLoadAction.ConnectionString,
                                                     sql,
                                                     FeedTableLoadAction.IsStoredProcedure,
                                                     FeedTableLoadAction.ProcedureParameters ) ;

            }
            else
            {
                if (FeedTableLoadAction.UseSqlBulkLoad)
                {
                    Log.InfoFormat("Creating and loading delimited file");

                    FeedStagingDB.ExecuteStoredProcedureSaveResultToDelimitedFile
                                                        (   resultFileName,
                                                            FeedTableLoadAction.ConnectionString,
                                                            sql,
                                                            FeedTableLoadAction.IsStoredProcedure,
                                                            FeedTableLoadAction.ProcedureParameters,
                                                            FeedTableLoadAction.FieldDelimiters,
                                                            FeedTableLoadAction.RowDelimiters 
                                                         ) ;

                    LoadDelimitedFile(null, resultFileName);
                }
                else
                {
                    Log.InfoFormat("Creating and loading csv file");

                    FeedStagingDB.ExecuteStoredProcedureSaveResultToCsv ( resultFileName,
                                                                            FeedTableLoadAction.ConnectionString,
                                                                            sql,
                                                                            FeedTableLoadAction.IsStoredProcedure,
                                                                            FeedTableLoadAction.ProcedureParameters ) ;
                    // csv parser expect rows separated by new line
                    // when the result is dumped to the csv file the file will have row headers - 
                    // use them if the option to use column from the destination table is not set
                    LoadCsvFile(null, resultFileName );

                    // move processed file to processed directry
                    FileHelper.MoveProcessedFile(resultFileName, FeedTableLoadAction.ProcessedFolderPath, null, Log);
                }

            }




            if (FeedTableLoadAction.PostLoadAction.Count != 0 )
                FeedStagingDB.DoPostLoadProcessing( null ,
                                                    null ,
                                                    String.Join(" ", FeedTableLoadAction.PostLoadAction.ToArray()),
                                                    ProcessFeed.FIRST_EFFECTIVE_DATE_PLACEHOLDER_STRING,
                                                    ProcessFeed.LAST_EFFECTTVE_DATE_PLACEHOLDER_STRING,
                                                    ProcessFeed.PROCESSED_BY_DEFAULT);

            TimeSpan processTimeDiff = DateTime.Now.Subtract ( processStart) ;
            
            Log.InfoFormat("Process completed in {0} hours", processTimeDiff.TotalHours.ToString() );
            return true;
        }

        public override bool ProcessEffectiveDate(DateTime? effDate)
        {
            return true;
        }
    }
}
