﻿/*
--	Copyright (c) 2007 - 2021 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml.Linq;

namespace DWJobs.DataSourceLoader
{
    public static class TableActionsParser
    {
        public static IEnumerable<TableLoadAction> LoadConfigurationDocument(string strFilePath)
        {
            var doc = XDocument.Load(strFilePath);
            var ns = doc.Root.Name.Namespace;

            var root = doc.Root.Descendants(ns + "TableActions");
            var elements = root.Elements(ns + "TableAction");
            var results = elements.Select(xel => ElementToTableLoadAction(ns, xel));

            return results.Where(tla => tla.Active);
        }

        static private TableLoadAction ElementToTableLoadAction(XNamespace ns, XElement xel)
        {
            var result = new TableLoadAction()
            {
                Active = Boolean.Parse(xel.Element(ns + "Active").Attribute("value").Value),
                PreLoadAction = new List<string>(),
                DateKeyColumn = xel.Element(ns + "DateKeyColumn") != null ?
                    xel.Element(ns + "DateKeyColumn").Attribute("value").Value : String.Empty,
                SchemaName = xel.Element(ns + "SchemaName").Attribute("value").Value,
                TableName = xel.Element(ns + "TableName").Attribute("value").Value,
                FieldTransformations = null,
                IgnoredInputColumns = new List<string>(),
                IncludedInputColumns = new List<string>(),
                PostEffDateLoadAction = new List<string>(),
                PostLoadAction = new List<string>(),
                ProcedureParameters = null,
                ReplaceExisting = false,
                TruncateExisting = false,
                SourceHasAllTargetTableColumnsInTheSameOrder = false,
                IsStoredProcedure = false,
                HeaderLinesToSkip = 0,
                UseSqlBulkLoad = false,
                UseDirectSql = false,
                PrependRowNumberColumn = false,
                DataSourceType = TableLoadAction.LoaderDataSourceType.FileDataSource,
                NumberOfParallelProcesses = 0,
                FieldDelimiters = "|",
                RowDelimiters = "\r\n"
            };

            if (xel.Element(ns + "PreLoadAction") != null)
            {
                var strExecutesSQL = xel.Element(ns + "PreLoadAction").Elements(ns + "ExecuteSQL").Select(
                    xelExecutesSQL => xelExecutesSQL.Attribute("value").Value);

                result.PreLoadAction.AddRange(strExecutesSQL);
            }

            if (xel.Element(ns + "StartDate") != null)
                result.StartDate = xel.Element(ns + "StartDate").Attribute("value").Value;
            if (xel.Element(ns + "EndDate") != null)
                result.EndDate = xel.Element(ns + "EndDate").Attribute("value").Value;

            if (xel.Element(ns + "EffectiveDateField" ) != null)
                result.EffectiveDateField = xel.Element(ns + "EffectiveDateField").Attribute("value").Value;

            if (xel.Element(ns + "FieldDelimiters") != null)
                result.FieldDelimiters = xel.Element(ns + "FieldDelimiters").Attribute("value").Value;
            if (xel.Element(ns + "RowDelimiters") != null)
                result.RowDelimiters = xel.Element(ns + "RowDelimiters").Attribute("value").Value;
            if (xel.Element(ns + "CsvFile") != null)
                result.CsvFile = Boolean.Parse(xel.Element(ns + "CsvFile").Attribute("value").Value);
            if (xel.Element(ns + "HasHeaderRow") != null)
                result.HasHeaderRow = Boolean.Parse(xel.Element(ns + "HasHeaderRow").Attribute("value").Value);

            if (xel.Element(ns + "HeaderLinesToSkip") != null)
                result.HeaderLinesToSkip = Int32.Parse(xel.Element(ns + "HeaderLinesToSkip").Attribute("value").Value);
            else if (result.CsvFile)
                result.HeaderLinesToSkip = 1; // for csv file this is default


            if (xel.Element(ns + "DropFolderPath") != null)
                result.DropFolderPath = xel.Element(ns + "DropFolderPath").Attribute("value").Value;
            if (xel.Element(ns + "ProcessedFolderPath") != null)
                result.ProcessedFolderPath = xel.Element(ns + "ProcessedFolderPath").Attribute("value").Value;

            if (xel.Element(ns + "SsasConnectionString") != null)
                result.SsasConnectionString = xel.Element(ns + "SsasConnectionString").Attribute("value").Value;
            if (xel.Element(ns + "NumberOfParallelProcesses") != null)
                result.NumberOfParallelProcesses = Int32.Parse(xel.Element(ns + "NumberOfParallelProcesses").Attribute("value").Value);



            // the destination table action might be truncate existing or replace existing
            if (xel.Element(ns + "ReplaceExisting") != null)
                result.ReplaceExisting = Boolean.Parse(xel.Element(ns + "ReplaceExisting").Attribute("value").Value);
            else if (xel.Element(ns + "TruncateExisting") != null)
                result.TruncateExisting = Boolean.Parse(xel.Element(ns + "TruncateExisting").Attribute("value").Value);

            // Field transformations
            if (xel.Element(ns + "FieldTransformations") != null)
            {
                var transformAttributes = xel.Element(ns + "FieldTransformations").Elements(ns + "FieldTransformation").Select(
                    fieldTransformation => new FieldTransformation( fieldTransformation.Attribute("FieldName").Value,
                                                                    fieldTransformation.Attribute("TransformType").Value,
                                                                    fieldTransformation.Attribute("TransformFrom").Value,
                                                                    fieldTransformation.Attribute("TransformTo").Value ) );

                result.FieldTransformations = new List<FieldTransformation>(transformAttributes);
            }

            // there may be input columns we need to ignore
            if (xel.Element(ns + "IgnoredInputColumns") != null)
            {
                var strIgnored = xel.Element(ns + "IgnoredInputColumns").Elements(ns + "IgnoredColumn").Select(
                    xelIgnored => xelIgnored.Attribute("value").Value);

                result.IgnoredInputColumns.AddRange(strIgnored);
            }
            if (xel.Element(ns + "IncludedInputColumns") != null)
            {
                var strIncluded = xel.Element(ns + "IncludedInputColumns").Elements(ns + "IncludedColumn").Select(
                    xelIncluded => xelIncluded.Attribute("value").Value);

                result.IncludedInputColumns.AddRange(strIncluded);
            }
            if (xel.Element(ns + "PostEffDateLoadAction") != null)
            {
                var strExecuteEffDatesSQL = xel.Element(ns + "PostEffDateLoadAction").Elements(ns + "ExecuteSQL").Select(
                    xelExecuteEffDatesSQL => xelExecuteEffDatesSQL.Attribute("value").Value);

                result.PostEffDateLoadAction.AddRange(strExecuteEffDatesSQL);
            }
            if (xel.Element(ns + "PostLoadAction") != null)
            {
                var strExecutesSQL = xel.Element(ns + "PostLoadAction").Elements(ns + "ExecuteSQL").Select(
                    xelExecutesSQL => xelExecutesSQL.Attribute("value").Value);

                result.PostLoadAction.AddRange(strExecutesSQL);
            }


            // the data source is either a flat file or sql data source
            var dsElement = xel.Element(ns + "DataSource");

            if (dsElement.Element(ns + "FileDataSource") != null)
            {
                result.DataSourceType = TableLoadAction.LoaderDataSourceType.FileDataSource;

                var fsElement = dsElement.Element(ns + "FileDataSource");

                result.CsvFile = Boolean.Parse(fsElement.Element(ns + "CsvFile").Attribute("value").Value);
                result.FileNameTemplate = fsElement.Element(ns + "FileNameTemplate").Attribute("value").Value;
                result.InputFolderPath = fsElement.Element(ns + "InputFolderPath").Attribute("value").Value;
                result.HasHeaderRow = Boolean.Parse(fsElement.Element(ns + "HasHeaderRow").Attribute("value").Value);

                // optional arguments
                if (fsElement.Element(ns + "FieldDelimiters") != null)
                    result.FieldDelimiters = fsElement.Element(ns + "FieldDelimiters").Attribute("value").Value;
                if (fsElement.Element(ns + "RowDelimiters") != null)
                    result.RowDelimiters = fsElement.Element(ns + "RowDelimiters").Attribute("value").Value;

                if (fsElement.Element(ns + "HeaderLinesToSkip") != null)
                    result.HeaderLinesToSkip = Int32.Parse(fsElement.Element(ns + "HeaderLinesToSkip").Attribute("value").Value);
                else if (result.CsvFile)
                    result.HeaderLinesToSkip = 1; // for csv file this is default
            }
            else if (dsElement.Element(ns + "SqlDataSource") != null)
            {
                result.DataSourceType = TableLoadAction.LoaderDataSourceType.SqlDataSource;

                var sqlElement = dsElement.Element(ns + "SqlDataSource");

                if (sqlElement.Element(ns + "ConnectionString") != null)
                    result.ConnectionString = sqlElement.Element(ns + "ConnectionString").Attribute("value").Value;
                if (sqlElement.Element(ns + "Sql") != null)
                    result.Sql = sqlElement.Element(ns + "Sql").Attribute("value").Value;
                if (sqlElement.Element(ns + "IsStoredProcedure") != null)
                    result.IsStoredProcedure = Boolean.Parse(sqlElement.Element(ns + "IsStoredProcedure").Attribute("value").Value);

                if (sqlElement.Element(ns + "ProcParameters") != null)
                {
                    var parameters = sqlElement.Element(ns + "ProcParameters").Elements(ns + "ProcParameter").Select(
                        procParam => new ProcParameter( procParam.Attribute("Name").Value, 
                                                        procParam.Attribute("Type").Value,
                                                        procParam.Attribute("Value").Value ) );

                    result.ProcedureParameters = new List<ProcParameter>( parameters );
                }
            
            }
            else if (dsElement.Element(ns + "SourceLoaderDataSource") != null)
            {
                result.DataSourceType = TableLoadAction.LoaderDataSourceType.SourceLoaderDataSource;
            }

            // this is not common.  Is there are replacement row for the header?
            if (xel.Element(ns + "HeaderRowReplacement") != null)
                result.HeaderRowReplacement = xel.Element(ns + "HeaderRowReplacement").Attribute("value").Value;

            if (xel.Element(ns + "SourceHasAllTargetTableColumnsInTheSameOrder") != null)
                result.SourceHasAllTargetTableColumnsInTheSameOrder = Boolean.Parse(xel.Element(ns + "SourceHasAllTargetTableColumnsInTheSameOrder").Attribute("value").Value);

            if (xel.Element(ns + "UseSqlBulkLoad") != null)
                result.UseSqlBulkLoad = Boolean.Parse(xel.Element(ns + "UseSqlBulkLoad").Attribute("value").Value);
            if (xel.Element(ns + "UseDirectSql") != null)
                result.UseDirectSql = Boolean.Parse(xel.Element(ns + "UseDirectSql").Attribute("value").Value);
            if (xel.Element(ns + "PrependRowNumberColumn") != null)
                result.PrependRowNumberColumn = Boolean.Parse(xel.Element(ns + "PrependRowNumberColumn").Attribute("value").Value);

            return result;
        }
    }
}