﻿/*
--	Copyright (c) 2007 - 2021 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
using System;
using System.Threading;
using Microsoft.AnalysisServices;

using log4net;

namespace DWJobs.ETLRunner
{
    public class AnalysisServicesProcessing
    {
        private readonly string connectionString;
        private readonly string providerName;
        private readonly string ssasServerName;
        private readonly string ssasDBName;
        private readonly string ssasAccessConfigurationName;
        private readonly int    ssasMaxParallelDefault;
        private readonly string debug;

        private ILog   log;

        public AnalysisServicesProcessing(string connectionString, string providerName, string ssasServerName, string ssasDBName, string ssasAccessConfigurationName, int ssasMaxParallel, string debug, ILog log)
        {
            this.connectionString = connectionString;
            this.providerName = providerName;
            this.ssasServerName = ssasServerName;
            this.ssasDBName = ssasDBName;
            this.ssasAccessConfigurationName = ssasAccessConfigurationName;
            this.ssasMaxParallelDefault = ssasMaxParallel;
            this.debug = debug;

            this.log = log;
        }

        public AnalysisServicesProcessing(string connectionString, string providerName, string ssasServerName, string ssasDBName, string debug, ILog log)
            : this (  connectionString, providerName, ssasServerName, ssasDBName, null, -1, debug, log )
        {
        }

        public string LoadAnalysisServices(string uniqueSessionId, bool isSSASFullLoad, DateTime? ssasStartDateToLoad, bool isSSASLoadCalculations )
        {
            string message = "";
            if (isSSASFullLoad)
                message = DoFullAnalysisServicesLoad( ssasStartDateToLoad);
            else
                message = DoIncrementalAnalysisServicesLoad( );

            if (isSSASLoadCalculations)
                DoLoadCalculations( );

            return message;
        }

        public string SynchronizeAnalysisServicesDatabase(string sourceSSASServerName, string sourceSSASDBName )
        {
            string message = "";

            AnalysisServicesSourceDatabase dbAnalysisServices = new AnalysisServicesSourceDatabase( this.connectionString, this.providerName, this.ssasServerName, this.ssasDBName, this.debug, this.log );
            string xmlaScript = null;
            XmlaResultCollection xmlaResults = null;
            Server olapServer = new Server();
            olapServer.Connect (this.ssasServerName) ;
            // Executing DB Synchronize XMLA Script
            xmlaScript = dbAnalysisServices.GenerateSSASSynchronizeXMLAScript(sourceSSASServerName, sourceSSASDBName );

            this.log.Info("Executing Synchronize XMLA Script");
            this. log. Info(xmlaScript. ToString( ) );
            
            xmlaResults = olapServer.Execute(xmlaScript);
            ProcessXMLAResults (xmlaResults ) ;
            olapServer. Disconnect ( ) ;
            return message;
        }

        private string DoFullAnalysisServicesLoad( DateTime? ssasStartDateToLoad)
        {
            string message = "";

            AnalysisServicesSourceDatabase dbAnalysisServices = new AnalysisServicesSourceDatabase(this.connectionString, this.providerName, this.ssasServerName, this.ssasDBName, this.debug, this.log);

            // Do Pre Processing
            bool noData;
            message += dbAnalysisServices.DoSSASDBLoadPreProcessing(true, out noData, ssasStartDateToLoad);
            if (noData)
                return message;

            string xmlaScript = null;
            XmlaResultCollection xmlaResults = null;
            Server olapServer = new Server();
            olapServer.Connect(this.ssasServerName);

            // Executing DB Delete XMLA Script
            xmlaScript = dbAnalysisServices.GenerateSSASDBDeleteXMLAScript();
            this.log.Info("Executing DB Delete XMLA Script");
            this.log.Info(xmlaScript.ToString());
            xmlaResults = olapServer.Execute(xmlaScript);
            ProcessXMLAResults(xmlaResults);

            // Executing DB Create XMLA Script
            xmlaScript = dbAnalysisServices.GenerateSSASDBCreateXMLAScript();
            this.log.Info("Executing DB Create XMLA Script");
            //  this.log.Info(xmlaScript.ToString());     do not loag the DB create under normal conditions
            xmlaResults = olapServer.Execute(xmlaScript);
            ProcessXMLAResults(xmlaResults);

            if (!String.IsNullOrEmpty(this.ssasAccessConfigurationName ))
            {
                xmlaScript = dbAnalysisServices.GenerateSSASPermissionXMLAScript( this.ssasAccessConfigurationName );
                this.log.Info("Executing DB Permission XMLA Script");
                xmlaResults = olapServer.Execute(xmlaScript);
                ProcessXMLAResults(xmlaResults);
                this.log.Info("Finished executing permission to the cube");
                this.log.Info("");
            }

            // Executing DB Load XMLA Script
            int ssasMaxParallel = this.ssasMaxParallelDefault;
            
            do
            {
                try
                {
                    xmlaScript = dbAnalysisServices.GenerateSSASDBLoadXMLAScript(ssasMaxParallel);
                    this.log.InfoFormat("Executing DB Load XMLA Script on {0} connections", ssasMaxParallel);
                    this.log.Info(xmlaScript.ToString());
                    xmlaResults = olapServer.Execute(xmlaScript);
                    ProcessXMLAResults(xmlaResults);
                    ssasMaxParallel = -1;     // this will exit from the loop
                }
                catch
                {
                    if (ssasMaxParallel == 0)
                        ssasMaxParallel = 5;
                    else if (ssasMaxParallel >= 5)
                        ssasMaxParallel = 3;
                    else if (ssasMaxParallel >= 3)
                        ssasMaxParallel = 1;
                    else
                        throw;

                    try
                    {
                        olapServer.Disconnect();
                    }
                    catch
                    {
                    }

                    Thread.Sleep(10000); // give connection some time to start

                    olapServer = new Server();
                    olapServer.Connect(this.ssasServerName);
                }
            } while( ssasMaxParallel >= 0 );

            olapServer.Disconnect();

            // Do Post Processing
            message += dbAnalysisServices.DoSSASDBLoadPostProcessing();

            return message;
        }

        private string DoIncrementalAnalysisServicesLoad( )
        {
            string message = "";

            AnalysisServicesSourceDatabase dbAnalysisServices = new AnalysisServicesSourceDatabase(this.connectionString, this.providerName, this.ssasServerName, this.ssasDBName, this.debug, this.log);

            // Do Pre Processing
            bool noData;
            message += dbAnalysisServices.DoSSASDBLoadPreProcessing(false, out noData, null);
            if (noData)
                return message;

            string xmlaScript = null;
            XmlaResultCollection xmlaResults = null;
            Server olapServer = new Server();
            olapServer.Connect(this.ssasServerName);

            // Executing Delete Partitions XMLA Script
            xmlaScript = dbAnalysisServices.GenerateSSASPartitionsDeleteCreateXMLAScript();
            this.log.Info("Executing Delete Create Partitions XMLA Script");
            this.log.Info(xmlaScript.ToString());
            xmlaResults = olapServer.Execute(xmlaScript);
            ProcessXMLAResults(xmlaResults);

            if (!String.IsNullOrEmpty(this.ssasAccessConfigurationName))
            {
                xmlaScript = dbAnalysisServices.GenerateSSASPermissionXMLAScript( this.ssasAccessConfigurationName);
                this.log.Info("Executing DB Permission XMLA Script");
                xmlaResults = olapServer.Execute(xmlaScript);
                ProcessXMLAResults(xmlaResults);
                this.log.Info("Finished executing permission to the cube");
                this.log.Info("");
            }

            // Executing DB Load XMLA Script
            int ssasMaxParallel = this.ssasMaxParallelDefault;

            do
            {
                try
                {
                    xmlaScript = dbAnalysisServices.GenerateSSASDBLoadXMLAScript(ssasMaxParallel);
                    this.log.InfoFormat("Executing DB Load XMLA Script on {0} connections", ssasMaxParallel);
                    this.log.Info(xmlaScript.ToString());
                    xmlaResults = olapServer.Execute(xmlaScript);
                    ProcessXMLAResults(xmlaResults);
                    ssasMaxParallel = -1;     // this will exit from the loop
                }
                catch
                {
                    if (ssasMaxParallel == 0)
                        ssasMaxParallel = 5;
                    else if (ssasMaxParallel >= 5)
                        ssasMaxParallel = 3;
                    else if (ssasMaxParallel >= 3)
                        ssasMaxParallel = 1;
                    else
                        throw;

                    try
                    {
                        olapServer.Disconnect();
                    }
                    catch
                    {
                    }

                    Thread.Sleep(10000); // give connection some time to start

                    olapServer = new Server();
                    olapServer.Connect(this.ssasServerName);
                }
            } while (ssasMaxParallel >= 0);

            olapServer.Disconnect();

            // Do Post Processing
            message += dbAnalysisServices.DoSSASDBLoadPostProcessing();

            return message;
        }

        private void DoLoadCalculations( )
        {
            AnalysisServicesSourceDatabase dbAnalysisServices = new AnalysisServicesSourceDatabase(this.connectionString, this.providerName, this.ssasServerName, this.ssasDBName, this.debug, this.log);

            string xmlaScript = null;
            XmlaResultCollection xmlaResults = null;
            Server olapServer = new Server();
            olapServer.Connect(this.ssasServerName);

            // Executing SSAS Cube Calculations XMLA Script
            xmlaScript = dbAnalysisServices.GenerateSSASCubeCalculationsXMLAScript( );
            this.log.Info("Executing Calculations XMLA Script");
            this.log.Info(xmlaScript.ToString());
            xmlaResults = olapServer.Execute(xmlaScript);
            ProcessXMLAResults(xmlaResults);

            olapServer.Disconnect();
        }

        private void ProcessXMLAResults(XmlaResultCollection xmlaResults)
        {
            bool error = false;

            foreach (XmlaResult xmlaResult in xmlaResults)
                foreach (XmlaMessage xmlaMessage in xmlaResult.Messages)
                {
                    if (xmlaMessage.GetType().Name == "XmlaError")
                    {
                        this.log.Error("Error executing XMLA Script");
                        this.log.Error(xmlaMessage.Description);
                        error = true;
                    }
                    else
                        this.log.Info(xmlaMessage.Description);
                }

            if (error)
                throw new ApplicationException("Error executing XMLA Script");

        }
    }
}
