﻿/*
--	Copyright (c) 2007 - 2021 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
using System;
using System.Data;
using System.Data.SqlClient;

using log4net;

namespace DWJobs.ETLRunner
{
    public class AnalysisServicesSourceDatabase
    {
        private string connectionString;
        private string providerName;
        private string ssasServerName;
        private string ssasDBName;
        private string debug;

        private ILog   log;

        private static string printText = null;

        private const int RESULT_SUCCESS = 0;

        public AnalysisServicesSourceDatabase(string connectionString, string providerName, string ssasServerName, string ssasDBName, string debug, ILog log)
        {
            this.connectionString = connectionString;
            this.providerName = providerName;
            this.ssasServerName = ssasServerName;
            this.ssasDBName = ssasDBName;
            this.debug = debug;

            this.log = log;
        }

        public string DoSSASDBLoadPreProcessing(bool fullLoad, out bool noData, DateTime? ssasStartDateToLoad)
        {
            int returnValue = -1;
            this.log.Info("Running SSAS DB Load Pre-Processing");

            string queryString = null;
            queryString = "[ssasload].[ETL SSAS DB - Do PreProcessing]";

            using (SqlConnection connection = new SqlConnection(this.connectionString))
            {
                connection.InfoMessage += new SqlInfoMessageEventHandler(AnalysisServicesSourceDatabase.InfoMessageHandler);

                SqlCommand command = new SqlCommand(queryString, connection);
                command.CommandType = System.Data.CommandType.StoredProcedure;
                command.CommandTimeout = 0;

                SqlParameter param = new SqlParameter("@SSASServerName", SqlDbType.VarChar);
                param.Value = this.ssasServerName;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@SSASDBName", SqlDbType.VarChar);
                param.Value = this.ssasDBName;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@FullLoad", SqlDbType.Bit);
                param.Value = fullLoad ? 1 : 0;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@StartDate", SqlDbType.Date);
                if (ssasStartDateToLoad == null)
                    param.Value = DBNull.Value;
                else
                    param.Value = ssasStartDateToLoad.Value;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@SQLServerConnectionString", SqlDbType.VarChar);
                param.Value = "Provider=SQLNCLI10.1;" + this.connectionString;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@NoData", SqlDbType.Bit);
                param.Direction = System.Data.ParameterDirection.Output;
                command.Parameters.Add(param);

                param = new SqlParameter("@Retval", SqlDbType.Int);
                param.Direction = System.Data.ParameterDirection.ReturnValue;
                command.Parameters.Add(param);

                param = new SqlParameter("@Debug", SqlDbType.Char);
                param.Value = this.debug;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                try
                {
                    connection.Open();
                    AnalysisServicesSourceDatabase.printText = "";
                    command.ExecuteScalar();
                    this.log.Info(AnalysisServicesSourceDatabase.printText);

                    string infoMessages = AnalysisServicesSourceDatabase.printText;

                    returnValue = Int32.Parse(command.Parameters["@Retval"].Value.ToString());

                    if (returnValue != RESULT_SUCCESS)
                        throw new ApplicationException(String.Format("Error running {0}", queryString));

                    this.log.InfoFormat("Finished running {0}", queryString);
                    this.log.InfoFormat("");

                    if (fullLoad)
                        noData = false;
                    else
                        noData = Boolean.Parse(command.Parameters["@NoData"].Value.ToString());

                    return infoMessages;
                }
                catch (SqlException e)
                {
                    this.log.Error("SQL Exception", e);
                    throw new ApplicationException(AnalysisServicesSourceDatabase.printText, e);
                }
                finally
                {
                    try
                    {
                        connection.Close();
                    }
                    catch
                    {
                    }
                }
            }
        }

        public string DoSSASDBLoadPostProcessing( )
        {
            int returnValue = -1;
            this.log.Info("Running SSAS DB Load Post-Processing");

            string queryString = null;
            queryString = "[ssasload].[ETL SSAS DB - Do PostProcessing]";

            using (SqlConnection connection = new SqlConnection(this.connectionString))
            {
                connection.InfoMessage += new SqlInfoMessageEventHandler(AnalysisServicesSourceDatabase.InfoMessageHandler);

                SqlCommand command = new SqlCommand(queryString, connection);
                command.CommandType = System.Data.CommandType.StoredProcedure;
                command.CommandTimeout = 0;

                SqlParameter param = new SqlParameter("@SSASServerName", SqlDbType.VarChar);
                param.Value = this.ssasServerName;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@SSASDBName", SqlDbType.VarChar);
                param.Value = this.ssasDBName;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@Retval", SqlDbType.Int);
                param.Direction = System.Data.ParameterDirection.ReturnValue;
                command.Parameters.Add(param);

                try
                {
                    connection.Open();
                    AnalysisServicesSourceDatabase.printText = "";
                    command.ExecuteScalar();
                    this.log.Info(AnalysisServicesSourceDatabase.printText);

                    string infoMessages = AnalysisServicesSourceDatabase.printText;

                    returnValue = Int32.Parse(command.Parameters["@Retval"].Value.ToString());

                    if (returnValue != RESULT_SUCCESS)
                        throw new ApplicationException(String.Format("Error running {0}", queryString));

                    this.log.InfoFormat("Finished running {0}", queryString);
                    this.log.InfoFormat("");

                    return infoMessages;
                }
                catch (SqlException e)
                {
                    this.log.Error("SQL Exception", e);
                    throw new ApplicationException(AnalysisServicesSourceDatabase.printText, e);
                }
                finally
                {
                    try
                    {
                        connection.Close();
                    }
                    catch
                    {
                    }
                }
            }
        }

        public string GenerateSSASDBDeleteXMLAScript( )
        {
            int returnValue = -1;
            this.log.Info("Generating SSAS DB Delete XMLA Script");

            string queryString = "[ssasload].[ETL SSAS DB - Generate Delete XMLA Script]";

            using (SqlConnection connection = new SqlConnection(this.connectionString))
            {
                connection.InfoMessage += new SqlInfoMessageEventHandler(AnalysisServicesSourceDatabase.InfoMessageHandler);

                SqlCommand command = new SqlCommand(queryString, connection);
                command.CommandType = System.Data.CommandType.StoredProcedure;
                command.CommandTimeout = 0;

                SqlParameter param = new SqlParameter("@SSASServerName", SqlDbType.VarChar);
                param.Value = this.ssasServerName;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@SSASDBName", SqlDbType.VarChar);
                param.Value = this.ssasDBName;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@Retval", SqlDbType.Int);
                param.Direction = System.Data.ParameterDirection.ReturnValue;
                command.Parameters.Add(param);

                try
                {
                    connection.Open();
                    AnalysisServicesSourceDatabase.printText = "";
                    command.ExecuteScalar();
                    string infoMessages = AnalysisServicesSourceDatabase.printText;

                    returnValue = Int32.Parse(command.Parameters["@Retval"].Value.ToString());

                    if (returnValue != RESULT_SUCCESS)
                        throw new ApplicationException(String.Format("Error running [ssasload].[ETL SSAS DB - Generate Delete XMLA Script]"));

                    this.log.InfoFormat("Finished running [ssasload].[ETL SSAS DB - Generate Delete XMLA Script]");
                    this.log.InfoFormat("");

                    return infoMessages;
                }
                catch (SqlException e)
                {
                    this.log.Error("SQL Exception", e);
                    throw new ApplicationException(AnalysisServicesSourceDatabase.printText, e);
                }
                finally
                {
                    try
                    {
                        connection.Close();
                    }
                    catch
                    {
                    }
                }
            }
        }

        public string GenerateSSASDBCreateXMLAScript( )
        {
            int returnValue = -1;
            this.log.Info("Generating SSAS DB Create XMLA Script");

            string queryString = "[ssasload].[ETL SSAS DB - Generate Create XMLA Script]";

            using (SqlConnection connection = new SqlConnection(this.connectionString))
            {
                connection.InfoMessage += new SqlInfoMessageEventHandler(AnalysisServicesSourceDatabase.InfoMessageHandler);

                SqlCommand command = new SqlCommand(queryString, connection);
                command.CommandType = System.Data.CommandType.StoredProcedure;
                command.CommandTimeout = 0;

                SqlParameter param = new SqlParameter("@SSASServerName", SqlDbType.VarChar);
                param.Value = this.ssasServerName;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@SSASDBName", SqlDbType.VarChar);
                param.Value = this.ssasDBName;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@Retval", SqlDbType.Int);
                param.Direction = System.Data.ParameterDirection.ReturnValue;
                command.Parameters.Add(param);

                try
                {
                    connection.Open();
                    AnalysisServicesSourceDatabase.printText = "";
                    command.ExecuteScalar();

                    string infoMessages = AnalysisServicesSourceDatabase.printText;

                    returnValue = Int32.Parse(command.Parameters["@Retval"].Value.ToString());

                    if (returnValue != RESULT_SUCCESS)
                        throw new ApplicationException(String.Format("Error running [ssasload].[ETL SSAS DB - Generate Create XMLA Script]"));

                    this.log.InfoFormat("Finished running [ssasload].[ETL SSAS DB - Generate Create XMLA Script]");
                    this.log.InfoFormat("");

                    return infoMessages;
                }
                catch (SqlException e)
                {
                    this.log.Error("SQL Exception", e);
                    throw new ApplicationException(AnalysisServicesSourceDatabase.printText, e);
                }
                finally
                {
                    try
                    {
                        connection.Close();
                    }
                    catch
                    {
                    }
                }
            }
        }

        public string GenerateSSASDBLoadXMLAScript( int maxParallel )
        {
            int returnValue = -1;
            this.log.Info("Generating SSAS DB Load XMLA Script");

            string queryString = "[ssasload].[ETL SSAS DB - Generate Load XMLA Script]";

            using (SqlConnection connection = new SqlConnection(this.connectionString))
            {
                connection.InfoMessage += new SqlInfoMessageEventHandler(AnalysisServicesSourceDatabase.InfoMessageHandler);

                SqlCommand command = new SqlCommand(queryString, connection);
                command.CommandType = System.Data.CommandType.StoredProcedure;
                command.CommandTimeout = 0;

                SqlParameter param = new SqlParameter("@SSASServerName", SqlDbType.VarChar);
                param.Value = this.ssasServerName;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@SSASDBName", SqlDbType.VarChar);
                param.Value = this.ssasDBName;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@MaxParallel", SqlDbType.Int);
                param.Value = maxParallel;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@Retval", SqlDbType.Int);
                param.Direction = System.Data.ParameterDirection.ReturnValue;
                command.Parameters.Add(param);

                try
                {
                    connection.Open();
                    AnalysisServicesSourceDatabase.printText = "";
                    command.ExecuteScalar();

                    string infoMessages = AnalysisServicesSourceDatabase.printText;

                    returnValue = Int32.Parse(command.Parameters["@Retval"].Value.ToString());

                    if (returnValue != RESULT_SUCCESS)
                        throw new ApplicationException(String.Format("Error ruuning [ssasload].[ETL SSAS DB - Generate Load XMLA Script]"));

                    this.log.InfoFormat("Finished running [ssasload].[ETL SSAS DB - Generate Load XMLA Script]");
                    this.log.InfoFormat("");

                    return infoMessages;
                }
                catch (SqlException e)
                {
                    this.log.Error("SQL Exception", e);
                    throw new ApplicationException(AnalysisServicesSourceDatabase.printText, e);
                }
                finally
                {
                    try
                    {
                        connection.Close();
                    }
                    catch
                    {
                    }
                }
            }
        }

        public string GenerateSSASPartitionsDeleteCreateXMLAScript( )
        {
            int returnValue = -1;
            this.log.Info("Generating SSAS Partitions Delete Create XMLA Script");

            string queryString = "[ssasload].[ETL SSAS DB - Generate Partitions Delete Create XMLA Script]";

            using (SqlConnection connection = new SqlConnection(this.connectionString))
            {
                connection.InfoMessage += new SqlInfoMessageEventHandler(AnalysisServicesSourceDatabase.InfoMessageHandler);

                SqlCommand command = new SqlCommand(queryString, connection);
                command.CommandType = System.Data.CommandType.StoredProcedure;
                command.CommandTimeout = 0;

                SqlParameter param = new SqlParameter("@SSASServerName", SqlDbType.VarChar);
                param.Value = this.ssasServerName;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@SSASDBName", SqlDbType.VarChar);
                param.Value = this.ssasDBName;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@Retval", SqlDbType.Int);
                param.Direction = System.Data.ParameterDirection.ReturnValue;
                command.Parameters.Add(param);

                try
                {
                    connection.Open();
                    AnalysisServicesSourceDatabase.printText = "";
                    command.ExecuteScalar();

                    string infoMessages = AnalysisServicesSourceDatabase.printText;

                    returnValue = Int32.Parse(command.Parameters["@Retval"].Value.ToString());

                    if (returnValue != RESULT_SUCCESS)
                        throw new ApplicationException(String.Format("Error running [ssasload].[ETL SSAS DB - Generate Partitions Delete Create XMLA Script]"));

                    this.log.InfoFormat("Finished running [ssasload].[ETL SSAS DB - Generate Partitions Delete Create XMLA Script]");
                    this.log.InfoFormat("");

                    return infoMessages;
                }
                catch (SqlException e)
                {
                    this.log.Error("SQL Exception", e);
                    throw new ApplicationException(AnalysisServicesSourceDatabase.printText, e);
                }
                finally
                {
                    try
                    {
                        connection.Close();
                    }
                    catch
                    {
                    }
                }
            }
        }

        public string GenerateSSASPermissionXMLAScript(string ssasAccessConfigurationName)
        {
            int returnValue = -1;
            this.log.Info("Generating SSAS Cube Permission XMLA Script");

            string queryString = "[ssasload].[ETL SSAS DB - Generate Permissions XMLA Script]";
            string xmlaOutput = string.Empty;

            using (SqlConnection connection = new SqlConnection(this.connectionString))
            {
                connection.InfoMessage += new SqlInfoMessageEventHandler(AnalysisServicesSourceDatabase.InfoMessageHandler);

                SqlCommand command = new SqlCommand(queryString, connection);
                command.CommandType = System.Data.CommandType.StoredProcedure;
                command.CommandTimeout = 0;

                SqlParameter param = new SqlParameter("@SSASServerName", SqlDbType.VarChar);
                param.Value = this.ssasServerName;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@SSASDBName", SqlDbType.VarChar);
                param.Value = this.ssasDBName;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@SSASAccessConfigurationName", SqlDbType.VarChar);
                param.Value = ssasAccessConfigurationName;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@Retval", SqlDbType.Int);
                param.Direction = System.Data.ParameterDirection.ReturnValue;
                command.Parameters.Add(param);

                try
                {
                    connection.Open();
                    AnalysisServicesSourceDatabase.printText = "";
                    command.ExecuteScalar();

                    string infoMessages = AnalysisServicesSourceDatabase.printText;

                    returnValue = Int32.Parse(command.Parameters["@Retval"].Value.ToString());

                    if (returnValue != RESULT_SUCCESS)
                        throw new ApplicationException(String.Format("Error running [ssasload].[ETL SSAS DB - Generate Permissions XMLA Script]"));

                    this.log.InfoFormat("Finished running [ssasload].[ETL SSAS DB - Generate Permissions XMLA Script]");
                    this.log.InfoFormat("");

                    return infoMessages;
                }
                catch (SqlException e)
                {
                    this.log.Error("SQL Exception", e);
                    throw new ApplicationException(AnalysisServicesSourceDatabase.printText, e);
                }
                finally
                {
                    try
                    {
                        connection.Close();
                    }
                    catch
                    {
                    }
                }
            }
        }

        public string GenerateSSASSynchronizeXMLAScript(string sourceSSASServerName, string sourceSSASDBName)
        {
            int returnValue = -1;
            this.log.Info("Generating SSAS Synchronize XMLA Script");

            string queryString = "[ssasload].[ETL SSAS DB - Generate Synchronize XMLA Script]";

            using (SqlConnection connection = new SqlConnection(this.connectionString))
            {
                connection.InfoMessage += new SqlInfoMessageEventHandler(AnalysisServicesSourceDatabase.InfoMessageHandler);

                SqlCommand command = new SqlCommand(queryString, connection);
                command.CommandType = System.Data.CommandType.StoredProcedure;
                command.CommandTimeout = 0;

                SqlParameter param = new SqlParameter("@SourceSSASServerName", SqlDbType.Char);
                param.Value = sourceSSASServerName;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@SourceSSASDBName", SqlDbType.Char);
                param.Value = sourceSSASDBName;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@Retval", SqlDbType.Int);
                param.Direction = System.Data.ParameterDirection.ReturnValue;
                command.Parameters.Add(param);

                try
                {
                    connection.Open();
                    AnalysisServicesSourceDatabase.printText = "";
                    command.ExecuteScalar();

                    string infoMessages = AnalysisServicesSourceDatabase.printText;

                    returnValue = Int32.Parse(command.Parameters["@Retval"].Value.ToString());

                    if (returnValue != RESULT_SUCCESS)
                        throw new ApplicationException(String.Format("Error running [ssasload].[ETL SSAS DB - Generate Synchronize XMLA Script]"));

                    this.log.InfoFormat("Finished running [ssasload].[ETL SSAS DB - Generate Synchronize XMLA Script]");
                    this.log.InfoFormat("");

                    return infoMessages;
                }
                catch (SqlException e)
                {
                    this.log.Error("SQL Exception", e);
                    throw new ApplicationException(AnalysisServicesSourceDatabase.printText, e);
                }
                finally
                {
                    try
                    {
                        connection.Close();
                    }
                    catch
                    {
                    }
                }
            }
        }

        public string GenerateSSASCubeCalculationsXMLAScript( )
        {
            int returnValue = -1;
            this.log.Info("Generating SSAS Cube Calculations XMLA Script");

            string queryString = "[ssasload].[ETL SSAS DB - Generate Cube Calculations XMLA Script]";

            using (SqlConnection connection = new SqlConnection(this.connectionString))
            {
                connection.InfoMessage += new SqlInfoMessageEventHandler(AnalysisServicesSourceDatabase.InfoMessageHandler);

                SqlCommand command = new SqlCommand(queryString, connection);
                command.CommandType = System.Data.CommandType.StoredProcedure;
                command.CommandTimeout = 0;

                SqlParameter param = new SqlParameter("@Retval", SqlDbType.Int);
                param.Direction = System.Data.ParameterDirection.ReturnValue;
                command.Parameters.Add(param);

                param = new SqlParameter("@SSASServerName", SqlDbType.VarChar);
                param.Value = this.ssasServerName;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@SSASDBName", SqlDbType.VarChar);
                param.Value = this.ssasDBName;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                try
                {
                    connection.Open();
                    AnalysisServicesSourceDatabase.printText = "";
                    command.ExecuteScalar();

                    string infoMessages = AnalysisServicesSourceDatabase.printText;

                    returnValue = Int32.Parse(command.Parameters["@Retval"].Value.ToString());

                    if (returnValue != RESULT_SUCCESS)
                        throw new ApplicationException(String.Format("Error running [ssasload].[ETL SSAS DB - Generate Cube Calculations XMLA Script]"));

                    this.log.InfoFormat("Finished running [ssasload].[ETL SSAS DB - Generate Cube Calculations XMLA Script]");
                    this.log.InfoFormat("");

                    return infoMessages;
                }
                catch (SqlException e)
                {
                    this.log.Error("SQL Exception", e);
                    throw new ApplicationException(AnalysisServicesSourceDatabase.printText, e);
                }
                finally
                {
                    try
                    {
                        connection.Close();
                    }
                    catch
                    {
                    }
                }
            }
        }

        private static void InfoMessageHandler(object mySender, SqlInfoMessageEventArgs myEvent)
        {
            for (int i = 0; i < myEvent.Errors.Count; i++)
            {
                AnalysisServicesSourceDatabase.printText += myEvent.Errors[i].ToString().Replace("System.Data.SqlClient.SqlError:", "") + "\r\n";
            }

        }
    }
}
