﻿/*
--	Copyright (c) 2007 - 2021 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Data;
using System.Data.SqlClient;

using log4net;

namespace DWJobs.ETLRunner
{
    public class DataMartDatabase
    {
        private readonly string connectionString;
        private string providerName;
        
        private readonly ILog   log;

        private string printText;

        private const int RESULT_SUCCESS = 0;

        public DataMartDatabase(string connectionString, string providerName, ILog log)
        {
            this.connectionString = connectionString;
            this.providerName = providerName;
            this.log = log;
        }

        public string PrepareETLSessionId(string uniqueSessionId)
        {
            int returnValue = -1;
            string storedProcedureName = "[dw].[ETL Data Mart - Prepare Session Id]";
            this.log.InfoFormat("Running {0}", storedProcedureName);

            string queryString = storedProcedureName;
            using (SqlConnection connection = new SqlConnection(this.connectionString))
            {
                connection.InfoMessage += new SqlInfoMessageEventHandler(InfoMessageHandler);

                SqlCommand command = new SqlCommand(queryString, connection);
                command.CommandType = System.Data.CommandType.StoredProcedure;
                command.CommandTimeout = 0;

                SqlParameter param = new SqlParameter("@UniqueSessionId", SqlDbType.VarChar);
                if (uniqueSessionId == null)
                    param.Value = DBNull.Value;
                else
                    param.Value = uniqueSessionId;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@Retval", SqlDbType.Int);
                param.Direction = System.Data.ParameterDirection.ReturnValue;
                command.Parameters.Add(param);

                try
                {
                    connection.Open();
                    command.ExecuteScalar();

                    string infoMessages = "done"; // this.printText;

                    returnValue = Int32.Parse(command.Parameters["@Retval"].Value.ToString());

                    if (returnValue != RESULT_SUCCESS)
                        throw new ApplicationException(String.Format("Error running {0}", storedProcedureName));

                    this.log.InfoFormat("Finished running {0}", storedProcedureName);
                    this.log.InfoFormat("");

                    return infoMessages;
                }
                catch (SqlException e)
                {
                    this.log.Error("SQL Exception", e);
                    throw new ApplicationException(this.printText, e);
                }
                finally
                {
                    try
                    {
                        connection.Close();
                    }
                    catch
                    {
                    }
                }
            }
        }

        public string PrepareETLSession( )
        {
            int returnValue = -1;
            string storedProcedureName = "[dw].[ETL Data Mart - Prepare Session]";
            this.log.InfoFormat("Running {0}", storedProcedureName);

            string queryString = storedProcedureName;
            using (SqlConnection connection = new SqlConnection(this.connectionString))
            {
                connection.InfoMessage += new SqlInfoMessageEventHandler(InfoMessageHandler);

                SqlCommand command = new SqlCommand(queryString, connection);
                command.CommandType = System.Data.CommandType.StoredProcedure;
                command.CommandTimeout = 0;

                SqlParameter param = new SqlParameter("@Retval", SqlDbType.Int);
                param.Direction = System.Data.ParameterDirection.ReturnValue;
                command.Parameters.Add(param);

                try
                {
                    connection.Open();
                    //                    this.printText = "";
                    command.ExecuteScalar();
                    //                    this.log.Info(this.printText);

                    string infoMessages = "done"; // this.printText;

                    returnValue = Int32.Parse(command.Parameters["@Retval"].Value.ToString());

                    if (returnValue != RESULT_SUCCESS)
                        throw new ApplicationException(String.Format("Error running {0}", storedProcedureName));

                    this.log.InfoFormat("Finished running {0}", storedProcedureName);
                    this.log.InfoFormat("");

                    return infoMessages;
                }
                catch (SqlException e)
                {
                    this.log.Error("SQL Exception", e);
                    throw new ApplicationException(this.printText, e);
                }
                finally
                {
                    try
                    {
                        connection.Close();
                    }
                    catch
                    {
                    }
                }
            }
        }

        public string CleanupETLSessionId(string uniqueSessionId)
        {
            int returnValue = -1;
            string storedProcedureName = "[dw].[ETL Data Mart - Cleanup Session Id]";
            this.log.InfoFormat("Running {0}", storedProcedureName);

            string queryString = storedProcedureName;
            using (SqlConnection connection = new SqlConnection(this.connectionString))
            {
                connection.InfoMessage += new SqlInfoMessageEventHandler(InfoMessageHandler);

                SqlCommand command = new SqlCommand(queryString, connection);
                command.CommandType = System.Data.CommandType.StoredProcedure;
                command.CommandTimeout = 0;

                SqlParameter param = new SqlParameter("@UniqueSessionId", SqlDbType.VarChar);
                if (uniqueSessionId == null)
                    param.Value = DBNull.Value;
                else
                    param.Value = uniqueSessionId;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@Retval", SqlDbType.Int);
                param.Direction = System.Data.ParameterDirection.ReturnValue;
                command.Parameters.Add(param);

                try
                {
                    connection.Open();
                    command.ExecuteScalar();

                    string infoMessages = "done"; // this.printText;

                    returnValue = Int32.Parse(command.Parameters["@Retval"].Value.ToString());

                    if (returnValue != RESULT_SUCCESS)
                        throw new ApplicationException(String.Format("Error running {0}", storedProcedureName));

                    this.log.InfoFormat("Finished running {0}", storedProcedureName);
                    this.log.InfoFormat("");

                    return infoMessages;
                }
                catch (SqlException e)
                {
                    this.log.Error("SQL Exception", e);
                    throw new ApplicationException(this.printText, e);
                }
                finally
                {
                    try
                    {
                        connection.Close();
                    }
                    catch
                    {
                    }
                }
            }
        }

        public string CleanupETLSession(bool isForAdjustments)
        {
            int returnValue = -1;
            string storedProcedureName = "[dw].[ETL Data Mart - Cleanup Session]";
            this.log.InfoFormat("Running {0}", storedProcedureName);

            string queryString = storedProcedureName;
            using (SqlConnection connection = new SqlConnection(this.connectionString))
            {
                connection.InfoMessage += new SqlInfoMessageEventHandler(InfoMessageHandler);

                SqlCommand command = new SqlCommand(queryString, connection);
                command.CommandType = System.Data.CommandType.StoredProcedure;
                command.CommandTimeout = 0;

                SqlParameter param = new SqlParameter("@ETLComponent", SqlDbType.Char);
                param.Value = (isForAdjustments ? "ADJ" : "FACT" );
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@Retval", SqlDbType.Int);
                param.Direction = System.Data.ParameterDirection.ReturnValue;
                command.Parameters.Add(param);

                try
                {
                    connection.Open();
                    //                    this.printText = "";
                    command.ExecuteScalar();
                    //                    this.log.Info(this.printText);

                    string infoMessages = "done"; // this.printText;

                    returnValue = Int32.Parse(command.Parameters["@Retval"].Value.ToString());

                    if (returnValue != RESULT_SUCCESS)
                        throw new ApplicationException(String.Format("Error running {0}", storedProcedureName));

                    this.log.InfoFormat("Finished running {0}", storedProcedureName);
                    this.log.InfoFormat("");

                    return infoMessages;
                }
                catch (SqlException e)
                {
                    this.log.Error("SQL Exception", e);
                    throw new ApplicationException(this.printText, e);
                }
                finally
                {
                    try
                    {
                        connection.Close();
                    }
                    catch
                    {
                    }
                }
            }
        }

        public string PrepareForInitialLoad( )
        {
            int returnValue = -1;
            this.log.Info("Running Prepare DW database For Initial Load");

            string queryString = "[dw].[ETL Data Mart - Prepare For Initial Load]";
            using (SqlConnection connection = new SqlConnection(this.connectionString))
            {
                connection.InfoMessage += new SqlInfoMessageEventHandler(InfoMessageHandler);

                SqlCommand command = new SqlCommand(queryString, connection);
                command.CommandType = System.Data.CommandType.StoredProcedure;
                command.CommandTimeout = 0;

                SqlParameter param = new SqlParameter("@Retval", SqlDbType.Int);
                param.Direction = System.Data.ParameterDirection.ReturnValue;
                command.Parameters.Add(param);

                try
                {
                    connection.Open();
//                    this.printText = "";
                    command.ExecuteScalar();
//                    this.log.Info(this.printText);

                    string infoMessages = "done"; // this.printText;

                    returnValue = Int32.Parse(command.Parameters["@Retval"].Value.ToString());

                    if (returnValue != RESULT_SUCCESS)
                        throw new ApplicationException(String.Format("Error running Prepare DW database For Initial Load"));

                    this.log.InfoFormat("Finished running Prepare DW database For Initial Load");
                    this.log.InfoFormat("");

                    return infoMessages;
                }
                catch (SqlException e)
                {
                    this.log.Error("SQL Exception", e);
                    throw new ApplicationException(this.printText, e);
                }
                finally
                {
                    try
                    {
                        connection.Close();
                    }
                    catch
                    {
                    }
                }
            }
        }

        public string UpdateStatisticsInDW( bool updateStatisticsInDynamicDimensions,
                                            bool updateStatisticsInFacts,
                                            bool recompile)
        {
            int returnValue = -1;
            this.log.Info("Running update statistics");

            string queryString = "[dw].[ETL Data Mart - Update Statistics]";
            using (SqlConnection connection = new SqlConnection(this.connectionString))
            {
                connection.InfoMessage += new SqlInfoMessageEventHandler(InfoMessageHandler);

                SqlCommand command = new SqlCommand(queryString, connection);
                command.CommandType = System.Data.CommandType.StoredProcedure;
                command.CommandTimeout = 0;

                SqlParameter param = new SqlParameter("@UpdateStatisticsInDynamicDimensions", SqlDbType.Bit);
                param.Value = updateStatisticsInDynamicDimensions ? 1 : 0;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@UpdateStatisticsInFacts", SqlDbType.Bit);
                param.Value = updateStatisticsInFacts ? 1 : 0;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@Recompile", SqlDbType.Bit);
                param.Value = recompile ? 1 : 0;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@Retval", SqlDbType.Int);
                param.Direction = System.Data.ParameterDirection.ReturnValue;
                command.Parameters.Add(param);

                try
                {
                    connection.Open();
//                    this.printText = "";
                    command.ExecuteScalar();
//                    this.log.Info(this.printText);

                    string infoMessages = "done"; // this.printText;

                    returnValue = Int32.Parse(command.Parameters["@Retval"].Value.ToString());

                    if (returnValue != RESULT_SUCCESS)
                        throw new ApplicationException(String.Format("Error ruuning Update Statistics In DW"));

                    this.log.InfoFormat("Finished running Update Statistics In DW");
                    this.log.InfoFormat("");

                    return infoMessages;
                }
                catch (SqlException e)
                {
                    this.log.Error("SQL Exception", e);
                    throw new ApplicationException(this.printText, e);
                }
                finally
                {
                    try
                    {
                        connection.Close();
                    }
                    catch
                    {
                    }
                }
            }
        }

        private string ETLFacts(string queryString, string ssasServerName, string ssasDBName, DateTime startDateToProcess, DateTime endDateToProcess, string uniqueSessionId)
        {
            int returnValue = -1;

            using (SqlConnection connection = new SqlConnection(this.connectionString))
            {
                connection.InfoMessage += new SqlInfoMessageEventHandler(InfoMessageHandler);

                SqlCommand command = new SqlCommand(queryString, connection);
                command.CommandType = System.Data.CommandType.StoredProcedure;
                command.CommandTimeout = 0;

                SqlParameter param = new SqlParameter("@Retval", SqlDbType.Int);
                param.Direction = System.Data.ParameterDirection.ReturnValue;
                command.Parameters.Add(param);

                if (!String.IsNullOrEmpty( ssasServerName) && !String.IsNullOrEmpty( ssasDBName ) )
                {
                    param = new SqlParameter("@SSASServerName", SqlDbType.Char);
                    param.Value = ssasServerName;
                    param.Direction = System.Data.ParameterDirection.Input;
                    command.Parameters.Add(param);

                    param = new SqlParameter("@SSASDBName", SqlDbType.Char);
                    param.Value = ssasDBName;
                    param.Direction = System.Data.ParameterDirection.Input;
                    command.Parameters.Add(param);
                }

                param = new SqlParameter("@StartDate", SqlDbType.Date);
                param.Value = startDateToProcess;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@EndDate", SqlDbType.Date);
                param.Value = endDateToProcess;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@UniqueSessionId", SqlDbType.Char);
                if (uniqueSessionId == null)
                    param.Value = DBNull.Value;
                else
                    param.Value = uniqueSessionId;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@Debug", SqlDbType.Char);
                param.Value = "N"; // Prints only, i.e.e, don't include SELECTs inside Debug
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                try
                {
                    connection.Open();
                    this.log.InfoFormat("Running ETL Facts from from {0} to {1}, spid - {2}", startDateToProcess.ToShortDateString(), endDateToProcess.ToShortDateString(), DataMartDatabase.GetSPID(connection));

                    this.printText = "";
                    command.ExecuteScalar();

                    returnValue = Int32.Parse(command.Parameters["@Retval"].Value.ToString());

                    if (returnValue != RESULT_SUCCESS)
                        throw new ApplicationException(String.Format("Error running ETL Facts"));

                    string message = String.Format("Finished running ETL of facts for dates from {0} to {1}", startDateToProcess.ToShortDateString(), endDateToProcess.ToShortDateString() );
                    this.log.Info(message);
                    this.log.Info("");

                    return message;
                }
                catch (SqlException e)
                {
                    this.log.ErrorFormat("SQL Exception Message: {0} Number: {1} Procedure: {2} Line Number: {3} Text from PRINT: {4} ", e.Message, e.Number, e.Procedure ?? "none", e.LineNumber, this.printText );
                        // The first one is 
                        //    A transport-level error has occurred when receiving results from the server. (provider: TCP Provider, error: 0 - The specified network name is no longer available.)
                        // The second A system assertion check has failed. Check the SQL Server error log for details. 
                    if (e.Number == 64 || e.Number == 3624 ) 
                        throw new ApplicationException();
                    else
                        throw;
                }
                finally
                {
                    try
                    {
                        connection.Close();
                    }
                    catch
                    {
                    }
                }
            }
        }

        public string ETLFacts(DateTime startDateToProcess, DateTime endDateToProcess, string uniqueSessionId)
        {
            return ETLFacts("[dw].[ETL Data Mart - Facts]", null, null, startDateToProcess, endDateToProcess, uniqueSessionId);
        }

        public string ETLAdjustments(string ssasServerName, string ssasDBName, DateTime startDateToProcess, DateTime endDateToProcess, string uniqueSessionId)
        {
            return ETLFacts("[dw].[ETL Data Mart - Adjustments]", ssasServerName, ssasDBName, startDateToProcess, endDateToProcess, uniqueSessionId);
        }

        public string ETLCalculations(string ssasServerName, string ssasDBName, DateTime startDateToProcess, DateTime endDateToProcess, string uniqueSessionId)
        {
            return ETLFacts("[dw].[ETL Data Mart - Calculations]", null, null, startDateToProcess, endDateToProcess, uniqueSessionId);
        }

        public string PrepareSharedDataForEffectiveDateETL(DateTime startDateToProcess, DateTime endDateToProcess, string uniqueSessionId)
        {
            return ETLFacts("[dw].[ETL Data Mart - Prepare Shared Data For Effective Date]", null, null, startDateToProcess, endDateToProcess, uniqueSessionId);
        }

        public string SynchronizeDimensionsWithDimAuthority(string dimAuthorityServerName, string dimAuthorityDBName)
        {
            int returnValue = -1;
            this.log.InfoFormat("Running Synchronize Dimensions With Dim Authority from {0}{1}{2} database", dimAuthorityServerName, dimAuthorityServerName == "" ? "" : ".", dimAuthorityDBName);

            string queryString = "[dw].[ETL Data Mart - Synchronize Dimensions]";
            using (SqlConnection connection = new SqlConnection(this.connectionString))
            {
                connection.InfoMessage += new SqlInfoMessageEventHandler(InfoMessageHandler);

                SqlCommand command = new SqlCommand(queryString, connection);
                command.CommandType = System.Data.CommandType.StoredProcedure;
                command.CommandTimeout = 0;

                SqlParameter param = new SqlParameter("@Retval", SqlDbType.Int);
                param.Direction = System.Data.ParameterDirection.ReturnValue;
                command.Parameters.Add(param);

                param = new SqlParameter("@DimAuthorityServerName", SqlDbType.Char);
                param.Value = dimAuthorityServerName;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@DimAuthorityDBName", SqlDbType.Char);
                param.Value = dimAuthorityDBName;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@Debug", SqlDbType.Char);
                param.Value = "N"; // Prints only, i.e.e, don't include SELECTs inside Debug
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                try
                {
                    connection.Open();
                    this.printText = "";
                    command.ExecuteScalar();

                    returnValue = Int32.Parse(command.Parameters["@Retval"].Value.ToString());

                    if (returnValue != RESULT_SUCCESS)
                        throw new ApplicationException(String.Format("Error running ETL Facts"));

                    string message = String.Format("Finished running Synchronize Dimensions With Dim Authority");
                    this.log.Info(message);
                    this.log.Info("");

                    return message;
                }
                catch (SqlException e)
                {
                    this.log.ErrorFormat("SQL Exception Message: {0} Number: {1} Procedure: {2} Line Number: {3} Text from PRINT: {4} ", e.Message, e.Number, e.Procedure ?? "none", e.LineNumber, this.printText);
                    // The first one is 
                    //    A transport-level error has occurred when receiving results from the server. (provider: TCP Provider, error: 0 - The specified network name is no longer available.)
                    // The second A system assertion check has failed. Check the SQL Server error log for details. 
                    if (e.Number == 64 || e.Number == 3624)
                        throw new ApplicationException();
                    else
                        throw;
                }
                finally
                {
                    try
                    {
                        connection.Close();
                    }
                    catch
                    {
                    }
                }
            }
        }

        public string GetDatesToLoad( bool isForAdjustments, DateTime? startDateToProcess, DateTime? endDateToProcess, out List<DateTime> datesToProcess)
        {
            int returnValue = -1;
            this.log.InfoFormat("Determining dates to load ETL Facts" );

            datesToProcess = new List<DateTime>();

            string queryString = "[dw].[ETL Data Mart - Determine Dates]";
            using (SqlConnection connection = new SqlConnection(this.connectionString))
            {
                connection.InfoMessage += new SqlInfoMessageEventHandler(InfoMessageHandler);

                SqlCommand command = new SqlCommand(queryString, connection);
                command.CommandType = System.Data.CommandType.StoredProcedure;
                command.CommandTimeout = 0;

                SqlParameter param = new SqlParameter("@Retval", SqlDbType.Int);
                param.Direction = System.Data.ParameterDirection.ReturnValue;
                command.Parameters.Add(param);

                param = new SqlParameter("@ETLComponent", SqlDbType.Char);
                param.Value = (isForAdjustments ? "ADJ" : "FACT" );
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@StartDate", SqlDbType.Date);
                if ( startDateToProcess == null)
                    param.Value = DBNull.Value;
                else
                    param.Value = startDateToProcess.Value;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@EndDate", SqlDbType.Date);
                if (endDateToProcess == null)
                    param.Value = DBNull.Value;
                else
                    param.Value = endDateToProcess.Value;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@Debug", SqlDbType.Char);
                param.Value = "N"; // Prints only, i.e.e, don't include SELECTs inside Debug
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                try
                {
                    connection.Open();
                    this.printText = "";

                    using (SqlDataReader reader = command.ExecuteReader())
                    {
                        while (reader != null && reader.Read())
                        {
                            datesToProcess.Add((DateTime)reader[0]);
                        }
                    }

                    this.log.Info(this.printText);

                    string infoMessages = this.printText;

                    returnValue = Int32.Parse(command.Parameters["@Retval"].Value.ToString());

                    if (returnValue != RESULT_SUCCESS)
                        throw new ApplicationException(String.Format("Error running ETL Facts"));

                    if (datesToProcess.Count == 0 )
                        this.log.InfoFormat("Skipping load of facts");
                    else
                        this.log.InfoFormat("Finished determining dates to load ETL Facts. Running ETL for {0} date(s). Dates - {1}", datesToProcess.Count, String.Join("; ", datesToProcess.Select(dateToProcess => dateToProcess.ToShortDateString()).ToArray()));
                    this.log.InfoFormat("");

                    return infoMessages;
                }
                catch (SqlException e)
                {
                    this.log.ErrorFormat("SQL Exception Message: {0} Number: {1} Procedure: {2} Line Number: {3} Text from PRINT: {4} ", e.Message, e.Number, e.Procedure ?? "none", e.LineNumber, this.printText );
                        // The first one is 
                        //    A transport-level error has occurred when receiving results from the server. (provider: TCP Provider, error: 0 - The specified network name is no longer available.)
                        // The second A system assertion check has failed. Check the SQL Server error log for details. 
                    if (e.Number == 64 || e.Number == 3624 ) 
                        throw new ApplicationException();
                    else
                        throw;
                }
                finally
                {
                    try
                    {
                        connection.Close();
                    }
                    catch
                    {
                    }
                }
            }
        }

        private void InfoMessageHandler(object mySender, SqlInfoMessageEventArgs myEvent)
        {
            for (int i = 0; i < myEvent.Errors.Count; i++)
            {
                if (this.log != null)
                    this.log.Info(myEvent.Errors[i].ToString().Replace("System.Data.SqlClient.SqlError:", ""));
                else
                    this.printText += myEvent.Errors[i].ToString().Replace("System.Data.SqlClient.SqlError:", "") + "\r\n";
            }

        }

        private static short GetSPID(SqlConnection connection)
        {
            short spid = 0;

            if (connection.State != ConnectionState.Open)
                return spid;

            using (SqlCommand cmd = new SqlCommand("SELECT @@SPID", connection ))
            {
                spid = (short)cmd.ExecuteScalar();
            }

            return spid;
        }

    }
}
