﻿/*
--	Copyright (c) 2007 - 2021 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Data;
using System.Data.SqlClient;

using log4net;

namespace DWJobs.ETLRunner
{
    public class DimensionAuthorityDatabase
    {
        private string connectionString;
        private string providerName;
        private ILog   log;

        private string printText;

        private const int RESULT_SUCCESS = 0;
        private const string LOAD_MODE_SERIAL = "S";
        private const string LOAD_MODE_PARALLEL = "P";

        public DimensionAuthorityDatabase(string connectionString, string providerName, ILog log)
        {
            this.connectionString = connectionString;
            this.providerName = providerName;
            this.log = log;
        }

        public string PrepareForInitialLoad( )
        {
            this.log.InfoFormat("Running Prepare Dimensions For Initial Load");

            string queryString = "[etl].[ETL Dimensions - Prepare For Initial Load]";
            using (SqlConnection connection = new SqlConnection(this.connectionString))
            {
                connection.InfoMessage += new SqlInfoMessageEventHandler(InfoMessageHandler);

                SqlCommand command = new SqlCommand(queryString, connection);
                command.CommandType = System.Data.CommandType.StoredProcedure;
                command.CommandTimeout = 0;

                SqlParameter param = new SqlParameter("@Retval", SqlDbType.Int);
                param.Direction = System.Data.ParameterDirection.ReturnValue;
                command.Parameters.Add(param);

                param = new SqlParameter("@Debug", SqlDbType.Char);
                param.Value = "N"; // Use "P" for Prints only, i.e, don't include SELECTs inside Debug
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                try
                {
                    connection.Open();
                    this.printText = "";
                    command.ExecuteScalar();
                    this.log.Info(this.printText);

                    string infoMessages = this.printText;

                    int returnValue = Int32.Parse(command.Parameters["@Retval"].Value.ToString());

                    if (returnValue != RESULT_SUCCESS)
                        throw new ApplicationException(String.Format("Error running Prepare Dimensions For Initial Load"));

                    this.log.InfoFormat("Finished running Prepare Dimensions For Initial Load");
                    this.log.InfoFormat("");

                    return infoMessages;
                }
                catch (SqlException e)
                {
                    this.log.Error("SQL Exception", e);
                    throw new ApplicationException(this.printText, e);
                }
                finally
                {
                    try
                    {
                        connection.Close();
                    }
                    catch
                    {
                    }
                }
            }
        }

        public string ETLDimensions(string changedTablesFlags, string datesList, bool loadSerially, string uniqueSessionId)
        {
            this.log.InfoFormat("Running ETL Dimensions from for dates {0}", datesList == null ? " - default" : datesList );

            string queryString = "[etl].[ETL Dimensions]";
            using (SqlConnection connection = new SqlConnection(this.connectionString))
            {
                connection.InfoMessage += new SqlInfoMessageEventHandler(InfoMessageHandler);

                SqlCommand command = new SqlCommand(queryString, connection);
                command.CommandType = System.Data.CommandType.StoredProcedure;
                command.CommandTimeout = 0;

                SqlParameter param = new SqlParameter("@Retval", SqlDbType.Int);
                param.Direction = System.Data.ParameterDirection.ReturnValue;
                command.Parameters.Add(param);

                param = new SqlParameter("@ChangedTablesFlags", SqlDbType.VarChar);
                param.Value = changedTablesFlags;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);
                
                param = new SqlParameter("@DatesList", SqlDbType.Text);
                if (datesList == null)
                    param.Value = DBNull.Value;
                else
                    param.Value = datesList;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@LoadMode", SqlDbType.Char);
                if (loadSerially)
                    param.Value = LOAD_MODE_SERIAL;
                else
                    param.Value = LOAD_MODE_PARALLEL;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@UniqueSessionId", SqlDbType.VarChar);
                if (uniqueSessionId == null)
                    param.Value = DBNull.Value;
                else
                    param.Value = uniqueSessionId;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@Debug", SqlDbType.Char);
                param.Value = "N"; // Use "P" for Prints only, i.e, don't include SELECTs inside Debug
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                try
                {
                    connection.Open();
                    this.printText = "";
                    command.ExecuteScalar();
                    this.log.Info(this.printText);

                    string infoMessages = this.printText;

                    int returnValue = Int32.Parse(command.Parameters["@Retval"].Value.ToString());

                    if (returnValue != RESULT_SUCCESS)
                        throw new ApplicationException(String.Format("Error running ETL Dimensions"));

                    this.log.InfoFormat("Finished running ETL of dimensions");
                    this.log.InfoFormat("");

                    return infoMessages;
                }
                catch (SqlException e)
                {
                    this.log.Error("SQL Exception", e);
                    throw new ApplicationException(this.printText, e);
                }
                finally
                {
                    try
                    {
                        connection.Close();
                    }
                    catch
                    {
                    }
                }
            }
        }

        public string PrepareDimensionsEffectiveDateSourceDataForParallelLoad( string changedTablesFlags, DateTime? effectiveDate, string uniqueSessionId)
        {
            this.log.InfoFormat("Running ETL Dimensions for effecive date {0}", effectiveDate == null ? " - default" : effectiveDate.Value.ToShortDateString() );

            string queryString = "[etl].[ETL Dimensions - Prepare Effective Date Source Data For Parallel Load]";
            using (SqlConnection connection = new SqlConnection(this.connectionString))
            {
                connection.InfoMessage += new SqlInfoMessageEventHandler(InfoMessageHandler);

                SqlCommand command = new SqlCommand(queryString, connection);
                command.CommandType = System.Data.CommandType.StoredProcedure;
                command.CommandTimeout = 0;

                SqlParameter param = new SqlParameter("@Retval", SqlDbType.Int);
                param.Direction = System.Data.ParameterDirection.ReturnValue;
                command.Parameters.Add(param);

                param = new SqlParameter("@ChangedTablesFlags", SqlDbType.VarChar);
                param.Value = changedTablesFlags;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@EffectiveDate", SqlDbType.Date);
                if (effectiveDate == null)
                    param.Value = DBNull.Value;
                else
                    param.Value = effectiveDate.Value.ToShortDateString();
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@UniqueSessionId", SqlDbType.VarChar);
                if (uniqueSessionId == null)
                    param.Value = DBNull.Value;
                else
                    param.Value = uniqueSessionId;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@Debug", SqlDbType.Char);
                param.Value = "N"; // Use "P" for Prints only, i.e, don't include SELECTs inside Debug
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                try
                {
                    connection.Open();
                    this.printText = "";
                    command.ExecuteScalar();
                    this.log.Info(this.printText);

                    string infoMessages = this.printText;

                    int returnValue = Int32.Parse(command.Parameters["@Retval"].Value.ToString());

                    if (returnValue != RESULT_SUCCESS)
                        throw new ApplicationException(String.Format("Error running Prepare Dimensions Effective Date Source Data For Parallel Load "));

                    this.log.InfoFormat("Finished running ETL of dimensions");
                    this.log.InfoFormat("");

                    return infoMessages;
                }
                catch (SqlException e)
                {
                    this.log.Error("SQL Exception", e);
                    throw new ApplicationException(this.printText, e);
                }
                finally
                {
                    try
                    {
                        connection.Close();
                    }
                    catch
                    {
                    }
                }
            }
        }

        public string GetDatesToLoad(DateTime? startDateToProcess, DateTime? endDateToProcess, out List<DateTime> datesToProcess, out string changedTablesFlags)
        {
            int returnValue = -1;
            this.log.InfoFormat("Determining dates to load ETL Dimensions" );

            datesToProcess = new List<DateTime>();

            string queryString = "[etl].[ETL Dimensions - Determine Dates]";
            using (SqlConnection connection = new SqlConnection(this.connectionString))
            {
                connection.InfoMessage += new SqlInfoMessageEventHandler(InfoMessageHandler);

                SqlCommand command = new SqlCommand(queryString, connection);
                command.CommandType = System.Data.CommandType.StoredProcedure;
                command.CommandTimeout = 0;

                SqlParameter param = new SqlParameter("@Retval", SqlDbType.Int);
                param.Direction = System.Data.ParameterDirection.ReturnValue;
                command.Parameters.Add(param);

                param = new SqlParameter("@ChangedTablesFlags", SqlDbType.VarChar);
                param.Direction = System.Data.ParameterDirection.Output;
                param.Size = 20;
                command.Parameters.Add(param);

                param = new SqlParameter("@StartDate", SqlDbType.Date);
                if (startDateToProcess == null)
                    param.Value = DBNull.Value;
                else
                    param.Value = startDateToProcess.Value;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@EndDate", SqlDbType.Date);
                if (endDateToProcess == null)
                    param.Value = DBNull.Value;
                else
                    param.Value = endDateToProcess.Value;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@Debug", SqlDbType.Char);
                param.Value = "N"; // Prints only, i.e.e, don't include SELECTs inside Debug
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                try
                {
                    connection.Open();
                    this.printText = "";

                    using (SqlDataReader reader = command.ExecuteReader())
                    {
                        while (reader != null && reader.Read())
                        {
                            datesToProcess.Add((DateTime)reader[0]);
                        }
                    }

                    this.log.Info(this.printText);

                    string infoMessages = this.printText;

                    changedTablesFlags =  command.Parameters["@ChangedTablesFlags"].Value.ToString();
                    returnValue = Int32.Parse(command.Parameters["@Retval"].Value.ToString());

                    if (returnValue != RESULT_SUCCESS)
                        throw new ApplicationException(String.Format("Error determining Dates For Next ETL Dimensions"));

                    if (datesToProcess.Count > 0)
                        this.log.InfoFormat("Finished determining dates to load Dimensions. Running ETL for {0} date(s). Dates - {1}", datesToProcess.Count, String.Join("; ", datesToProcess.Select(dateToProcess => dateToProcess.ToShortDateString()).ToArray()));
                    else
                        this.log.InfoFormat("There are no dates to load the Dimensions.");

                    this.log.InfoFormat("");

                    return infoMessages;
                }
                catch (SqlException e)
                {
                    this.log.ErrorFormat("SQL Exception Message: {0} Number: {1} Procedure: {2} Line Number: {3} Text from PRINT: {4} ", e.Message, e.Number, e.Procedure ?? "none", e.LineNumber, this.printText);
                    // The first one is 
                    //    A transport-level error has occurred when receiving results from the server. (provider: TCP Provider, error: 0 - The specified network name is no longer available.)
                    // The second A system assertion check has failed. Check the SQL Server error log for details. 
                    if (e.Number == 64 || e.Number == 3624)
                        throw new ApplicationException();
                    else
                        throw;
                }
                finally
                {
                    try
                    {
                        connection.Close();
                    }
                    catch
                    {
                    }
                }
            }
        }


        public string PrepareETLDimensionsSession( string uniqueSessionId)
        {
            int returnValue = -1;
            string queryString = "[etl].[ETL Dimensions - Prepare Session]";
            this.log.InfoFormat("Running {0}", queryString);

            using (SqlConnection connection = new SqlConnection(this.connectionString))
            {
                connection.InfoMessage += new SqlInfoMessageEventHandler(InfoMessageHandler);

                SqlCommand command = new SqlCommand(queryString, connection);
                command.CommandType = System.Data.CommandType.StoredProcedure;
                command.CommandTimeout = 0;

                //param = new SqlParameter("@StartDate", SqlDbType.Date);
                //param.Value = startDateToProcess;
                //param.Direction = System.Data.ParameterDirection.Input;
                //command.Parameters.Add(param);

                //param = new SqlParameter("@EndDate", SqlDbType.Date);
                //param.Value = endDateToProcess;
                //param.Direction = System.Data.ParameterDirection.Input;
                //command.Parameters.Add(param);

                SqlParameter param = new SqlParameter("@UniqueSessionId", SqlDbType.VarChar);
                if (uniqueSessionId == null)
                    param.Value = DBNull.Value;
                else
                    param.Value = uniqueSessionId;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@Retval", SqlDbType.Int);
                param.Direction = System.Data.ParameterDirection.ReturnValue;
                command.Parameters.Add(param);

                try
                {
                    connection.Open();
                    //                    this.printText = "";
                    command.ExecuteScalar();
                    //                    this.log.Info(this.printText);

                    string infoMessages = "done"; // this.printText;

                    returnValue = Int32.Parse(command.Parameters["@Retval"].Value.ToString());

                    if (returnValue != RESULT_SUCCESS)
                        throw new ApplicationException(String.Format("Error running {0}", queryString));

                    this.log.InfoFormat("Finished running {0}", queryString);
                    this.log.InfoFormat("");

                    return infoMessages;
                }
                catch (SqlException e)
                {
                    this.log.Error("SQL Exception", e);
                    throw new ApplicationException(this.printText, e);
                }
                finally
                {
                    try
                    {
                        connection.Close();
                    }
                    catch
                    {
                    }
                }
            }
        }

        public string CleanupETLDimensionsSession(string uniqueSessionId)
        {
            int returnValue = -1;
            string queryString = "[etl].[ETL Dimensions - Cleanup Session]";
            this.log.InfoFormat("Running {0}", queryString);

            using (SqlConnection connection = new SqlConnection(this.connectionString))
            {
                connection.InfoMessage += new SqlInfoMessageEventHandler(InfoMessageHandler);

                SqlCommand command = new SqlCommand(queryString, connection);
                command.CommandType = System.Data.CommandType.StoredProcedure;
                command.CommandTimeout = 0;

                SqlParameter param = new SqlParameter("@UniqueSessionId", SqlDbType.VarChar);
                if (uniqueSessionId == null)
                    param.Value = DBNull.Value;
                else
                    param.Value = uniqueSessionId;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@Retval", SqlDbType.Int);
                param.Direction = System.Data.ParameterDirection.ReturnValue;
                command.Parameters.Add(param);

                try
                {
                    connection.Open();
                    //                    this.printText = "";
                    command.ExecuteScalar();
                    //                    this.log.Info(this.printText);

                    string infoMessages = "done"; // this.printText;

                    returnValue = Int32.Parse(command.Parameters["@Retval"].Value.ToString());

                    if (returnValue != RESULT_SUCCESS)
                        throw new ApplicationException(String.Format("Error running {0}", queryString));

                    this.log.InfoFormat("Finished running {0}", queryString);
                    this.log.InfoFormat("");

                    return infoMessages;
                }
                catch (SqlException e)
                {
                    this.log.Error("SQL Exception", e);
                    throw new ApplicationException(this.printText, e);
                }
                finally
                {
                    try
                    {
                        connection.Close();
                    }
                    catch
                    {
                    }
                }
            }
        }

        private void InfoMessageHandler(object mySender, SqlInfoMessageEventArgs myEvent)
        {
            for (int i = 0; i < myEvent.Errors.Count; i++)
            {
                if (this.log != null)
                    this.log.Info(myEvent.Errors[i].ToString().Replace("System.Data.SqlClient.SqlError:", ""));
                else
                    this.printText += myEvent.Errors[i].ToString().Replace("System.Data.SqlClient.SqlError:", "") + "\r\n";
            }

        }
    }
}
