﻿/*
--	Copyright (c) 2007 - 2021 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;

using log4net;

namespace DWJobs.ETLRunner
{
    public class Etl
    {
        // TODO private readonly AppConfiguration.SchedulingAlgorithm schedulingAlgorithm;
        private readonly string providerName;
        private readonly string ssasServerName;
        private readonly string ssasDBName;

        private static int MAX_THREADS = 16;                    // max wait handles cannot be more than 64
        private static int MAX_THREADS_FOR_DIMENSIONS_LOAD = 6; // max wait handles cannot be more than 64
        private static int MAX_DAYS_TO_LOAD_DIMS_SERIALLY = 1;  // max days when do not use parallel load

        private static int PROCESS_ETL_DIMENSIONS                               = 1;
        private static int PROCESS_ETL_FACTS                                    = 2;
        private static int PROCESS_PREPARE_SHARED_DATA_FOR_EFFECTIVE_DATE_ETL   = 3;
        private static int PROCESS_ETL_ADJUSTMENTS                              = 4;
        private static int PROCESS_ETL_CALCULATIONS                             = 5;

        private static int SCHEDULING_ALGORITHM_ROUND_ROBIN = 1;
        private static int SCHEDULING_ALGORITHM_RANGE       = 2;

        private static int MAX_FAILURES_WHEN_USING_ROUND_ROBIN = 3;

        private ILog log;

        public Etl(string providerName, string ssasServerName, string ssasDBName, ILog log)
        {
            this.providerName  = providerName;
            this.ssasServerName = ssasServerName;
            this.ssasDBName = ssasDBName;

            this.log = log;
        }

        public string LoadDimensions(string uniqueSessionId, string dimensionAuthorityDBconnectionString, bool isInitialLoad, DateTime? startDateToLoad, DateTime? endDateToLoad, int numberOfParallelProcesses, int timeoutMinutesOnParallelProcesses)
        {
            string mailMessage = "";

            try
            {
                var dbDim = new DimensionAuthorityDatabase(dimensionAuthorityDBconnectionString, this.providerName, this.log);

                dbDim.PrepareETLDimensionsSession(uniqueSessionId);

                if (isInitialLoad)
                    mailMessage += dbDim.PrepareForInitialLoad();

                List<DateTime> datesToProcess;
                string changedTablesFlags;
                mailMessage += dbDim.GetDatesToLoad( startDateToLoad, endDateToLoad, out datesToProcess, out changedTablesFlags);



                if (datesToProcess.Count > 0)
                {
                    TimeSpan dateDiff = datesToProcess.Last().Subtract(datesToProcess.First());
                    TimeSpan dateDiff1 = new DateTime(2016, 12, 22).Subtract(new DateTime(2016, 12, 21) );

                    string datesList = String.Join(",", datesToProcess.Select(dateToProcess => dateToProcess.ToString("yyyyMMdd")).ToArray());
                    
                    if (datesToProcess.Count <= MAX_DAYS_TO_LOAD_DIMS_SERIALLY)
                        numberOfParallelProcesses = 0;

                    if (numberOfParallelProcesses <= 1) // RunETL serially because need at least two threads to run in parallel
                    {
                        this.log.Info("Running the ETL Dimensions serially.");
                        mailMessage += dbDim.ETLDimensions(changedTablesFlags, datesList, true, uniqueSessionId);
                    }
                    else
                    {
                        mailMessage += ParallelETLDimensions(dimensionAuthorityDBconnectionString, changedTablesFlags, datesToProcess, numberOfParallelProcesses, timeoutMinutesOnParallelProcesses, uniqueSessionId);
                    }
                }
            }
            finally
            {
                var dbDimCleanup = new DimensionAuthorityDatabase(dimensionAuthorityDBconnectionString, this.providerName, this.log);
                dbDimCleanup.CleanupETLDimensionsSession(uniqueSessionId);
            }


            return mailMessage;
        }

        public string CleanUpAllSessionsInDimAuth(string dimensionAuthorityDBconnectionString)
        {
            string mailMessage = "";
            
            var dbDimCleanup = new DimensionAuthorityDatabase(dimensionAuthorityDBconnectionString, this.providerName, this.log);
            dbDimCleanup.CleanupETLDimensionsSession(null);

            return mailMessage;
        }

        public string PrepareEtlSessionId(string uniqueSessionId, string dwDBconnectionString)
        {
            string mailMessage = "";

            var dbFact = new DataMartDatabase(dwDBconnectionString, this.providerName, this.log);
            mailMessage += dbFact.PrepareETLSessionId(uniqueSessionId);

            return mailMessage;
        }

        public string CleanupEtlSessionId(string uniqueSessionId, string dwDBconnectionString)
        {
            string mailMessage = "";

            var dbFact = new DataMartDatabase(dwDBconnectionString, this.providerName, this.log);
            mailMessage += dbFact.CleanupETLSessionId(uniqueSessionId);

            return mailMessage;
        }

        public string LoadFacts(string uniqueSessionId, string dwDBconnectionString, bool isInitialLoad, string dimAuthorityServerName, string dimAuthorityDBName, DateTime? startDateToLoad, DateTime? endDateToLoad, int numberOfParallelProcesses, int timeoutMinutesOnParallelProcesses)
        {
            string mailMessage = "";

            var dbFact = new DataMartDatabase(dwDBconnectionString, this.providerName, this.log);
            if (isInitialLoad)
                mailMessage += dbFact.PrepareForInitialLoad( );

            // Do not need to run in the beginning, since we always run at the end
            // dbFact.UpdateStatisticsInDW(true, true, false);

            if ( !String.IsNullOrEmpty( dimAuthorityDBName ) ) // dimensions are in a different database
                dbFact.SynchronizeDimensionsWithDimAuthority(dimAuthorityServerName, dimAuthorityDBName);
            dbFact.PrepareETLSession();

            List<DateTime> datesToProcess;
            mailMessage += dbFact.GetDatesToLoad(false, startDateToLoad, endDateToLoad, out datesToProcess);

            if (datesToProcess.Count > 0)
            {
                if (numberOfParallelProcesses == 0)
                {
                    this.log.Info("Running the ETL serially.");
                    dbFact.PrepareSharedDataForEffectiveDateETL(datesToProcess.First(), datesToProcess.Last(), uniqueSessionId);
                    mailMessage += dbFact.ETLFacts(datesToProcess.First(), datesToProcess.Last(), uniqueSessionId);
                }
                else
                {
                    //mailMessage += ParallelETLFacts(PROCESS_PREPARE_SHARED_DATA_FOR_EFFECTIVE_DATE_ETL, dwDBconnectionString, datesToProcess, numberOfParallelProcesses, timeoutMinutesOnParallelProcesses, uniqueSessionId, SCHEDULING_ALGORITHM_RANGE);
                    mailMessage += ParallelETLFacts(PROCESS_PREPARE_SHARED_DATA_FOR_EFFECTIVE_DATE_ETL, dwDBconnectionString, datesToProcess, numberOfParallelProcesses, timeoutMinutesOnParallelProcesses, uniqueSessionId, SCHEDULING_ALGORITHM_ROUND_ROBIN);
                    mailMessage += ParallelETLFacts(PROCESS_ETL_FACTS, dwDBconnectionString, datesToProcess, numberOfParallelProcesses, timeoutMinutesOnParallelProcesses, uniqueSessionId, SCHEDULING_ALGORITHM_ROUND_ROBIN );
                }
            }
            else
                this.log.Info("Skipping ETL Facts. No changed data to load.");

            dbFact.CleanupETLSession(false);
            if (isInitialLoad)
                dbFact.UpdateStatisticsInDW(true, true, true);

            return mailMessage;
        }

        public string UpdateStatisticsInDW( string dwDBconnectionString )
        {
            string mailMessage = "";

            var dbFact = new DataMartDatabase(dwDBconnectionString, this.providerName, this.log);

            dbFact.UpdateStatisticsInDW(true, true, true);

            return mailMessage;
        }

        public string LoadAdjustmentsAndCalculations(string uniqueSessionId, string dwDBconnectionString, bool isInitialLoad, string dimAuthorityServerName, string dimAuthorityDBName, DateTime? startDateToLoad, DateTime? endDateToLoad, int numberOfParallelProcesses, int timeoutMinutesOnParallelProcesses)
        {
            string mailMessage = "";

            var dbFact = new DataMartDatabase(dwDBconnectionString, this.providerName, this.log);
            // dbFact.UpdateStatisticsInDW(true, true, true);   -- Update statistics always run at the end of the load

            dbFact.PrepareETLSession();

            List<DateTime> datesToProcess;
            mailMessage += dbFact.GetDatesToLoad(true, startDateToLoad, endDateToLoad, out datesToProcess);

            if (datesToProcess.Count > 0)
            {
                if (numberOfParallelProcesses == 0)
                {
                    this.log.Info("Running the ETL Adjustments serially.");
                    mailMessage += dbFact.ETLAdjustments(this.ssasServerName, this.ssasDBName, datesToProcess.First(), datesToProcess.Last(), uniqueSessionId);
                    // dbFact.UpdateStatisticsInDW(true, true, true);  -- Adj change is small do not need statistics
                    mailMessage += dbFact.ETLCalculations(this.ssasServerName, this.ssasDBName, datesToProcess.First(), datesToProcess.Last(), uniqueSessionId);
                }
                else
                {
                    mailMessage += ParallelETLFacts(PROCESS_ETL_ADJUSTMENTS, dwDBconnectionString, datesToProcess, numberOfParallelProcesses, timeoutMinutesOnParallelProcesses, uniqueSessionId, SCHEDULING_ALGORITHM_ROUND_ROBIN);
                    // dbFact.UpdateStatisticsInDW(true, true, true); -- Adj change is small do not need statistics
                    mailMessage += ParallelETLFacts(PROCESS_ETL_CALCULATIONS, dwDBconnectionString, datesToProcess, numberOfParallelProcesses, timeoutMinutesOnParallelProcesses, uniqueSessionId, SCHEDULING_ALGORITHM_ROUND_ROBIN);
                    //                mailMessage += dbFact.ETLCalculations(this.sourceServerName, this.sourceDBName, this.ssasServerName, this.ssasDBName, datesToProcess.First(), datesToProcess.Last(), null);
                }
            }
            else
                this.log.Info("Skipping ETL Adjustments. No changed data to load.");

            dbFact.CleanupETLSession(true);
            if (isInitialLoad)
                dbFact.UpdateStatisticsInDW(true, true, true);

            return mailMessage;
        }

        private string ParallelETLFacts(int processToRun, string dwDBconnectionString, List<DateTime> datesToProcess, int numberOfParallelProcesses, int timeoutMinutesOnParallelProcesses, string uniqueSessionId, int schedulingAlgorithm)
        {
            DateTime startDateToProcess;
            DateTime endDateToProcess;
            List<DateTime> datesToProcessInThread;
            int parallelLoadDays = 0;

            numberOfParallelProcesses = Math.Min(MAX_THREADS, numberOfParallelProcesses);

            if (schedulingAlgorithm == SCHEDULING_ALGORITHM_RANGE) // Adjust number of threads for even load
            {
                parallelLoadDays = datesToProcess.Count/numberOfParallelProcesses + 1;
                numberOfParallelProcesses = datesToProcess.Count / parallelLoadDays + (datesToProcess.Count % parallelLoadDays == 0 ? 0 : 1);
            }
            else if (schedulingAlgorithm == SCHEDULING_ALGORITHM_ROUND_ROBIN)
                numberOfParallelProcesses = Math.Min(datesToProcess.Count, numberOfParallelProcesses);

            string mailMessage = "";

            // Launch a separate thread for each date range
            var manualEvents = new List<ManualResetEvent>();
            var loadStates = new List<LoadState>();
            var threads = new List<Thread>();

            this.log.InfoFormat("Running the ETL on {0} threads.", numberOfParallelProcesses);
            for (int i = 0; i < numberOfParallelProcesses; i++)
            {
                manualEvents.Add(new ManualResetEvent(false));
                Thread doLoad = null;

                if (schedulingAlgorithm == SCHEDULING_ALGORITHM_RANGE)
                {
                    startDateToProcess = datesToProcess[i * parallelLoadDays];
                    endDateToProcess = (i == numberOfParallelProcesses - 1 ? datesToProcess.Last() : datesToProcess[(i + 1) * parallelLoadDays - 1]);
                }
                else if (schedulingAlgorithm == SCHEDULING_ALGORITHM_ROUND_ROBIN)
                {
                    startDateToProcess = datesToProcess[i];
                    endDateToProcess = datesToProcess[i];
                }
                else
                    throw new ApplicationException("Unknown scheduling algorithm");


                int startDateIndex = datesToProcess.IndexOf( startDateToProcess );
                int endDateIndex = datesToProcess.IndexOf( endDateToProcess );

                datesToProcessInThread = new List<DateTime>(datesToProcess.GetRange(startDateIndex, endDateIndex - startDateIndex + 1));

                var loadState = new LoadState(processToRun, this.providerName, dwDBconnectionString, this.ssasServerName, this.ssasDBName, null, datesToProcessInThread, uniqueSessionId, i, manualEvents[i], log);
                loadStates.Add( loadState );
                doLoad = new Thread(Etl.DoFactsLoad);
                threads.Add( doLoad);
                doLoad.Start(loadState);
                Thread.Sleep(3000); // give connection some time to start
            }

            bool success = false;

            if (schedulingAlgorithm == SCHEDULING_ALGORITHM_RANGE)
            {
                success = WaitHandle.WaitAll(manualEvents.ToArray(), new TimeSpan(0, timeoutMinutesOnParallelProcesses, 0));

                if (!success)
                {
                    this.log.Warn("The ETL timed out. Terminating threads.");
                    // Abort every thread as connection willnot be terminated if just exist the application
                    foreach (var thread in threads)
                    {
                        try
                        {
                            thread.Abort();
                        }
                        catch
                        {
                        }
                    }

                    throw new ApplicationException("Try reducing the number of parallel processes");
                }

                foreach (LoadState loadState in loadStates)
                {
                    if (loadState.ErrorMessage != "" )
                        throw new ApplicationException(loadState.ErrorMessage);
                }
            }
            else if (schedulingAlgorithm == SCHEDULING_ALGORITHM_ROUND_ROBIN)   // TODO. When timed out throttle down immediately
            {
                int nextDateToScheduleIndex = numberOfParallelProcesses;
                int countFailures = 0;
                while (true)
                {

                    int indxCompletedThread = WaitHandle.WaitAny(manualEvents.ToArray(), new TimeSpan(0, timeoutMinutesOnParallelProcesses, 0));

                    if (loadStates[indxCompletedThread].ErrorMessage != "")
                    {
                        startDateToProcess = loadStates[indxCompletedThread].DatesToProcessInThread[0];

                        this.log.InfoFormat("Failed processing date {0}.", startDateToProcess.ToString("yyyy-MM-dd") );
                        countFailures++;
                        if (countFailures > MAX_FAILURES_WHEN_USING_ROUND_ROBIN && numberOfParallelProcesses > 1 ) // Throtlle down the number of threads
                        {
                            Thread.Sleep(30000); // give connections some time to recover

                            this.log.InfoFormat("Reducing number of threads from {0} to {1}. Will re-run ETL for {2}", numberOfParallelProcesses, numberOfParallelProcesses - 1, startDateToProcess.ToString("yyyy-MM-dd"));
                            numberOfParallelProcesses--;
                            countFailures = 0;

                            manualEvents.RemoveAt(indxCompletedThread);
                            loadStates.RemoveAt(indxCompletedThread);
                            threads.RemoveAt(indxCompletedThread);

                            // Add failed date to be the next one processed when the next thread is finished
                            datesToProcess.Insert(nextDateToScheduleIndex, startDateToProcess);
                            continue;
                        }
                        else if (countFailures > MAX_FAILURES_WHEN_USING_ROUND_ROBIN && numberOfParallelProcesses == 1) // Quit - something is wrong
                        {
                            throw new ApplicationException("The ETL process failed.");
                        }
                        else // re-run this date
                        {
                            Thread.Sleep(30000); // give connections some time to recover
                            this.log.InfoFormat("Re-running ETL for {0}", startDateToProcess.ToString("yyyy-MM-dd"));
                            loadStates[indxCompletedThread].ErrorMessage = "";
                            datesToProcessInThread = loadStates[indxCompletedThread].DatesToProcessInThread;
                        }
                    }
                    else if (nextDateToScheduleIndex == datesToProcess.Count)  // we are done with all dates, wait untill all connections are done
                    {
                        this.log.InfoFormat("Dates processed successfully. Done with all dates, wait untill all connections are done. Reducing number of threads from {0} to {1}", numberOfParallelProcesses, numberOfParallelProcesses - 1 );
                        numberOfParallelProcesses--;

                        manualEvents.RemoveAt(indxCompletedThread);
                        loadStates.RemoveAt(indxCompletedThread);
                        threads.RemoveAt(indxCompletedThread);

                        if (manualEvents.Count == 0)
                            break; // all done
                        continue; // wait on other threads
                    }
                    else // Find next date if there are still dates to load and start a new thread
                    {
                        startDateToProcess = datesToProcess[nextDateToScheduleIndex];
                        datesToProcessInThread = new List<DateTime>(datesToProcess.GetRange(nextDateToScheduleIndex, 1));
                        nextDateToScheduleIndex++;
                    }
                                                                                    
                    endDateToProcess = startDateToProcess;

                    manualEvents[indxCompletedThread] = new ManualResetEvent(false);
                    Thread doLoad = new Thread(Etl.DoFactsLoad);
                    var loadState = new LoadState(processToRun, this.providerName, dwDBconnectionString, this.ssasServerName, this.ssasDBName, null, datesToProcessInThread, uniqueSessionId, indxCompletedThread, manualEvents[indxCompletedThread], log);

                    loadStates[indxCompletedThread] = loadState;
                    threads[indxCompletedThread] = doLoad;
                    doLoad.Start(loadState);
                    Thread.Sleep(3000); // give connection some time to start
                }

            }
            
            return mailMessage;
        }

        private static void DoFactsLoad(Object data)
        {
            var loadState = (LoadState)data;
            string mailMessage = "";

            try
            {
                var dbFact = new DataMartDatabase(loadState.DBCconnectionString, loadState.ProviderName, loadState.Log);
                if (loadState.ProcessToRun == PROCESS_ETL_FACTS)
                    mailMessage = dbFact.ETLFacts(loadState.DatesToProcessInThread[0], loadState.DatesToProcessInThread.Last(), loadState.UniqueSessionId);
                else if (loadState.ProcessToRun == PROCESS_PREPARE_SHARED_DATA_FOR_EFFECTIVE_DATE_ETL)
                    mailMessage = dbFact.PrepareSharedDataForEffectiveDateETL(loadState.DatesToProcessInThread[0], loadState.DatesToProcessInThread.Last(), loadState.UniqueSessionId);
                else if (loadState.ProcessToRun == PROCESS_ETL_ADJUSTMENTS)
                    mailMessage = dbFact.ETLAdjustments(loadState.SsasServerName, loadState.SsasDBName, loadState.DatesToProcessInThread[0], loadState.DatesToProcessInThread.Last(), loadState.UniqueSessionId);
                else if (loadState.ProcessToRun == PROCESS_ETL_CALCULATIONS)
                    mailMessage = dbFact.ETLCalculations(loadState.SsasServerName, loadState.SsasDBName, loadState.DatesToProcessInThread[0], loadState.DatesToProcessInThread.Last(), loadState.UniqueSessionId);
            }
            catch (Exception e)
            {
                loadState.ErrorMessage = String.Format("Error loading facts.\n" + mailMessage);
                loadState.Log.Error(loadState.ErrorMessage, e);
            }
            finally
            {
                loadState.ManualEvent.Set();
            }

        }

        internal class LoadState
        {
            public LoadState(int processToRun, string providerName, string dbCconnectionString, string ssasServerName, string ssasDBName, string changedTablesFlags, List<DateTime> datesToProcessInThread, string uniqueSessionId, int threadSeqNum, ManualResetEvent manualEvent, ILog log)
            {
                this.ErrorMessage = "";

                this.ProcessToRun = processToRun; 
                this.ProviderName = providerName;
                this.DBCconnectionString = dbCconnectionString;
                this.SsasServerName = ssasServerName;
                this.SsasDBName = ssasDBName;
                this.ChangedTablesFlags = changedTablesFlags; 
                this.DatesToProcessInThread = datesToProcessInThread;
                this.UniqueSessionId = uniqueSessionId;
                this.ThreadSeqNum = threadSeqNum;

                this.ManualEvent = manualEvent;

                this.Log = log;
            }

            public ILog Log
            {
                get;
                set;
            }

            public ManualResetEvent ManualEvent
            {
                get;
                set;
            }

            public string ChangedTablesFlags
            {
                get;
                set;
            }

            public List<DateTime> DatesToProcessInThread
            {
                get;
                set;
            }

            public string SsasServerName
            {
                get;
                set;
            }

            public string SsasDBName
            {
                get;
                set;
            }

            public int ProcessToRun
            {
                get;
                set;
            }

            public string ProviderName
            {
                get;
                set;
            }

            public string DBCconnectionString
            {
                get;
                set;
            }

            public string ErrorMessage
            {
                get;
                set;
            }

            public string UniqueSessionId
            {
                get;
                set;
            }

            public int ThreadSeqNum
            {
                get;
                set;
            }
        }

        private string ParallelETLDimensions(string dimensionAuthorityDBconnectionString, string changedTablesFlags, List<DateTime> datesToProcess, int numberOfParallelProcesses, int timeoutMinutesOnParallelProcesses, string uniqueSessionId)
        {
            numberOfParallelProcesses = Math.Min(MAX_THREADS_FOR_DIMENSIONS_LOAD, numberOfParallelProcesses);

               // This formula is different from facts as we need one thread for main ETL process
            int parallelLoadDays = datesToProcess.Count / ( numberOfParallelProcesses - 1 ) + 1;

            numberOfParallelProcesses = datesToProcess.Count / parallelLoadDays + (datesToProcess.Count % parallelLoadDays == 0 ? 0 : 1) + 1;

            string mailMessage = "";

            var manualEvents = new ManualResetEvent[numberOfParallelProcesses];
            var loadStates = new List<LoadState>();
            var threads = new List<Thread>();

            this.log.InfoFormat("Running the ETL Dimensions on {0} threads.", numberOfParallelProcesses);

            // Number of threads to preppare source data will be one less than total threads as we need the first one to run main ETL process

            for (int i = 0; i < numberOfParallelProcesses; i++)
            {
                manualEvents[i] = new ManualResetEvent(false);
                Thread doLoad = null;

                List<DateTime> datesToProcessInThread = null;
                // The first thread will be used for main ETL process
                if (i == 0)
                {
                    datesToProcessInThread = new List<DateTime>( datesToProcess );
                }
                else
                {
                    datesToProcessInThread = new List<DateTime>();
                        // Dates have to be distributed accroos remaining threads as we need to prepare dimensions dates in calendar order 
                    int increment = (datesToProcess.Count % parallelLoadDays == 0 ? datesToProcess.Count / parallelLoadDays : datesToProcess.Count / parallelLoadDays + 1);
                    for (int j = i - 1; j < datesToProcess.Count; j += increment)
                        datesToProcessInThread.Add( datesToProcess[j] );
                }

                var loadState = new LoadState(PROCESS_ETL_DIMENSIONS, this.providerName, dimensionAuthorityDBconnectionString, null, null, changedTablesFlags, datesToProcessInThread, uniqueSessionId, i, manualEvents[i], log);
                loadStates.Add(loadState);
                doLoad = new Thread(Etl.DoDimensionsLoad);
                threads.Add(doLoad);
                doLoad.Start(loadState);
                Thread.Sleep(5000); // give connection some time to start
            }

            bool success = WaitHandle.WaitAll(manualEvents, new TimeSpan(0, timeoutMinutesOnParallelProcesses, 0));

            if (!success)
            {
                this.log.Warn("The ETL Dimensions timed out. Terminating threads.");
                // Abort every thread as connection willnot be terminated if just exist the application
                foreach (var thread in threads)
                {
                    try
                    {
                        thread.Abort();
                    }
                    catch
                    {
                    }
                }

                throw new ApplicationException("Try reducing the number of parallel processes");
            }

            foreach (LoadState loadState in loadStates)
            {
                if (loadState.ErrorMessage != "")
                    throw new ApplicationException(loadState.ErrorMessage);
            }

            return mailMessage;
        }

        private static void DoDimensionsLoad(Object data)
        {
            var loadState = (LoadState)data;
            string mailMessage = "";

            try
            {
                var dbDim = new DimensionAuthorityDatabase(loadState.DBCconnectionString, loadState.ProviderName, loadState.Log);
                if (loadState.ThreadSeqNum == 0) // first thread
                {
                    string datesList = String.Join(",", loadState.DatesToProcessInThread.Select(dateToProcess => dateToProcess.ToString("yyyyMMdd")).ToArray());
                    mailMessage = dbDim.ETLDimensions(loadState.ChangedTablesFlags, datesList, false, loadState.UniqueSessionId);
                }
                else
                    foreach (var dateToProcess in loadState.DatesToProcessInThread)
                        mailMessage = dbDim.PrepareDimensionsEffectiveDateSourceDataForParallelLoad(loadState.ChangedTablesFlags, dateToProcess, loadState.UniqueSessionId);
            }
            catch (Exception e)
            {
                loadState.ErrorMessage = String.Format("Error loading facts.\n" + mailMessage);
                loadState.Log.Error(loadState.ErrorMessage, e);
            }
            finally
            {
                loadState.ManualEvent.Set();
            }

        }

    }
}
