﻿/*
--	Copyright (c) 2007 - 2021 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
using System;
using System.Diagnostics;
using DWJobs.Common;
using DWJobs.Common.AppEnvironment;

namespace DWJobs.ETLRunner
{
    class Program
    {
        const string EMAIL_FROM = "ETL@DBTimes.com";
        const string JOB_LOAD_DIMENSIONS = "DIMS";
        const string JOB_LOAD_FACTS = "FACTS";
        const string JOB_LOAD_SSAS = "SSAS";
        const string JOB_LOAD_ADJ = "ADJ";
        const string JOB_LOAD_SSAS_AND_ADJ = "SSASADJ";
        const string JOB_LOAD_ADJ_AND_SSAS = "ADJSSAS";
        const string JOB_UPDATE_STATISTICS_IN_DW = "UPDATESTATISTICSINDW";
        const string JOB_CLEAN_UP_ALL_SESSIONS_IN_DW = "CLEANUPALLSESSIONSINDW";
        const string JOB_CLEAN_UP_ALL_SESSIONS_IN_DIM_AUTH = "CLEANUPALLSESSIONSINDIMAUTH";

        static int Main(string[] args)
        {


            string mailSubject = "ETL";
            string mailBody = "";
            int exitCode = 0;
            DateTime etlStart = DateTime.Now;

            try
            {
                AppConfiguration.SetOverrides(args);
                AppConfiguration.Log.Info("***  START   ******************************************");
                AppConfiguration.Log.Info("Starting " + AppConfiguration.Description );


                AppConfiguration.Log.InfoFormat("{0}", AppConfiguration.ToString());
                mailBody = AppConfiguration.ToString();

                string job = AppConfiguration.Job.ToString().ToUpper();

                if (!String.IsNullOrEmpty(AppConfiguration.Environment) && !String.IsNullOrEmpty(job) )
                    mailSubject += " - job " + AppConfiguration.Environment + ", " + AppConfiguration.Job.ToString();

                if (job.Contains(JOB_UPDATE_STATISTICS_IN_DW))
                    Program.RunUpdateStatisticsInDW();
                else if (job.Contains(JOB_CLEAN_UP_ALL_SESSIONS_IN_DW))
                    Program.RunCleanUpAllSessionsInDW();
                else if (job.Contains(JOB_CLEAN_UP_ALL_SESSIONS_IN_DIM_AUTH))
                    Program.RunCleanUpAllSessionsInDimAuth();
                else
                    Program.RunMainETL(job);

                AppConfiguration.Log.Info("Completed ...");
                mailBody += "\n\nProcess completed successfully";

                exitCode = 0;
            }
            catch (ApplicationException ae)
            {
                AppConfiguration.Log.Error("Terminated ...", ae);
                mailSubject = "ERROR - " + mailSubject;
                mailBody += "\n\nProcess terminated. See log for details";
                exitCode = -1;
            }
            catch (Exception e)
            {
                AppConfiguration.Log.Fatal("Terminated (not handled exception )...", e);
                mailSubject = "ERROR - " + mailSubject;
                mailBody += "\n\nProcess terminated. See log for details";
                exitCode = -1;
            }
            finally
            {
                //    Console.ReadKey(true);
                TimeSpan timeDiff = DateTime.Now.Subtract(etlStart);
                string totalTimeFormatted = String.Format("Total processing time - {0} hrs {1} min {2} sec\n", timeDiff.Hours + timeDiff.Days * 24, timeDiff.Minutes, timeDiff.Seconds);
                try
                {
                    if (!String.IsNullOrEmpty(AppConfiguration.MailToList))
                    {
                        mailBody += "\n\nLog file name: \"file://" + AppConfiguration.LogFileName + "\"";
                        mailBody += "\n\n" + totalTimeFormatted;

                        //Mailer.SendMailMessage(EMAIL_FROM, AppConfiguration.MailToList, mailSubject, mailBody, null, AppConfiguration.Log);
                    }
                }
                catch
                {
                }

                AppConfiguration.Log.Info(totalTimeFormatted);
                AppConfiguration.Log.Info("***  END   ********************************************");

                Environment.Exit(exitCode);
                Process.GetCurrentProcess().Kill();
            }

            Environment.Exit(exitCode);
            return 0;
        }

        static void RunUpdateStatisticsInDW()
        {
            Etl etl = null;
            etl = new Etl(AppConfiguration.DBProviderName, AppConfiguration.SSASServerName, AppConfiguration.SSASDBName, AppConfiguration.Log);
            etl.UpdateStatisticsInDW( AppConfiguration.DWDBConnectionString );
        }

        static void RunCleanUpAllSessionsInDW()
        {
            Etl etl = null;
            etl = new Etl(AppConfiguration.DBProviderName, AppConfiguration.SSASServerName, AppConfiguration.SSASDBName, AppConfiguration.Log);
            etl.CleanupEtlSessionId(null, AppConfiguration.DWDBConnectionString);
        }

        static void RunCleanUpAllSessionsInDimAuth()
        {
            Etl etl = null;
            etl = new Etl(AppConfiguration.DBProviderName, AppConfiguration.SSASServerName, AppConfiguration.SSASDBName, AppConfiguration.Log);
            etl.CleanUpAllSessionsInDimAuth( AppConfiguration.DimensionAuthorityDBConnectionString );
        }

        static void RunMainETL(string job)
        {

            // determine start date to load
            DateTime? startDateToLoad = AppConfiguration.StartDateToLoad;

            // if go back days is spefified use it
            if (startDateToLoad == null && AppConfiguration.GoBackDaysForStartDateToLoad > 0)
                startDateToLoad = DateTime.Today.AddDays(-AppConfiguration.GoBackDaysForStartDateToLoad);

            AppConfiguration.Log.InfoFormat("Start day to load is {0}", startDateToLoad == null ? "not specified - will use default" : startDateToLoad.Value.ToShortDateString());


            // determine SSAS start date to load
            DateTime? ssasStartDateToLoad = null;

            // if go back days is spefified use it
            if (AppConfiguration.SSASGoBackDaysForStartDateToLoad > 0)
                ssasStartDateToLoad = DateTime.Today.AddDays(-AppConfiguration.SSASGoBackDaysForStartDateToLoad);
            else
                ssasStartDateToLoad = AppConfiguration.SSASStartDateToLoad;

            AppConfiguration.Log.InfoFormat("SSAS start day to load is {0}", ssasStartDateToLoad == null ? "not specified - will use default" : ssasStartDateToLoad.Value.ToShortDateString());
            
            // Start Etl 
            Etl etl = null;

            // this id does not have to be the same for dimensions and ffacts, but just easier to keep it like this
            // The id HAS to be the same for all steps in facts Database load including cubes load
            string uniqueSessionId = Guid.NewGuid().ToString();

            // Load dimensions
            if (job.Contains(JOB_LOAD_DIMENSIONS))
            {
                etl = new Etl(AppConfiguration.DBProviderName, AppConfiguration.SSASServerName, AppConfiguration.SSASDBName, AppConfiguration.Log);
                etl.LoadDimensions(uniqueSessionId, AppConfiguration.DimensionAuthorityDBConnectionString, AppConfiguration.IsInitialLoad, startDateToLoad, AppConfiguration.EndDateToLoad, AppConfiguration.NumberOfParallelProcesses, AppConfiguration.TimeoutMinutesOnParallelProcesses);
            }

            // Load facts
            if (job.Contains(JOB_LOAD_FACTS) || job.Contains(JOB_LOAD_ADJ) || job.Contains(JOB_LOAD_SSAS) )
            {
                try
                {
                    etl = new Etl(AppConfiguration.DBProviderName, AppConfiguration.SSASServerName, AppConfiguration.SSASDBName, AppConfiguration.Log);
                    etl.PrepareEtlSessionId(uniqueSessionId, AppConfiguration.DWDBConnectionString);

                    if (job.Contains(JOB_LOAD_FACTS))
                    {
                        etl = new Etl(AppConfiguration.DBProviderName, AppConfiguration.SSASServerName, AppConfiguration.SSASDBName, AppConfiguration.Log);
                        etl.LoadFacts(uniqueSessionId, AppConfiguration.DWDBConnectionString, AppConfiguration.IsInitialLoad, AppConfiguration.DimAuthorityServerName, AppConfiguration.DimAuthorityDBName, startDateToLoad, AppConfiguration.EndDateToLoad, AppConfiguration.NumberOfParallelProcesses, AppConfiguration.TimeoutMinutesOnParallelProcesses);
                    }

                    // Load data into SSAS the first or only time
                    if ( job.Contains(JOB_LOAD_SSAS) && !( job.Contains(JOB_LOAD_ADJ_AND_SSAS) && !job.Contains(JOB_LOAD_SSAS_AND_ADJ) ) )
                    {
                        var analysisServicesProcessing = new AnalysisServicesProcessing(AppConfiguration.SSASSourceDBConnectionString, AppConfiguration.DBProviderName, AppConfiguration.SSASServerName, AppConfiguration.SSASDBName, AppConfiguration.SSASAccessConfigurationName, AppConfiguration.SSASMaxParallel, AppConfiguration.Debug, AppConfiguration.Log);
                        analysisServicesProcessing.LoadAnalysisServices(uniqueSessionId, AppConfiguration.IsSSASFullLoad, ssasStartDateToLoad, AppConfiguration.IsSSASLoadCalculations );
                    }

                    // Load Adjustments and calculations
                    if (job.Contains(JOB_LOAD_ADJ))
                    {
                        etl = new Etl(AppConfiguration.DBProviderName, AppConfiguration.SSASServerName, AppConfiguration.SSASDBName, AppConfiguration.Log);
                        etl.LoadAdjustmentsAndCalculations(uniqueSessionId, AppConfiguration.DWDBConnectionString, AppConfiguration.IsInitialLoad, AppConfiguration.DimAuthorityServerName, AppConfiguration.DimAuthorityDBName, startDateToLoad, AppConfiguration.EndDateToLoad, AppConfiguration.NumberOfParallelProcesses, AppConfiguration.TimeoutMinutesOnParallelProcesses);
                    }

                    // Load SSAS second time after Adjustments
                    if (job.Contains(JOB_LOAD_ADJ_AND_SSAS))
                    {
                        var analysisServicesProcessing = new AnalysisServicesProcessing(AppConfiguration.SSASSourceDBConnectionString, AppConfiguration.DBProviderName, AppConfiguration.SSASServerName, AppConfiguration.SSASDBName, AppConfiguration.SSASAccessConfigurationName, AppConfiguration.SSASMaxParallel, AppConfiguration.Debug, AppConfiguration.Log);
                        analysisServicesProcessing.LoadAnalysisServices(uniqueSessionId, AppConfiguration.IsSSASFullLoad, ssasStartDateToLoad, AppConfiguration.IsSSASLoadCalculations );
                    }
                }
                finally
                {
                    etl = new Etl(AppConfiguration.DBProviderName, AppConfiguration.SSASServerName, AppConfiguration.SSASDBName, AppConfiguration.Log);
                    etl.CleanupEtlSessionId(uniqueSessionId, AppConfiguration.DWDBConnectionString);
                }
            } // Load facts
        }
    }
}
