/*
--	Copyright (c) 2007 - 2021 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;
using System.Data.SqlTypes;
using System.Globalization;
using System.Text.RegularExpressions;
using Microsoft.SqlServer.Server;
using Microsoft.AnalysisServices.AdomdClient;

namespace DWJobs.ClrObjects
{
    public partial class Reporting
    {
        static int PRINT_STRING_LENGHT_MAX = 4000;




        static string[] DATE_FORMATS = { "yyyy-MM-dd", "yyyyMMdd", "MM/dd/yyyy" };

        [Microsoft.SqlServer.Server.SqlProcedure]
        public static void SelectViaMdx([SqlFacet(MaxSize = -1)]SqlString analysisSevicesConnectionString, [SqlFacet(MaxSize = -1)]SqlString mdxQuery, [SqlFacet(MaxSize = -1)]SqlString outputTable)
        {
            /****
                Output table string should look like this:
               "[Load Date]         DATE,
                [Insrument Name]    VARCHAR (300 ),
                [Volume]            FLOAT,
               "
             1. Each actualColumn must be palced inside brackets
             2. Only Three types of data allowed: DATE, VARCHAR and FLOAT
             3. The MDX must return the same number of columns as specified in the result definition
            ****/

            try
            {
                var sqlOutputColumns = new List<SqlMetaData>();
                var outputColumns = outputTable.ToString().Split(',');

                // Parse each column
                var regex = new Regex(@"^\s*\[(?<columnname>.*)\]\s*(?<columntype>VARCHAR|DATE|FLOAT|varchar|date|float)\s*\(??\s*(?<columnsize>[0-9]*|MAX|max)??\s*\)??\s*$");
                var columns = new SqlMetaData[outputColumns.GetLength(0)];

                int columnIndx = 0;
                string outputColumnsParsed = "";

                foreach (var outputColumn in outputColumns)
                {
                    Match match = regex.Match(outputColumn);

                    if (match.Success)
                    {
                        string outputColumnParsed = "[" + match.Groups["columnname"].Value + "]\t\t" + match.Groups["columntype"].Value.ToUpper();

                        if (match.Groups["columntype"].Value.ToUpper() == "FLOAT")
                            columns[columnIndx++] = new SqlMetaData(match.Groups["columnname"].Value, SqlDbType.Float);
                        else if (match.Groups["columntype"].Value.ToUpper() == "DATE")
                            columns[columnIndx++] = new SqlMetaData(match.Groups["columnname"].Value, SqlDbType.Date);
                        else if (match.Groups["columntype"].Value.ToUpper() == "VARCHAR")
                        {
                            if (match.Groups["columnsize"].Value.ToUpper() == "MAX")
                            {
                                columns[columnIndx++] = new SqlMetaData(match.Groups["columnname"].Value, SqlDbType.VarChar, SqlMetaData.Max);
                                outputColumnParsed += "( MAX )";
                            }
                            else
                            {
                                int lenght = (!String.IsNullOrEmpty(match.Groups["columnsize"].Value) ? Int32.Parse(match.Groups["columnsize"].Value) : 500);
                                if (lenght == 0)
                                    lenght = 500;
                                columns[columnIndx++] = new SqlMetaData(match.Groups["columnname"].Value, SqlDbType.VarChar, lenght);
                                outputColumnParsed += "( " + lenght.ToString() + " )";
                            }
                        }
                        else
                            continue;

                        outputColumnParsed += "\n";
                        outputColumnsParsed += outputColumnParsed;
                    }
                }

                var actualColumns = new SqlMetaData[columnIndx];
                for (int actualColumnIndx = 0; actualColumnIndx < columnIndx; actualColumnIndx++)
                    actualColumns[actualColumnIndx] = columns[actualColumnIndx];

                using (var conn = new AdomdConnection(analysisSevicesConnectionString.ToString()))
                {
                    conn.Open();
                    using (SqlConnection sqlConnection = new SqlConnection("context connection=true"))
                    {
                        sqlConnection.Open();

                        // Print MDX and table columns
                        string message = "SSAS connection string:\n" + analysisSevicesConnectionString.ToString();

                        message += "\n\nExecuting MDX query:\n" + mdxQuery.ToString();
                        message += "\n\nThe result will be selected into the table with following " + actualColumns.Length.ToString() + " columns:\n" + outputColumnsParsed;
                        for (int messageIndx = 0; messageIndx < message.Length; messageIndx += PRINT_STRING_LENGHT_MAX)
                            SqlContext.Pipe.Send(message.Substring(messageIndx, Math.Min(message.Length - messageIndx, PRINT_STRING_LENGHT_MAX)));

                        AdomdCommand adomdCommand = conn.CreateCommand();
                        adomdCommand.CommandText = mdxQuery.ToString();

                        DateTime queryStart = DateTime.Now;
                        AdomdDataReader adomdReader = adomdCommand.ExecuteReader();

                        int countRows = 0;
                        string errorMsg = "";
                        SqlDataRecord record = new SqlDataRecord(actualColumns);

                        SqlContext.Pipe.SendResultsStart(record);
                        while (adomdReader.Read())
                        {
                            // Send result row
                            if (adomdReader.FieldCount != actualColumns.Length)
                            {
                                errorMsg = String.Format("ERROR: The number of columns in MDX query - {0} does not match the number of columns - {1} - defined for the result table", adomdReader.FieldCount, actualColumns.Length);
                                throw new ApplicationException(errorMsg);
                            }

                            // EffDate key is in format: YYYYMMDD

                            int columnOrdinal = 0;
                            foreach (var actualColumn in actualColumns)
                            {
                                if (actualColumn.SqlDbType == SqlDbType.Date)
                                {
                                    DateTime date;
                                    if (DateTime.TryParseExact(adomdReader.GetString(columnOrdinal), DATE_FORMATS, CultureInfo.InvariantCulture, DateTimeStyles.None, out date))
                                        record.SetDateTime(columnOrdinal, date);
                                    else
                                    {
                                        errorMsg = String.Format("ERROR: The value '{0}' for [{1}] actualColumn in row {2} failed to convert to DATE type. May need to change [{1}] type to VARCHAR", adomdReader.GetString(columnOrdinal), actualColumn.Name, countRows + 1);

                                        throw new ApplicationException(errorMsg);
                                    }
                                }
                                else if (actualColumn.SqlDbType == SqlDbType.VarChar)
                                    record.SetString(columnOrdinal, adomdReader.GetString(columnOrdinal));
                                else if (actualColumn.SqlDbType == SqlDbType.Float)
                                {
                                    if (adomdReader.IsDBNull(columnOrdinal) || adomdReader.GetString(columnOrdinal) == "NaN")
                                        record.SetDBNull(columnOrdinal);
                                    else
                                    {
                                        double value = Double.Parse(adomdReader.GetString(columnOrdinal));
                                        record.SetDouble(columnOrdinal, value);
                                    }
                                }
                                columnOrdinal++;
                            }
                            SqlContext.Pipe.SendResultsRow(record);
                            countRows++;

                        }

                        // Mark the end of the result-set.

                        SqlContext.Pipe.SendResultsEnd();
                        TimeSpan timeDiff = DateTime.Now.Subtract(queryStart);

                        message = String.Format("Selected {0} rows in {1} sec.", countRows, timeDiff.TotalSeconds);
                        SqlContext.Pipe.Send(message);
                        adomdReader.Close();
                    }
                }
            }

            catch (Exception)
            {
                string errorText = String.Format("Error in DoQueryAndSaveToFile");
                throw;
            }
            finally
            {
            }
        }
    }
}
