﻿/*
--	Copyright (c) 2007 - 2021 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
using System;
using System.Data;
using System.Data.SqlClient;

using DWJobs.Common;
using log4net;

namespace DWJobs.TabularSetsComparer
{
    public class CompareDatabase
    {
        private readonly string connectionString;
        private string providerName;
        private string debug;
        private readonly ILog log;
        private string printText;
        
        private const int RESULT_SUCCESS = 0;

        public CompareDatabase(string connectionString, string providerName, ILog log, string debug)
        {
            this.connectionString = connectionString;
            this.providerName = providerName;
            this.log = log;
            this.debug = debug;
        }

        public int CompareTablularResults(  string outputCsvFileFullName        , 
                                            string ssasConnectionStringLeft     , 
                                            string ssasConnectionStringRight    , 
                                            string aliasLeft                    , 
                                            string aliasRight                   ,
                                            string queryLeft                 , 
                                            string queryRight                ,
                                            string resultSetColumnsAndTypes     ,
                                            string uniqueKeyColumns             ,
                                            string columnsToExcludeFromCompare    ,
                                            double floatCompareThreshold            )
        {
            int returnValue = -1;

            this.log.InfoFormat("Executing compare with results to file {0}", outputCsvFileFullName );
            string queryString = "[gen].[Compare Tablular Results]";

            using (SqlConnection connection = new SqlConnection(this.connectionString))
            {
                connection.InfoMessage += new SqlInfoMessageEventHandler( InfoMessageHandler) ;

                SqlCommand command = new SqlCommand(queryString, connection);
                command.CommandType = System.Data.CommandType.StoredProcedure;
                command.CommandTimeout = 0;
                
                SqlParameter param = new SqlParameter("@Retval", SqlDbType.Int);
                param.Direction = System.Data. ParameterDirection.ReturnValue;
                command.Parameters.Add( param) ;

                param = new SqlParameter("@SSASConnectionStringLeft", SqlDbType.Char);
                if (String.IsNullOrEmpty(ssasConnectionStringLeft))
                    param.Value = DBNull.Value;
                else
                    param.Value = ssasConnectionStringLeft;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@SSASConnectionStringRight", SqlDbType.Char);
                if (String.IsNullOrEmpty(ssasConnectionStringRight))
                    param.Value = DBNull.Value;
                else
                    param.Value = ssasConnectionStringRight;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add(param);

                param = new SqlParameter("@AliasLeft", SqlDbType.Char);
                if (String.IsNullOrEmpty(aliasLeft) )
                    param.Value = DBNull.Value;
                else
                    param.Value = aliasLeft;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters. Add ( param) ;

                param = new SqlParameter( "@AliasRight", SqlDbType.Char) ;
                if (String.IsNullOrEmpty(aliasRight) )
                    param.Value = DBNull.Value;
                else
                    param.Value = aliasRight;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters. Add( param );

                param = new SqlParameter("@QueryLeft", SqlDbType.Char);
                param.Value = queryLeft;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add( param) ;

                param = new SqlParameter("@QueryRight", SqlDbType.Char);
                param.Value = queryRight;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add( param) ;

                param = new SqlParameter("@TablularResultSetColumnsTypes", SqlDbType.Char);
                param.Value = resultSetColumnsAndTypes;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add( param) ;

                param = new SqlParameter("@UniqueKeyColumns", SqlDbType.Char);
                if (String.IsNullOrEmpty(uniqueKeyColumns) )
                    param.Value = DBNull.Value;
                else
                    param.Value = uniqueKeyColumns;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add( param) ;

                param = new SqlParameter("@ColumnsToExcludeFromCompare", SqlDbType.Char);
                if (String.IsNullOrEmpty(columnsToExcludeFromCompare) )
                    param.Value = DBNull.Value;
                else
                    param.Value = columnsToExcludeFromCompare;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add( param) ;

                param = new SqlParameter("@FloatCompareThreshold", SqlDbType.Float);
                param.Value = floatCompareThreshold;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add( param) ;

                param = new SqlParameter("@Debug", SqlDbType.Char);
                param.Value = this.debug;
                param.Direction = System.Data.ParameterDirection.Input;
                command.Parameters.Add ( param) ;
                
                try
                {
                    connection.Open();
                    this.printText = "";

                    DatabaseHelper.DumpSelectResultToCSVFile(command, outputCsvFileFullName );
                    
                    returnValue = Int32.Parse( command.Parameters[ "@Retval" ].Value.ToString( ) );
                    
                    this.log.InfoFormat("Finished running compare with results to CSV file {0}", outputCsvFileFullName ); 
                    this.log.InfoFormat ( "" ) ;
                    return returnValue;
                }
                catch (SqlException e)
                {
                    this.log.ErrorFormat("SQL Exception Message: {0} Number: {1} Procedure: {2} Line Number {3} Text from PRINT: {4} ", e.Message, e.Number, e.Procedure ?? "none", e.LineNumber, this.printText );
                    
                    // The first one is
                    // Transport level error has occurred when receiving result from the server ( Provider: TCP Provider, error 0 - The specified network name is no longer available) 
                    // The second A system assertion check has faired. check the SQL server error log for details
                    if (e.Number == 64 || e.Number == 3624 ) 
                        throw new ApplicationException();
                    else
                        throw;
                }
                finally
                {
                    try
                    {
                        connection.Close();
                    }
                    catch
                    {
                    }
                }
            }
        }

        private void InfoMessageHandler(object mySender, SqlInfoMessageEventArgs myEvent)
        {
            for (int i = 0; i < myEvent.Errors.Count; i++)
            {
                if (this.log != null)
                    this.log.Info(myEvent.Errors[i].ToString().Replace("System.Data.SqlClient.SqlError:", ""));
                else
                    this.printText += myEvent.Errors[i].ToString().Replace("System.Data.SqlClient.SqlError:", "") + "\r\n";
            }

        }
    }
}
