﻿/*
--	Copyright (c) 2007 - 2021 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--	
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--	
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--	
--	3. Altered versions must be plainly marked as such, and must not
--		be misrepresented (by explicit claim or omission) as being
--		the original software.
--
--	4. This notice must not be removed or altered.
*/
using System;
using System.Linq;
using System.Diagnostics;

using DWJobs.Common;
using DWJobs.Common.AppEnvironment;
using DWJobs.TabularSetsComparer.InputConfigXml;

namespace DWJobs.TabularSetsComparer
{
    class Program
    {
        static int Main(string[] args)
        {
            const string EMAIL_FROM= "TabularSetsComparer@dbtimes.com";
            string mailSubject = "Data Compare";
            string mailBody = "";
            int exitCode = 0;
            
            DateTime processStart = DateTime.Now;
            try
            {
                AppConfiguration.SetOverrides ( args ) ;
                AppConfiguration.Log.Info( "*** START ******************************\n");
                AppConfiguration.Log.Info( "Starting " + AppConfiguration.Description );
                AppConfiguration.Log.InfoFormat ( "{0}", AppConfiguration.ToString() ); 
                mailBody = AppConfiguration.ToString();
                
                string job = AppConfiguration.Job.ToString().ToUpper();

                if (!String.IsNullOrEmpty( AppConfiguration.Environment ) && !String.IsNullOrEmpty(job))
                    mailSubject += " - job " + AppConfiguration.Environment;

                var results = InputConfigXml.CompareScenarioImporter.LoadConfigurationDocument(AppConfiguration.InputConfigFile) ;

                CompareScenario[] result = results.ToArray();

                // ResultSetsCompare
                string allCompareResultCSVFileNames = "";
                bool anyDifferencesFound = false;

                foreach (CompareScenario compareScenario in result)
                {
                    ITabularSetsComparer resultSetscomparer = TabularSetsComparerFactory.Create (compareScenario, new CompareDatabase (AppConfiguration.DestDBConnectionString, AppConfiguration.DBProviderName, AppConfiguration.Log, AppConfiguration.Debug ) );
                    DateTime scenarioProcessStart = DateTime.Now;
                    string compareResultCSVFileNames;
                    bool differencesFound = resultSetscomparer.CompareResultSets(out compareResultCSVFileNames);

                    if ( ! anyDifferencesFound)
                        anyDifferencesFound = differencesFound;
                    allCompareResultCSVFileNames += compareResultCSVFileNames;
                    TimeSpan timeDiff = DateTime.Now.Subtract( scenarioProcessStart );
                    AppConfiguration.Log.InfoFormat ("Result sets comparison  completed in {0} min {1} sec ", Math.Floor( timeDiff.TotalMinutes ), timeDiff.Seconds );
                }

                if (anyDifferencesFound)
                    mailSubject = "DIFFERENCES FOUND" + mailSubject;
                mailBody = "\n\nCompare results:\n\n" + allCompareResultCSVFileNames;
                AppConfiguration.Log.Info(mailSubject);
                AppConfiguration.Log.Info(mailBody);
                AppConfiguration.Log.Info( "Completed ... ");
                mailBody += "\n\nProcess completed successfully";
                exitCode = 0;
            }
            catch (ApplicationException ae)
            {
                AppConfiguration.Log.Error("Terminated ...", ae);
                mailSubject = "ERROR - " + mailSubject;
                mailBody += "\n\nPnocess terminated. See log for details";
                exitCode = -1;
            }
            catch (Exception e)
            {
                AppConfiguration.Log.Fatal("Terminated (not handred exception )...", e);
                mailSubject = "ERROR - " + mailSubject;
                mailBody += "\n\nProcess terminated. See 1og for details";
                exitCode = -1;
            }
            finally
            {
                TimeSpan timeDiff = DateTime.Now.Subtract(processStart);
                string totalTimeFormatted = String.Format("Total processing time - {0} hrs {1} min {2} sec\n", Math.Floor(timeDiff.TotalHours), timeDiff.Minutes, timeDiff.Seconds);
                try
                {
                    if (!String.IsNullOrEmpty(AppConfiguration.MailToList))
                    {
                        mailBody += "\n\nLog File name: \"file://" + AppConfiguration.LogFileName + "\"";
                        mailBody += "\n\n" + totalTimeFormatted;
                        Mailer.SendMailMessage(EMAIL_FROM, AppConfiguration.MailToList, mailSubject, mailBody, null, AppConfiguration.Log);
                    }
                }
                catch
                {
                }
                AppConfiguration.Log.Info(totalTimeFormatted);
                AppConfiguration.Log.Info("*** END ***********************************");
                Environment.Exit(exitCode);
                Process.GetCurrentProcess().Kill();
            }
            Environment.Exit(exitCode);
            return 0;
        }
    }
}
